"""
Action enums.
"""

import typing as t

import typing_extensions as te

from composio.client.enums.base import (
    ACTIONS_CACHE,
    ActionData,
    _AnnotatedEnum,
    add_runtime_action,
    enum,
)


class SentinalObject:
    """Sentinal object."""

    sentinal = None


@enum
class Action(_AnnotatedEnum[ActionData], path=ACTIONS_CACHE):
    """Class to represent `Action` entity."""

    APIFY_CREATE_APIFY_ACTOR: "Action"
    APIFY_GET_ACTOR_ID: "Action"
    APIFY_GET_LAST_RUN_DATA: "Action"
    APIFY_LIST_APIFY_ACTORS: "Action"
    APIFY_LIST_APIFY_TASKS: "Action"
    APIFY_SEARCH_STORE: "Action"
    ASANA_ALLOCATIONS_CREATE_RECORD: "Action"
    ASANA_ALLOCATIONS_DELETE_ALLOCATION_BY_ID: "Action"
    ASANA_ALLOCATIONS_GET_MULTIPLE: "Action"
    ASANA_ALLOCATIONS_GET_RECORD_BY_ID: "Action"
    ASANA_ALLOCATIONS_UPDATE_RECORD_BY_ID: "Action"
    ASANA_ATTACHMENTS_DELETE_SPECIFIC: "Action"
    ASANA_ATTACHMENTS_GET_ALL_FOR_OBJECT: "Action"
    ASANA_ATTACHMENTS_GET_ATTACHMENT_RECORD: "Action"
    ASANA_ATTACHMENTS_UPLOAD_ATTACHMENT: "Action"
    ASANA_AUDIT_LOG_API_GET_AUDIT_LOG_EVENTS: "Action"
    ASANA_BATCH_API_SUBMIT_PARALLEL_REQUESTS: "Action"
    ASANA_CUSTOM_FIELDS_ADDE_NUM_OPTION: "Action"
    ASANA_CUSTOM_FIELDS_CREATE_NEW_FIELD_RECORD: "Action"
    ASANA_CUSTOM_FIELDS_DELETE_FIELD_RECORD: "Action"
    ASANA_CUSTOM_FIELDS_GET_METADATA: "Action"
    ASANA_CUSTOM_FIELDS_LIST_WORK_SPACE_CUSTOM_FIELDS: "Action"
    ASANA_CUSTOM_FIELDS_REORDER_E_NUM_OPTION: "Action"
    ASANA_CUSTOM_FIELDS_UPDATE_E_NUM_OPTION: "Action"
    ASANA_CUSTOM_FIELDS_UPDATE_FIELD_RECORD: "Action"
    ASANA_CUSTOM_FIELD_SETTINGS_GET_PORTFOLIO_CUSTOM_FIELD_SETTINGS: "Action"
    ASANA_CUSTOM_FIELD_SETTINGS_GET_PROJECT_CUSTOM_FIELD_SETTINGS: "Action"
    ASANA_EVENTS_GET_RESOURCE_EVENTS: "Action"
    ASANA_GOALS_ADD_COLLABORATORS_TO_GOAL: "Action"
    ASANA_GOALS_CREATE_METRIC: "Action"
    ASANA_GOALS_CREATE_NEW_GOAL_RECORD: "Action"
    ASANA_GOALS_DELETE_RECORD: "Action"
    ASANA_GOALS_GET_COMPACT_RECORDS: "Action"
    ASANA_GOALS_GET_GOAL_RECORD: "Action"
    ASANA_GOALS_GET_PARENT_GOALS: "Action"
    ASANA_GOALS_REMOVE_FOLLOWERS_FROM_GOAL: "Action"
    ASANA_GOALS_UPDATE_GOAL_RECORD: "Action"
    ASANA_GOALS_UPDATE_METRIC_CURRENT_VALUE: "Action"
    ASANA_GOAL_RELATIONSHIPS_CREATE_SUPPORTING_RELATIONSHIP: "Action"
    ASANA_GOAL_RELATIONSHIPS_GET_COMPACT_RECORDS: "Action"
    ASANA_GOAL_RELATIONSHIPS_GET_RECORD_BY_ID: "Action"
    ASANA_GOAL_RELATIONSHIPS_REMOVE_SUPPORTING_RELATIONSHIP: "Action"
    ASANA_GOAL_RELATIONSHIPS_UPDATE_GOAL_RELATIONSHIP_RECORD: "Action"
    ASANA_JOBS_GET_BY_ID: "Action"
    ASANA_MEMBERSHIPS_CREATE_NEW_RECORD: "Action"
    ASANA_MEMBERSHIPS_DELETE_RECORD: "Action"
    ASANA_MEMBERSHIPS_GET_MEMBERSHIP_RECORD: "Action"
    ASANA_MEMBERSHIPS_GET_MULTIPLE: "Action"
    ASANA_MEMBERSHIPS_UPDATE_MEMBERSHIP_RECORD: "Action"
    ASANA_ORGANIZATION_EXPORTS_CREATE_EXPORT_REQUEST: "Action"
    ASANA_ORGANIZATION_EXPORTS_GET_EXPORT_DETAILS: "Action"
    ASANA_PORTFOLIOS_ADD_CUSTOM_FIELD_SETTING: "Action"
    ASANA_PORTFOLIOS_ADD_MEMBERS_TO_PORTFOLIO: "Action"
    ASANA_PORTFOLIOS_ADD_PORTFOLIO_ITEM: "Action"
    ASANA_PORTFOLIOS_CREATE_NEW_PORTFOLIO_RECORD: "Action"
    ASANA_PORTFOLIOS_DELETE_RECORD: "Action"
    ASANA_PORTFOLIOS_GET_ITEMS: "Action"
    ASANA_PORTFOLIOS_GET_RECORD: "Action"
    ASANA_PORTFOLIOS_LIST_MULTIPLE_PORTFOLIOS: "Action"
    ASANA_PORTFOLIOS_REMOVE_CUSTOM_FIELD_SETTING: "Action"
    ASANA_PORTFOLIOS_REMOVE_ITEM_FROM_PORTFOLIO: "Action"
    ASANA_PORTFOLIOS_REMOVE_MEMBERS_FROM_PORTFOLIO: "Action"
    ASANA_PORTFOLIOS_UPDATE_PORTFOLIO_RECORD: "Action"
    ASANA_PORTFOLIO_MEMBERSHIPS_GET_COMPACT: "Action"
    ASANA_PORTFOLIO_MEMBERSHIPS_GET_COMPLETE_RECORD: "Action"
    ASANA_PORTFOLIO_MEMBERSHIPS_LIST_MULTIPLE_MEMBERSHIPS: "Action"
    ASANA_PROJECTS_ADD_CUSTOM_FIELD_SETTING: "Action"
    ASANA_PROJECTS_ADD_FOLLOWERS_TO_PROJECT: "Action"
    ASANA_PROJECTS_ADD_MEMBERS_TO_PROJECT: "Action"
    ASANA_PROJECTS_CREATE_IN_WORK_SPACE: "Action"
    ASANA_PROJECTS_CREATE_NEW_PROJECT_RECORD: "Action"
    ASANA_PROJECTS_CREATE_PROJECT_FOR_TEAM: "Action"
    ASANA_PROJECTS_CREATE_PROJECT_TEMPLATE_JOB: "Action"
    ASANA_PROJECTS_DELETE_PROJECT_BY_ID: "Action"
    ASANA_PROJECTS_DUPLICATE_PROJECT_JOB: "Action"
    ASANA_PROJECTS_GET_ALL_IN_WORK_SPACE: "Action"
    ASANA_PROJECTS_GET_PROJECT_RECORD: "Action"
    ASANA_PROJECTS_GET_TASK_COUNTS: "Action"
    ASANA_PROJECTS_GET_TEAM_PROJECTS: "Action"
    ASANA_PROJECTS_LIST_MULTIPLE: "Action"
    ASANA_PROJECTS_REMOVE_CUSTOM_FIELD: "Action"
    ASANA_PROJECTS_REMOVE_MEMBERS_FROM_PROJECT: "Action"
    ASANA_PROJECTS_REMOVE_PROJECT_FOLLOWERS: "Action"
    ASANA_PROJECTS_TASK_PROJECTS_LIST: "Action"
    ASANA_PROJECTS_UPDATE_PROJECT_RECORD: "Action"
    ASANA_PROJECT_BRIEFS_CREATE_NEW_RECORD: "Action"
    ASANA_PROJECT_BRIEFS_GET_FULL_RECORD: "Action"
    ASANA_PROJECT_BRIEFS_REMOVE_BRIEF: "Action"
    ASANA_PROJECT_BRIEFS_UPDATE_BRIEF_RECORD: "Action"
    ASANA_PROJECT_MEMBERSHIPS_GET_COMPACT_RECORDS: "Action"
    ASANA_PROJECT_MEMBERSHIPS_GET_RECORD: "Action"
    ASANA_PROJECT_STATUSES_CREATE_NEW_STATUS_UPDATE_RECORD: "Action"
    ASANA_PROJECT_STATUSES_DELETE_SPECIFIC_STATUS_UPDATE: "Action"
    ASANA_PROJECT_STATUSES_GET_PROJECT_UPDATES: "Action"
    ASANA_PROJECT_STATUSES_GET_STATUS_UPDATE_RECORD: "Action"
    ASANA_PROJECT_TEMPLATES_DELETE_TEMPLATE_RECORD: "Action"
    ASANA_PROJECT_TEMPLATES_GET_ALL_TEMPLATE_RECORDS: "Action"
    ASANA_PROJECT_TEMPLATES_GET_RECORD: "Action"
    ASANA_PROJECT_TEMPLATES_INSTANTIATE_PROJECT_JOB: "Action"
    ASANA_PROJECT_TEMPLATES_LIST_MULTIPLE: "Action"
    ASANA_RULES_TRIGGER_RULE_REQUEST: "Action"
    ASANA_SECTIONS_ADD_TASK_TO_SECTION: "Action"
    ASANA_SECTIONS_CREATE_NEW_SECTION: "Action"
    ASANA_SECTIONS_DELETE_SECTION: "Action"
    ASANA_SECTIONS_GET_RECORD: "Action"
    ASANA_SECTIONS_LIST_PROJECT_SECTIONS: "Action"
    ASANA_SECTIONS_MOVE_OR_INSERT: "Action"
    ASANA_SECTIONS_UPDATE_SECTION_RECORD: "Action"
    ASANA_STATUS_UPDATES_CREATE_NEW_STATUS_UPDATE_RECORD: "Action"
    ASANA_STATUS_UPDATES_DELETE_SPECIFIC_STATUS_UPDATE: "Action"
    ASANA_STATUS_UPDATES_GET_COMPACT_RECORDS: "Action"
    ASANA_STATUS_UPDATES_GET_RECORD_BY_ID: "Action"
    ASANA_STORIES_CREATE_COMMENT: "Action"
    ASANA_STORIES_DELETE_STORY_RECORD: "Action"
    ASANA_STORIES_GET_FULL_RECORD: "Action"
    ASANA_STORIES_GET_TASK_STORIES: "Action"
    ASANA_STORIES_UPDATE_FULL_RECORD: "Action"
    ASANA_TAGS_CREATE_NEW_TAG_RECORD: "Action"
    ASANA_TAGS_CREATE_TAG_IN_WORK_SPACE: "Action"
    ASANA_TAGS_GET_FILTERED_TAGS: "Action"
    ASANA_TAGS_GET_RECORD: "Action"
    ASANA_TAGS_GET_TASK_TAGS: "Action"
    ASANA_TAGS_LIST_FILTERED_TAGS: "Action"
    ASANA_TAGS_REMOVE_TAG: "Action"
    ASANA_TAGS_UPDATE_TAG_RECORD: "Action"
    ASANA_TASKS_ADD_FOLLOWERS_TO_TASK: "Action"
    ASANA_TASKS_ADD_PROJECT_TO_TASK: "Action"
    ASANA_TASKS_ADD_TAG_TO_TASK: "Action"
    ASANA_TASKS_CREATE_NEW_TASK: "Action"
    ASANA_TASKS_CREATE_SUB_TASK_RECORD: "Action"
    ASANA_TASKS_DELETE_TASK: "Action"
    ASANA_TASKS_DUPLICATE_TASK_JOB: "Action"
    ASANA_TASKS_GET_ALL_DEPENDENCIES: "Action"
    ASANA_TASKS_GET_BY_CUSTOM_ID: "Action"
    ASANA_TASKS_GET_DEPENDENTS: "Action"
    ASANA_TASKS_GET_MULTIPLE: "Action"
    ASANA_TASKS_GET_MULTIPLE_WITH_TAG: "Action"
    ASANA_TASKS_GET_SECTION_TASKS: "Action"
    ASANA_TASKS_GET_SUB_TASK_LIST: "Action"
    ASANA_TASKS_GET_TASKS_BY_PROJECT: "Action"
    ASANA_TASKS_GET_TASK_RECORD: "Action"
    ASANA_TASKS_GET_USER_TASK_LIST_TASKS: "Action"
    ASANA_TASKS_REMOVE_FOLLOWERS_FROM_TASK: "Action"
    ASANA_TASKS_REMOVE_PROJECT_FROM_TASK: "Action"
    ASANA_TASKS_REMOVE_TAG_FROM_TASK: "Action"
    ASANA_TASKS_SEARCH_IN_WORK_SPACE: "Action"
    ASANA_TASKS_SET_DEPENDENCIES_FOR_TASK: "Action"
    ASANA_TASKS_SET_PARENT_TASK: "Action"
    ASANA_TASKS_SET_TASK_DEPENDENTS: "Action"
    ASANA_TASKS_UPDATE_TASK_RECORD: "Action"
    ASANA_TASK_SUN_LINK_DEPENDENCIES_FROM_TASK: "Action"
    ASANA_TASK_SUN_LINK_DEPENDENTS: "Action"
    ASANA_TASK_TEMPLATES_DELETE_TASK_TEMPLATE: "Action"
    ASANA_TASK_TEMPLATES_GET_SINGLE_TEMPLATE: "Action"
    ASANA_TASK_TEMPLATES_INSTANTIATE_TASK_JOB: "Action"
    ASANA_TASK_TEMPLATES_LIST_MULTIPLE: "Action"
    ASANA_TEAMS_ADD_USER_TO_TEAM: "Action"
    ASANA_TEAMS_CREATE_TEAM_RECORD: "Action"
    ASANA_TEAMS_GET_TEAM_RECORD: "Action"
    ASANA_TEAMS_GET_USER_TEAMS: "Action"
    ASANA_TEAMS_LIST_WORK_SPACE_TEAMS: "Action"
    ASANA_TEAMS_REMOVE_USER_FROM_TEAM: "Action"
    ASANA_TEAMS_UPDATE_TEAM_RECORD: "Action"
    ASANA_TEAM_MEMBERSHIPS_GET_COMPACT: "Action"
    ASANA_TEAM_MEMBERSHIPS_GET_COMPACT_RECORDS: "Action"
    ASANA_TEAM_MEMBERSHIPS_GET_RECORD_BY_ID: "Action"
    ASANA_TEAM_MEMBERSHIPS_GET_USER_COMPACT: "Action"
    ASANA_TIME_PERIODS_GET_COMPACT_TIME_PERIODS: "Action"
    ASANA_TIME_PERIODS_GET_FULL_RECORD: "Action"
    ASANA_TIME_TRACKING_ENTRIES_CREATE_NEW_TIME_ENTRY_RECORD: "Action"
    ASANA_TIME_TRACKING_ENTRIES_DELETE_TIME_ENTRY: "Action"
    ASANA_TIME_TRACKING_ENTRIES_GET_FOR_TASK: "Action"
    ASANA_TIME_TRACKING_ENTRIES_GET_RECORD: "Action"
    ASANA_TIME_TRACKING_ENTRIES_UPDATE_TIME_TRACKING_ENTRY: "Action"
    ASANA_TYPE_AHEAD_GET_RESULTS: "Action"
    ASANA_USERS_GET_FAVORITES_FOR_USER: "Action"
    ASANA_USERS_GET_USER_RECORD: "Action"
    ASANA_USERS_LIST_MULTIPLE_USERS: "Action"
    ASANA_USERS_LIST_TEAM_USERS: "Action"
    ASANA_USERS_LIST_WORK_SPACE_USERS: "Action"
    ASANA_USER_TASK_LISTS_GET_RECORD: "Action"
    ASANA_USER_TASK_LISTS_GET_USER_TASK_LIST: "Action"
    ASANA_WEB_HOOKS_ESTABLISH_WEB_HOOK: "Action"
    ASANA_WEB_HOOKS_GET_WEB_HOOK_RECORD: "Action"
    ASANA_WEB_HOOKS_LIST_MULTIPLE_WEB_HOOKS: "Action"
    ASANA_WEB_HOOKS_REMOVE_WEB_HOOK: "Action"
    ASANA_WEB_HOOKS_UPDATE_WEB_HOOK_FILTERS: "Action"
    ASANA_WORK_SPACES_ADD_USER_TO_WORK_SPACE: "Action"
    ASANA_WORK_SPACES_GET_WORK_SPACE_RECORD: "Action"
    ASANA_WORK_SPACES_LIST_MULTIPLE: "Action"
    ASANA_WORK_SPACES_REMOVE_USER_FROM_WORK_SPACE: "Action"
    ASANA_WORK_SPACES_UPDATE_WORK_SPACE_RECORD: "Action"
    ASANA_WORK_SPACE_MEMBERSHIPS_GET_RECORD_BY_ID: "Action"
    ASANA_WORK_SPACE_MEMBERSHIPS_GET_USER_MEMBERSHIPS: "Action"
    ASANA_WORK_SPACE_MEMBERSHIPS_LIST_FOR_WORK_SPACE: "Action"
    ATTIO_ASSERT_A_LIST_ENTRY_BY_PARENT: "Action"
    ATTIO_ASSERT_A_RECORD: "Action"
    ATTIO_CREATE_AN_ATTRIBUTE: "Action"
    ATTIO_CREATE_AN_ENTRY_ADD_RECORD_TO_LIST: "Action"
    ATTIO_CREATE_AN_OBJECT: "Action"
    ATTIO_CREATE_A_COMMENT: "Action"
    ATTIO_CREATE_A_LIST: "Action"
    ATTIO_CREATE_A_NOTE: "Action"
    ATTIO_CREATE_A_RECORD: "Action"
    ATTIO_CREATE_A_SELECT_OPTION: "Action"
    ATTIO_CREATE_A_STATUS: "Action"
    ATTIO_CREATE_A_TASK: "Action"
    ATTIO_CREATE_A_WEB_HOOK: "Action"
    ATTIO_DELETE_A_COMMENT: "Action"
    ATTIO_DELETE_A_LIST_ENTRY: "Action"
    ATTIO_DELETE_A_NOTE: "Action"
    ATTIO_DELETE_A_RECORD: "Action"
    ATTIO_DELETE_A_TASK: "Action"
    ATTIO_DELETE_A_WEB_HOOK: "Action"
    ATTIO_GET_AN_ATTRIBUTE: "Action"
    ATTIO_GET_AN_OBJECT: "Action"
    ATTIO_GET_A_COMMENT: "Action"
    ATTIO_GET_A_LIST: "Action"
    ATTIO_GET_A_LIST_ENTRY: "Action"
    ATTIO_GET_A_NOTE: "Action"
    ATTIO_GET_A_RECORD: "Action"
    ATTIO_GET_A_TASK: "Action"
    ATTIO_GET_A_THREAD: "Action"
    ATTIO_GET_A_WEB_HOOK: "Action"
    ATTIO_GET_A_WORK_SPACE_MEMBER: "Action"
    ATTIO_IDENTIFY: "Action"
    ATTIO_LIST_ALL_LISTS: "Action"
    ATTIO_LIST_ATTRIBUTES: "Action"
    ATTIO_LIST_ATTRIBUTE_VALUES_FOR_A_LIST_ENTRY: "Action"
    ATTIO_LIST_ENTRIES: "Action"
    ATTIO_LIST_NOTES: "Action"
    ATTIO_LIST_OBJECTS: "Action"
    ATTIO_LIST_RECORDS: "Action"
    ATTIO_LIST_RECORD_ATTRIBUTE_VALUES: "Action"
    ATTIO_LIST_RECORD_ENTRIES: "Action"
    ATTIO_LIST_SELECT_OPTIONS: "Action"
    ATTIO_LIST_STATUSES: "Action"
    ATTIO_LIST_TASKS: "Action"
    ATTIO_LIST_THREADS: "Action"
    ATTIO_LIST_WEB_HOOKS: "Action"
    ATTIO_LIST_WORK_SPACE_MEMBERS: "Action"
    ATTIO_UPDATE_AN_ATTRIBUTE: "Action"
    ATTIO_UPDATE_AN_OBJECT: "Action"
    ATTIO_UPDATE_A_LIST: "Action"
    ATTIO_UPDATE_A_LIST_ENTRY_APPEND_MULTI_SELECT_VALUES: "Action"
    ATTIO_UPDATE_A_LIST_ENTRY_OVERWRITE_MULTI_SELECT_VALUES: "Action"
    ATTIO_UPDATE_A_RECORD: "Action"
    ATTIO_UPDATE_A_SELECT_OPTION: "Action"
    ATTIO_UPDATE_A_STATUS: "Action"
    ATTIO_UPDATE_A_TASK: "Action"
    ATTIO_UPDATE_A_WEB_HOOK: "Action"
    BREVO_ACCOUNT_GET_USER_ACTIVITY_LOGS: "Action"
    BREVO_ACCOUNT_INFORMATION_DETAILS: "Action"
    BREVO_COMPANIES_CREATE_COMPANY: "Action"
    BREVO_COMPANIES_DELETE_COMPANY: "Action"
    BREVO_COMPANIES_GET_ALL: "Action"
    BREVO_COMPANIES_GET_ATTRIBUTES: "Action"
    BREVO_COMPANIES_GET_COMPANY_BY_ID: "Action"
    BREVO_COMPANIES_LINK_UN_LINK_WITH_CONTACT_DEAL: "Action"
    BREVO_COMPANIES_UPDATE_COMPANY: "Action"
    BREVO_CONTACTS_ADD_TO_LIST: "Action"
    BREVO_CONTACTS_CREATE_ATTRIBUTE: "Action"
    BREVO_CONTACTS_CREATE_DOUBLE_OPT_IN_CONTACT: "Action"
    BREVO_CONTACTS_CREATE_FOLDER: "Action"
    BREVO_CONTACTS_CREATE_LIST: "Action"
    BREVO_CONTACTS_CREATE_NEW_CONTACT: "Action"
    BREVO_CONTACTS_DELETE_CONTACT: "Action"
    BREVO_CONTACTS_DELETE_FOLDER: "Action"
    BREVO_CONTACTS_DELETE_LIST: "Action"
    BREVO_CONTACTS_EXPORT_PROCESS_ID: "Action"
    BREVO_CONTACTS_GET_ALL_CONTACTS: "Action"
    BREVO_CONTACTS_GET_ALL_FOLDERS: "Action"
    BREVO_CONTACTS_GET_ALL_LISTS: "Action"
    BREVO_CONTACTS_GET_ALL_SEGMENTS: "Action"
    BREVO_CONTACTS_GET_DETAILS: "Action"
    BREVO_CONTACTS_GET_EMAIL_CAMPAIGN_STATS: "Action"
    BREVO_CONTACTS_GET_FOLDER_DETAILS: "Action"
    BREVO_CONTACTS_GET_FOLDER_LISTS: "Action"
    BREVO_CONTACTS_GET_LIST_CONTACTS: "Action"
    BREVO_CONTACTS_GET_LIST_DETAILS: "Action"
    BREVO_CONTACTS_IMPORT_CONTACTS_PROCESS: "Action"
    BREVO_CONTACTS_LIST_ATTRIBUTES: "Action"
    BREVO_CONTACTS_REMOVE_ATTRIBUTE: "Action"
    BREVO_CONTACTS_REMOVE_CONTACT_FROM_LIST: "Action"
    BREVO_CONTACTS_UPDATE_ATTRIBUTE: "Action"
    BREVO_CONTACTS_UPDATE_CONTACT_BY_ID: "Action"
    BREVO_CONTACTS_UPDATE_FOLDER: "Action"
    BREVO_CONTACTS_UPDATE_LIST: "Action"
    BREVO_CONTACTS_UPDATE_MULTIPLE: "Action"
    BREVO_CONVERSATIONS_DELETE_AUTOMATED_MESSAGE: "Action"
    BREVO_CONVERSATIONS_DELETE_MESSAGE_SENT_BY_AGENT: "Action"
    BREVO_CONVERSATIONS_GET_AUTOMATED_MESSAGE: "Action"
    BREVO_CONVERSATIONS_GET_MESSAGE_BY_ID: "Action"
    BREVO_CONVERSATIONS_SEND_AUTOMATED_MESSAGE: "Action"
    BREVO_CONVERSATIONS_SEND_MESSAGE_AS_AGENT: "Action"
    BREVO_CONVERSATIONS_SET_AGENT_ONLINE_STATUS: "Action"
    BREVO_CONVERSATIONS_UPDATE_AGENT_MESSAGE: "Action"
    BREVO_CONVERSATIONS_UPDATE_PUSHED_MESSAGE: "Action"
    BREVO_COUPONS_CREATE_COLLECTION: "Action"
    BREVO_COUPONS_CREATE_COUPON_COLLECTION: "Action"
    BREVO_COUPONS_GET_BY_ID: "Action"
    BREVO_COUPONS_LIST_COUPON_COLLECTIONS: "Action"
    BREVO_COUPONS_UPDATE_COUPON_COLLECTION_BY_ID: "Action"
    BREVO_DEALS_CREATE_NEW_DEAL: "Action"
    BREVO_DEALS_DELETE_DEAL: "Action"
    BREVO_DEALS_GET_ALL_DEALS: "Action"
    BREVO_DEALS_GET_ALL_PIPELINES: "Action"
    BREVO_DEALS_GET_ATTRIBUTES: "Action"
    BREVO_DEALS_GET_BY_ID: "Action"
    BREVO_DEALS_GET_DETAILS: "Action"
    BREVO_DEALS_GET_PIPELINE_STAGES: "Action"
    BREVO_DEALS_LINK_UN_LINK_PATCH: "Action"
    BREVO_DEALS_UPDATE_DEAL_BY_ID: "Action"
    BREVO_DOMAINS_AUTHENTICATE_DOMAIN: "Action"
    BREVO_DOMAINS_CREATE_NEW_DOMAIN: "Action"
    BREVO_DOMAINS_DELETE_DOMAIN: "Action"
    BREVO_DOMAINS_GET_ALL: "Action"
    BREVO_DOMAINS_VALIDATE_CONFIGURATION: "Action"
    BREVO_EMAIL_CAMPAIGNS_CREATE_CAMPAIGN: "Action"
    BREVO_EMAIL_CAMPAIGNS_EXPORT_RECIPIENTS_POST: "Action"
    BREVO_EMAIL_CAMPAIGNS_GET_AB_TEST_RESULT: "Action"
    BREVO_EMAIL_CAMPAIGNS_GET_ALL: "Action"
    BREVO_EMAIL_CAMPAIGNS_GET_REPORT: "Action"
    BREVO_EMAIL_CAMPAIGNS_GET_SHARED_URL: "Action"
    BREVO_EMAIL_CAMPAIGNS_REMOVE_CAMPAIGN: "Action"
    BREVO_EMAIL_CAMPAIGNS_SEND_IMMEDIATE: "Action"
    BREVO_EMAIL_CAMPAIGNS_SEND_REPORT: "Action"
    BREVO_EMAIL_CAMPAIGNS_SEND_TEST_TO_TEST_LIST: "Action"
    BREVO_EMAIL_CAMPAIGNS_UPDATE_CAMPAIGN: "Action"
    BREVO_EMAIL_CAMPAIGNS_UPDATE_STATUS: "Action"
    BREVO_EMAIL_CAMPAIGNS_UPLOAD_IMAGE_TO_GALLERY: "Action"
    BREVO_EVENT_TRACK_INTERACTION: "Action"
    BREVO_EXTERNAL_FEEDS_CREATE_FEED: "Action"
    BREVO_EXTERNAL_FEEDS_DELETE_FEED_BYU_U_ID: "Action"
    BREVO_EXTERNAL_FEEDS_GET_ALL_FEEDS: "Action"
    BREVO_EXTERNAL_FEEDS_GET_FEED_BYU_U_ID: "Action"
    BREVO_EXTERNAL_FEEDS_UPDATE_FEED_BYU_U_ID: "Action"
    BREVO_E_COMMERCE_ACTIVATE_APP: "Action"
    BREVO_E_COMMERCE_CREATE_CATEGORIES_BATCH: "Action"
    BREVO_E_COMMERCE_CREATE_CATEGORY: "Action"
    BREVO_E_COMMERCE_CREATE_ORDER_BATCH: "Action"
    BREVO_E_COMMERCE_CREATE_PRODUCT: "Action"
    BREVO_E_COMMERCE_CREATE_PRODUCTS_BATCH: "Action"
    BREVO_E_COMMERCE_GET_ALL_CATEGORIES: "Action"
    BREVO_E_COMMERCE_GET_CATEGORY_DETAILS: "Action"
    BREVO_E_COMMERCE_GET_ORDERS: "Action"
    BREVO_E_COMMERCE_GET_PRODUCT_DETAILS: "Action"
    BREVO_E_COMMERCE_LIST_ALL_PRODUCTS: "Action"
    BREVO_E_COMMERCE_MANAGE_ORDER_STATUS: "Action"
    BREVO_FILES_DELETE_FILE: "Action"
    BREVO_FILES_DOWNLOAD_FILE: "Action"
    BREVO_FILES_GET_ALL_FILES: "Action"
    BREVO_FILES_GET_FILE_DETAILS: "Action"
    BREVO_FILES_UPLOAD_FILE: "Action"
    BREVO_INBOUND_PARSING_GET_ALL_EVENTS: "Action"
    BREVO_INBOUND_PARSING_GET_ATTACHMENT_BY_TOKEN: "Action"
    BREVO_INBOUND_PARSING_GET_EMAIL_EVENTS: "Action"
    BREVO_MASTER_ACCOUNTS_END_INVITATION_TO_ADMIN_USER: "Action"
    BREVO_MASTER_ACCOUNT_CHECK_ADMIN_USER_PERMISSIONS: "Action"
    BREVO_MASTER_ACCOUNT_CREATE_GROUP_OF_SUB_ACCOUNTS: "Action"
    BREVO_MASTER_ACCOUNT_CREATE_SUB_ACCOUNT: "Action"
    BREVO_MASTER_ACCOUNT_CREATE_SUB_ACCOUNT_KEY: "Action"
    BREVO_MASTER_ACCOUNT_DELETE_GROUP: "Action"
    BREVO_MASTER_ACCOUNT_DELETE_SUB_ACCOUNT: "Action"
    BREVO_MASTER_ACCOUNT_ENABLE_DISABLE: "Action"
    BREVO_MASTER_ACCOUNT_GENERATES_SO_TOKEN: "Action"
    BREVO_MASTER_ACCOUNT_GENERATES_SO_TOKEN_2: "Action"
    BREVO_MASTER_ACCOUNT_GET_DETAILS: "Action"
    BREVO_MASTER_ACCOUNT_GET_GROUP_DETAILS: "Action"
    BREVO_MASTER_ACCOUNT_GET_SUB_ACCOUNT_DETAILS: "Action"
    BREVO_MASTER_ACCOUNT_LIST_ADMIN_USERS: "Action"
    BREVO_MASTER_ACCOUNT_LIST_GROUPS: "Action"
    BREVO_MASTER_ACCOUNT_LIST_SUB_ACCOUNTS: "Action"
    BREVO_MASTER_ACCOUNT_RESEND_CANCEL_ADMIN_USER_INVITATION: "Action"
    BREVO_MASTER_ACCOUNT_REVOKE_ADMIN_USER: "Action"
    BREVO_MASTER_ACCOUNT_UN_LINK_SUB_ACCOUNT_FROM_GROUP: "Action"
    BREVO_MASTER_ACCOUNT_UPDATE_GROUP_SUB_ACCOUNTS: "Action"
    BREVO_MASTER_ACCOUNT_UPDATE_SUB_ACCOUNT_PLAN: "Action"
    BREVO_NOTES_CREATE_NEW_NOTE: "Action"
    BREVO_NOTES_GET_ALL: "Action"
    BREVO_NOTES_GET_BY_ID: "Action"
    BREVO_NOTES_REMOVE_BY_ID: "Action"
    BREVO_NOTES_UPDATE_NOTE_BY_ID: "Action"
    BREVO_PROCESS_GET_ALL_PROCESSES: "Action"
    BREVO_PROCESS_GET_PROCESS_INFORMATION: "Action"
    BREVO_RESELLER_ADD_CHILD_CREDITS: "Action"
    BREVO_RESELLER_ASSOCIATE_DEDICATED_IP_TO_CHILD: "Action"
    BREVO_RESELLER_CREATE_CHILD: "Action"
    BREVO_RESELLER_CREATE_CHILD_DOMAIN: "Action"
    BREVO_RESELLER_DELETE_CHILD_BY_IDENTIFIER: "Action"
    BREVO_RESELLER_DELETE_SENDER_DOMAIN_BY_CHILD_IDENTIFIER_AND_DOMAIN_NAME: "Action"
    BREVO_RESELLER_DISSOCIATE_IP_TO_CHILD: "Action"
    BREVO_RESELLER_GET_CHILD_ACCOUNT_CREATION_STATUS: "Action"
    BREVO_RESELLER_GET_CHILD_DETAILS: "Action"
    BREVO_RESELLER_GET_CHILD_DOMAINS: "Action"
    BREVO_RESELLER_GET_SESSION_TOKEN: "Action"
    BREVO_RESELLER_LIST_CHILDREN_ACCOUNTS: "Action"
    BREVO_RESELLER_REMOVE_CREDITS_FROM_CHILD: "Action"
    BREVO_RESELLER_UPDATE_CHILD_ACCOUNT_STATUS: "Action"
    BREVO_RESELLER_UPDATE_CHILD_INFO: "Action"
    BREVO_RESELLER_UPDATE_SENDER_DOMAIN: "Action"
    BREVO_SENDERS_CREATE_NEW_SENDER: "Action"
    BREVO_SENDERS_GET_ALL_DEDICATED_IPS: "Action"
    BREVO_SENDERS_GET_DEDICATED_IPS: "Action"
    BREVO_SENDERS_LIST_ALL: "Action"
    BREVO_SENDERS_REMOVE_SENDER: "Action"
    BREVO_SENDERS_UPDATES_ENDERBY_ID: "Action"
    BREVO_SENDERS_VALIDATE_SENDER_USING_OT_P: "Action"
    BREVO_SMS_CAMPAIGNS_CREATE_CAMPAIGN: "Action"
    BREVO_SMS_CAMPAIGNS_EXPORT_RECIPIENTS_PROCESS: "Action"
    BREVO_SMS_CAMPAIGNS_GET_ALL_INFORMATION: "Action"
    BREVO_SMS_CAMPAIGNS_GET_CAMPAIGN_BY_ID: "Action"
    BREVO_SMS_CAMPAIGNS_REMOVE_CAMPAIGN_BY_ID: "Action"
    BREVO_SMS_CAMPAIGNS_SEND_CAMPAIGN_REPORT: "Action"
    BREVO_SMS_CAMPAIGNS_SEND_IMMEDIATELY: "Action"
    BREVO_SMS_CAMPAIGNS_SEND_TESTS_MS: "Action"
    BREVO_SMS_CAMPAIGNS_UPDATE_CAMPAIGN_BY_ID: "Action"
    BREVO_SMS_CAMPAIGNS_UPDATE_STATUS: "Action"
    BREVO_TASKS_CREATE_NEW_TASK: "Action"
    BREVO_TASKS_GET_ALL: "Action"
    BREVO_TASKS_GET_ALL_TASK_TYPES: "Action"
    BREVO_TASKS_GET_TASK_BY_ID: "Action"
    BREVO_TASKS_REMOVE_TASK: "Action"
    BREVO_TASKS_UPDATE_TASK: "Action"
    BREVO_TRANSACTIONAL_EMAILS_ADD_BLOCKED_DOMAIN: "Action"
    BREVO_TRANSACTIONAL_EMAILS_CREATE_TEMPLATE: "Action"
    BREVO_TRANSACTIONAL_EMAILS_DELETE_LOG: "Action"
    BREVO_TRANSACTIONAL_EMAILS_DELETE_SCHEDULED_EMAILS: "Action"
    BREVO_TRANSACTIONAL_EMAILS_DELETE_TEMPLATE_BY_ID: "Action"
    BREVO_TRANSACTIONAL_EMAILS_GET_ACTIVITY_PER_DAY: "Action"
    BREVO_TRANSACTIONAL_EMAILS_GET_AGGREGATED_REPORT: "Action"
    BREVO_TRANSACTIONAL_EMAILS_GET_ALL_ACTIVITY: "Action"
    BREVO_TRANSACTIONAL_EMAILS_GET_BLOCKED_DOMAINS: "Action"
    BREVO_TRANSACTIONAL_EMAILS_GET_CONTENT_BYU_U_ID: "Action"
    BREVO_TRANSACTIONAL_EMAILS_GET_EMAIL_STATUS_BY_ID: "Action"
    BREVO_TRANSACTIONAL_EMAILS_GET_LIST: "Action"
    BREVO_TRANSACTIONAL_EMAILS_GET_TEMPLATE_INFO: "Action"
    BREVO_TRANSACTIONAL_EMAILS_LIST_BLOCKED_CONTACTS: "Action"
    BREVO_TRANSACTIONAL_EMAILS_LIST_TEMPLATES: "Action"
    BREVO_TRANSACTIONAL_EMAILS_REMOVE_HARD_BOUNCES: "Action"
    BREVO_TRANSACTIONAL_EMAILS_SEND_TEST_TEMPLATE: "Action"
    BREVO_TRANSACTIONAL_EMAILS_SEND_TRANSACTIONAL_EMAIL: "Action"
    BREVO_TRANSACTIONAL_EMAILS_UNBLOCK_CONTACT: "Action"
    BREVO_TRANSACTIONAL_EMAILS_UNBLOCK_DOMAIN: "Action"
    BREVO_TRANSACTIONAL_EMAILS_UPDATE_TEMPLATE: "Action"
    BREVO_TRANSACTIONAL_SMS_GETS_MS_ACTIVITY_AGGREGATED_PER_DAY: "Action"
    BREVO_TRANSACTIONAL_SMS_GET_AGGREGATED_REPORT: "Action"
    BREVO_TRANSACTIONAL_SMS_GET_ALL_EVENTS: "Action"
    BREVO_TRANSACTIONAL_SMS_SENDS_MS_MESSAGE_TO_MOBILE: "Action"
    BREVO_TRANSACTIONAL_WHAT_S_APPS_END_MESSAGE: "Action"
    BREVO_TRANSACTIONAL_WHAT_S_APP_GET_ACTIVITY: "Action"
    BREVO_USERS_END_INVITATION: "Action"
    BREVO_USER_CHECK_PERMISSION: "Action"
    BREVO_USER_GET_ALL_USERS: "Action"
    BREVO_USER_RESEND_INVITATION: "Action"
    BREVO_USER_REVOKE_PERMISSION_BY_EMAIL: "Action"
    BREVO_USER_UPDATE_PERMISSIONS: "Action"
    BREVO_WEB_HOOKS_CREATE_HOOK: "Action"
    BREVO_WEB_HOOKS_DELETE_WEB_HOOK: "Action"
    BREVO_WEB_HOOKS_EXPORT_ALL_EVENTS: "Action"
    BREVO_WEB_HOOKS_GET_ALL: "Action"
    BREVO_WEB_HOOKS_GET_DETAILS: "Action"
    BREVO_WEB_HOOKS_UPDATE_WEB_HOOK_BY_ID: "Action"
    BREVO_WHAT_S_APP_CAMPAIGNS_CREATE_AND_SEND: "Action"
    BREVO_WHAT_S_APP_CAMPAIGNS_CREATE_TEMPLATE: "Action"
    BREVO_WHAT_S_APP_CAMPAIGNS_DELETE_CAMPAIGN: "Action"
    BREVO_WHAT_S_APP_CAMPAIGNS_GET_ACCOUNT_INFO: "Action"
    BREVO_WHAT_S_APP_CAMPAIGNS_GET_ALL: "Action"
    BREVO_WHAT_S_APP_CAMPAIGNS_GET_CAMPAIGN_BY_ID: "Action"
    BREVO_WHAT_S_APP_CAMPAIGNS_GET_TEMPLATES: "Action"
    BREVO_WHAT_S_APP_CAMPAIGNS_SEND_TEMPLATE_FOR_APPROVAL: "Action"
    BREVO_WHAT_S_APP_CAMPAIGNS_UPDATE_CAMPAIGN_BY_ID: "Action"
    CLICKUP_ATTACHMENTS_UPLOAD_FILE_TO_TASK_AS_ATTACHMENT: "Action"
    CLICKUP_AUTHORIZATION_GET_ACCESS_TOKEN: "Action"
    CLICKUP_AUTHORIZATION_GET_WORK_SPACE_LIST: "Action"
    CLICKUP_AUTHORIZATION_VIEW_ACCOUNT_DETAILS: "Action"
    CLICKUP_COMMENTS_ADD_TO_LIST_COMMENT: "Action"
    CLICKUP_COMMENTS_CREATE_CHAT_VIEW_COMMENT: "Action"
    CLICKUP_COMMENTS_CREATE_NEW_TASK_COMMENT: "Action"
    CLICKUP_COMMENTS_DELETE_TASK_COMMENT: "Action"
    CLICKUP_COMMENTS_GET_LIST_COMMENTS: "Action"
    CLICKUP_COMMENTS_GET_TASK_COMMENTS: "Action"
    CLICKUP_COMMENTS_GET_VIEW_COMMENTS: "Action"
    CLICKUP_COMMENTS_UPDATE_TASK_COMMENT: "Action"
    CLICKUP_CUSTOM_FIELDS_GET_LIST_FIELDS: "Action"
    CLICKUP_CUSTOM_FIELDS_REMOVE_FIELD_VALUE: "Action"
    CLICKUP_CUSTOM_TASK_TYPES_GET_AVAILABLE_TASK_TYPES: "Action"
    CLICKUP_FOLDERS_CREATE_NEW_FOLDER: "Action"
    CLICKUP_FOLDERS_GET_CONTENTS_OF: "Action"
    CLICKUP_FOLDERS_GET_FOLDER_CONTENT: "Action"
    CLICKUP_FOLDERS_REMOVE_FOLDER: "Action"
    CLICKUP_FOLDERS_RENAME_FOLDER: "Action"
    CLICKUP_GOALS_ADD_KEY_RESULT: "Action"
    CLICKUP_GOALS_ADD_NEW_GOAL_TO_WORK_SPACE: "Action"
    CLICKUP_GOALS_GET_DETAILS: "Action"
    CLICKUP_GOALS_GET_WORK_SPACE_GOALS: "Action"
    CLICKUP_GOALS_REMOVE_GOAL: "Action"
    CLICKUP_GOALS_REMOVE_TARGET: "Action"
    CLICKUP_GOALS_UPDATE_GOAL_DETAILS: "Action"
    CLICKUP_GOALS_UPDATE_KEY_RESULT: "Action"
    CLICKUP_GUESTS_ADD_GUEST_TO_FOLDER: "Action"
    CLICKUP_GUESTS_ADD_TO_TASK: "Action"
    CLICKUP_GUESTS_EDIT_GUEST_ON_WORK_SPACE: "Action"
    CLICKUP_GUESTS_GET_GUEST_INFORMATION: "Action"
    CLICKUP_GUESTS_INVITE_TO_WORK_SPACE: "Action"
    CLICKUP_GUESTS_REMOVE_FROM_LIST: "Action"
    CLICKUP_GUESTS_REVOKE_ACCESS_FROM_FOLDER: "Action"
    CLICKUP_GUESTS_REVOKE_ACCESS_TO_TASK: "Action"
    CLICKUP_GUESTS_REVOKE_GUEST_ACCESS_TO_WORK_SPACE: "Action"
    CLICKUP_GUESTS_SHARE_LIST_WITH: "Action"
    CLICKUP_LISTS_ADD_TASK_TO_LIST: "Action"
    CLICKUP_LISTS_ADD_TO_FOLDER: "Action"
    CLICKUP_LISTS_CREATE_FOLDER_LESS_LIST: "Action"
    CLICKUP_LISTS_GET_FOLDER_LESS: "Action"
    CLICKUP_LISTS_GET_FOLDER_LISTS: "Action"
    CLICKUP_LISTS_GET_LIST_DETAILS: "Action"
    CLICKUP_LISTS_REMOVE_LIST: "Action"
    CLICKUP_LISTS_REMOVE_TASK_FROM_LIST: "Action"
    CLICKUP_LISTS_UPDATE_LIST_INFO_DUE_DATE_PRIORITY_ASSIGN_EE_COLOR: "Action"
    CLICKUP_MEMBERS_GET_LIST_USERS: "Action"
    CLICKUP_MEMBERS_GET_TASK_ACCESS: "Action"
    CLICKUP_ROLES_LIST_AVAILABLE_CUSTOM_ROLES: "Action"
    CLICKUP_SHARED_HIERARCHY_VIEW_TASKS_LISTS_FOLDERS: "Action"
    CLICKUP_SPACES_ADD_NEW_SPACE_TO_WORK_SPACE: "Action"
    CLICKUP_SPACES_GET_DETAILS: "Action"
    CLICKUP_SPACES_GET_SPACE_DETAILS: "Action"
    CLICKUP_SPACES_REMOVE_SPACE: "Action"
    CLICKUP_SPACES_UPDATE_DETAILS_AND_ENABLE_CLICK_APPS: "Action"
    CLICKUP_TAGS_ADD_TO_TASK: "Action"
    CLICKUP_TAGS_CREATE_SPACE_TAG: "Action"
    CLICKUP_TAGS_GET_SPACE: "Action"
    CLICKUP_TAGS_REMOVE_FROM_TASK: "Action"
    CLICKUP_TAGS_REMOVE_SPACE_TAG: "Action"
    CLICKUP_TAGS_UPDATE_SPACE_TAG: "Action"
    CLICKUP_TASKS_CREATE_NEW_TASK: "Action"
    CLICKUP_TASKS_FILTER_TEAM_TASKS: "Action"
    CLICKUP_TASKS_GET_LIST_TASKS: "Action"
    CLICKUP_TASKS_GET_TASK_DETAILS: "Action"
    CLICKUP_TASKS_GET_TIME_IN_STATUS: "Action"
    CLICKUP_TASKS_GET_TIME_IN_STATUS_BULK: "Action"
    CLICKUP_TASKS_REMOVE_TASK_BY_ID: "Action"
    CLICKUP_TASKS_UPDATE_TASK_FIELDS: "Action"
    CLICKUP_TASK_CHECKLISTS_ADD_LINE_ITEM: "Action"
    CLICKUP_TASK_CHECKLISTS_CREATE_NEW_CHECKLIST: "Action"
    CLICKUP_TASK_CHECKLISTS_REMOVE_CHECKLIST: "Action"
    CLICKUP_TASK_CHECKLISTS_REMOVE_CHECKLIST_ITEM: "Action"
    CLICKUP_TASK_CHECKLISTS_UPDATE_CHECKLIST: "Action"
    CLICKUP_TASK_CHECKLISTS_UPDATE_CHECKLIST_ITEM: "Action"
    CLICKUP_TASK_RELATIONSHIPS_ADD_DEPENDENCY: "Action"
    CLICKUP_TASK_RELATIONSHIPS_LINK_TASKS: "Action"
    CLICKUP_TASK_RELATIONSHIPS_REMOVE_DEPENDENCY: "Action"
    CLICKUP_TASK_RELATIONSHIPS_REMOVE_LINK_BETWEEN_TASKS: "Action"
    CLICKUP_TASK_TEMPLATES_CREATE_FROM_TEMPLATE: "Action"
    CLICKUP_TASK_TEMPLATES_GET_TEMPLATES: "Action"
    CLICKUP_TEAMS_USER_GROUPS_CREATE_TEAM: "Action"
    CLICKUP_TEAMS_USER_GROUPS_GET_USER_GROUPS: "Action"
    CLICKUP_TEAMS_USER_GROUPS_REMOVE_GROUP: "Action"
    CLICKUP_TEAMS_USER_GROUPS_UPDATE_USER_GROUP: "Action"
    CLICKUP_TEAMS_WORK_SPACES_GET_WORK_SPACE_PLAN: "Action"
    CLICKUP_TEAMS_WORK_SPACES_GET_WORK_SPACE_SEATS: "Action"
    CLICKUP_TIME_TRACKING_ADD_TAGS_FROM_TIME_ENTRIES: "Action"
    CLICKUP_TIME_TRACKING_CHANGE_TAG_NAMES: "Action"
    CLICKUP_TIME_TRACKING_CREATE_TIME_ENTRY: "Action"
    CLICKUP_TIME_TRACKING_GET_ALL_TAGS_FROM_TIME_ENTRIES: "Action"
    CLICKUP_TIME_TRACKING_GET_CURRENT_TIME_ENTRY: "Action"
    CLICKUP_TIME_TRACKING_GET_SINGLE_TIME_ENTRY: "Action"
    CLICKUP_TIME_TRACKING_GET_TIME_ENTRIES_WITHIN_DATE_RANGE: "Action"
    CLICKUP_TIME_TRACKING_GET_TIME_ENTRY_HISTORY: "Action"
    CLICKUP_TIME_TRACKING_LEGACY_EDIT_TIME_TRACKED: "Action"
    CLICKUP_TIME_TRACKING_LEGACY_GET_TRACKED_TIME: "Action"
    CLICKUP_TIME_TRACKING_LEGACY_RECORD_TIME_FOR_TASK: "Action"
    CLICKUP_TIME_TRACKING_LEGACY_REMOVE_TRACKED_TIME: "Action"
    CLICKUP_TIME_TRACKING_REMOVE_ENTRY: "Action"
    CLICKUP_TIME_TRACKING_REMOVE_TAGS_FROM_TIME_ENTRIES: "Action"
    CLICKUP_TIME_TRACKING_START_TIMER: "Action"
    CLICKUP_TIME_TRACKING_STOP_TIME_ENTRY: "Action"
    CLICKUP_TIME_TRACKING_UPDATE_TIME_ENTRY_DETAILS: "Action"
    CLICKUP_USERS_DEACTIVATE_FROM_WORK_SPACE: "Action"
    CLICKUP_USERS_GET_USER_DETAILS: "Action"
    CLICKUP_USERS_INVITE_USER_TO_WORK_SPACE: "Action"
    CLICKUP_USERS_UPDATE_USER_DETAILS: "Action"
    CLICKUP_VIEWS_DELETE_VIEW_BY_ID: "Action"
    CLICKUP_VIEWS_FOLDER_VIEWS_GET: "Action"
    CLICKUP_VIEWS_GET_EVERYTHING_LEVEL: "Action"
    CLICKUP_VIEWS_GET_LIST_VIEWS: "Action"
    CLICKUP_VIEWS_GET_TASKS_IN_VIEW: "Action"
    CLICKUP_VIEWS_GET_VIEW_INFO: "Action"
    CLICKUP_VIEWS_SPACE_VIEWS_GET: "Action"
    CLICKUP_WEB_HOOKS_CREATE_WEB_HOOK: "Action"
    CLICKUP_WEB_HOOKS_REMOVE_WEB_HOOK_BY_ID: "Action"
    CLICKUP_WEB_HOOKS_UPDATE_EVENTS_TO_MONITOR: "Action"
    CLICKUP_WEB_HOOKS_WORK_SPACE_GET: "Action"
    CODEINTERPRETER_CREATE_SANDBOX: "Action"
    CODEINTERPRETER_EXECUTE_CODE: "Action"
    CODEINTERPRETER_GET_FILE_CMD: "Action"
    CODEINTERPRETER_RUN_TERMINAL_CMD: "Action"
    COMPOSIO_CHECK_ACTIVE_CONNECTION: "Action"
    COMPOSIO_INITIATE_CONNECTION: "Action"
    COMPOSIO_RETRIEVE_ACTIONS: "Action"
    COMPOSIO_RETRIEVE_APPS: "Action"
    COMPOSIO_WAIT_FOR_CONNECTION: "Action"
    DISCORD_ADDGROUPDMUSER: "Action"
    DISCORD_ADDGUILDMEMBER: "Action"
    DISCORD_ADDGUILDMEMBERROLE: "Action"
    DISCORD_ADDMYMESSAGEREACTION: "Action"
    DISCORD_ADDTHREADMEMBER: "Action"
    DISCORD_BANUSERFROMGUILD: "Action"
    DISCORD_BULKBANUSERSFROMGUILD: "Action"
    DISCORD_BULKDELETEMESSAGES: "Action"
    DISCORD_CREATEAPPLICATIONCOMMAND: "Action"
    DISCORD_CREATEAUTOMODERATIONRULE: "Action"
    DISCORD_CREATEDM: "Action"
    DISCORD_CREATEGUILD: "Action"
    DISCORD_CREATEGUILDAPPLICATIONCOMMAND: "Action"
    DISCORD_CREATEGUILDCHANNEL: "Action"
    DISCORD_CREATEGUILDEMOJI: "Action"
    DISCORD_CREATEGUILDFROMTEMPLATE: "Action"
    DISCORD_CREATEGUILDROLE: "Action"
    DISCORD_CREATEGUILDSCHEDULEDEVENT: "Action"
    DISCORD_CREATEGUILDSTICKER: "Action"
    DISCORD_CREATEGUILDTEMPLATE: "Action"
    DISCORD_CREATEMESSAGE: "Action"
    DISCORD_CREATESTAGEINSTANCE: "Action"
    DISCORD_CREATETHREADFROMMESSAGE: "Action"
    DISCORD_CREATEWEBHOOK: "Action"
    DISCORD_CROSSPOSTMESSAGE: "Action"
    DISCORD_DELETEALLMESSAGEREACTIONS: "Action"
    DISCORD_DELETEALLMESSAGEREACTIONSBYEMOJI: "Action"
    DISCORD_DELETEAPPLICATIONCOMMAND: "Action"
    DISCORD_DELETEAUTOMODERATIONRULE: "Action"
    DISCORD_DELETECHANNEL: "Action"
    DISCORD_DELETECHANNELPERMISSIONOVERWRITE: "Action"
    DISCORD_DELETEGROUPDMUSER: "Action"
    DISCORD_DELETEGUILD: "Action"
    DISCORD_DELETEGUILDAPPLICATIONCOMMAND: "Action"
    DISCORD_DELETEGUILDEMOJI: "Action"
    DISCORD_DELETEGUILDINTEGRATION: "Action"
    DISCORD_DELETEGUILDMEMBER: "Action"
    DISCORD_DELETEGUILDMEMBERROLE: "Action"
    DISCORD_DELETEGUILDROLE: "Action"
    DISCORD_DELETEGUILDSCHEDULEDEVENT: "Action"
    DISCORD_DELETEGUILDSTICKER: "Action"
    DISCORD_DELETEGUILDTEMPLATE: "Action"
    DISCORD_DELETEMESSAGE: "Action"
    DISCORD_DELETEMYMESSAGEREACTION: "Action"
    DISCORD_DELETEORIGINALWEBHOOKMESSAGE: "Action"
    DISCORD_DELETESTAGEINSTANCE: "Action"
    DISCORD_DELETETHREADMEMBER: "Action"
    DISCORD_DELETEUSERMESSAGEREACTION: "Action"
    DISCORD_DELETEWEBHOOK: "Action"
    DISCORD_DELETEWEBHOOKBYTOKEN: "Action"
    DISCORD_DELETEWEBHOOKMESSAGE: "Action"
    DISCORD_EXECUTEGITHUBCOMPATIBLEWEBHOOK: "Action"
    DISCORD_EXECUTESLACKCOMPATIBLEWEBHOOK: "Action"
    DISCORD_FOLLOWCHANNEL: "Action"
    DISCORD_GETACTIVEGUILDTHREADS: "Action"
    DISCORD_GETAPPLICATION: "Action"
    DISCORD_GETAPPLICATIONCOMMAND: "Action"
    DISCORD_GETAPPLICATIONROLECONNECTIONSMETADATA: "Action"
    DISCORD_GETAPPLICATIONUSERROLECONNECTION: "Action"
    DISCORD_GETAUTOMODERATIONRULE: "Action"
    DISCORD_GETBOTGATEWAY: "Action"
    DISCORD_GETCHANNEL: "Action"
    DISCORD_GETGATEWAY: "Action"
    DISCORD_GETGUILD: "Action"
    DISCORD_GETGUILDAPPLICATIONCOMMAND: "Action"
    DISCORD_GETGUILDAPPLICATIONCOMMANDPERMISSIONS: "Action"
    DISCORD_GETGUILDBAN: "Action"
    DISCORD_GETGUILDEMOJI: "Action"
    DISCORD_GETGUILDMEMBER: "Action"
    DISCORD_GETGUILDNEWMEMBERWELCOME: "Action"
    DISCORD_GETGUILDPREVIEW: "Action"
    DISCORD_GETGUILDSCHEDULEDEVENT: "Action"
    DISCORD_GETGUILDSONBOARDING: "Action"
    DISCORD_GETGUILDSTICKER: "Action"
    DISCORD_GETGUILDTEMPLATE: "Action"
    DISCORD_GETGUILDVANITYURL: "Action"
    DISCORD_GETGUILDWEBHOOKS: "Action"
    DISCORD_GETGUILDWELCOMESCREEN: "Action"
    DISCORD_GETGUILDWIDGET: "Action"
    DISCORD_GETGUILDWIDGETPNG: "Action"
    DISCORD_GETGUILDWIDGETSETTINGS: "Action"
    DISCORD_GETMESSAGE: "Action"
    DISCORD_GETMYAPPLICATION: "Action"
    DISCORD_GETMYGUILDMEMBER: "Action"
    DISCORD_GETMYOAUTH2APPLICATION: "Action"
    DISCORD_GETMYOAUTH2AUTHORIZATION: "Action"
    DISCORD_GETMYUSER: "Action"
    DISCORD_GETORIGINALWEBHOOKMESSAGE: "Action"
    DISCORD_GETPUBLICKEYS: "Action"
    DISCORD_GETSTAGEINSTANCE: "Action"
    DISCORD_GETSTICKER: "Action"
    DISCORD_GETTHREADMEMBER: "Action"
    DISCORD_GETUSER: "Action"
    DISCORD_GETWEBHOOK: "Action"
    DISCORD_GETWEBHOOKBYTOKEN: "Action"
    DISCORD_GETWEBHOOKMESSAGE: "Action"
    DISCORD_INVITERESOLVE: "Action"
    DISCORD_INVITEREVOKE: "Action"
    DISCORD_JOINTHREAD: "Action"
    DISCORD_LEAVEGUILD: "Action"
    DISCORD_LEAVETHREAD: "Action"
    DISCORD_LISTAPPLICATIONCOMMANDS: "Action"
    DISCORD_LISTAUTOMODERATIONRULES: "Action"
    DISCORD_LISTCHANNELINVITES: "Action"
    DISCORD_LISTCHANNELWEBHOOKS: "Action"
    DISCORD_LISTGUILDAPPLICATIONCOMMANDPERMISSIONS: "Action"
    DISCORD_LISTGUILDAPPLICATIONCOMMANDS: "Action"
    DISCORD_LISTGUILDAUDITLOGENTRIES: "Action"
    DISCORD_LISTGUILDBANS: "Action"
    DISCORD_LISTGUILDCHANNELS: "Action"
    DISCORD_LISTGUILDEMOJIS: "Action"
    DISCORD_LISTGUILDINTEGRATIONS: "Action"
    DISCORD_LISTGUILDINVITES: "Action"
    DISCORD_LISTGUILDMEMBERS: "Action"
    DISCORD_LISTGUILDROLES: "Action"
    DISCORD_LISTGUILDSCHEDULEDEVENTS: "Action"
    DISCORD_LISTGUILDSCHEDULEDEVENTUSERS: "Action"
    DISCORD_LISTGUILDSTICKERS: "Action"
    DISCORD_LISTGUILDTEMPLATES: "Action"
    DISCORD_LISTGUILDVOICEREGIONS: "Action"
    DISCORD_LISTMESSAGEREACTIONSBYEMOJI: "Action"
    DISCORD_LISTMESSAGES: "Action"
    DISCORD_LISTMYCONNECTIONS: "Action"
    DISCORD_LISTMYGUILDS: "Action"
    DISCORD_LISTMYPRIVATEARCHIVEDTHREADS: "Action"
    DISCORD_LISTPINNEDMESSAGES: "Action"
    DISCORD_LISTPRIVATEARCHIVEDTHREADS: "Action"
    DISCORD_LISTPUBLICARCHIVEDTHREADS: "Action"
    DISCORD_LISTSTICKERPACKS: "Action"
    DISCORD_LISTTHREADMEMBERS: "Action"
    DISCORD_LISTVOICEREGIONS: "Action"
    DISCORD_PINMESSAGE: "Action"
    DISCORD_PREVIEWPRUNEGUILD: "Action"
    DISCORD_PRUNEGUILD: "Action"
    DISCORD_PUTGUILDSONBOARDING: "Action"
    DISCORD_SEARCHGUILDMEMBERS: "Action"
    DISCORD_SETCHANNELPERMISSIONOVERWRITE: "Action"
    DISCORD_SETGUILDAPPLICATIONCOMMANDPERMISSIONS: "Action"
    DISCORD_SETGUILDMFALEVEL: "Action"
    DISCORD_SYNCGUILDTEMPLATE: "Action"
    DISCORD_TRIGGERTYPINGINDICATOR: "Action"
    DISCORD_UNBANUSERFROMGUILD: "Action"
    DISCORD_UNPINMESSAGE: "Action"
    DISCORD_UPDATEAPPLICATION: "Action"
    DISCORD_UPDATEAPPLICATIONCOMMAND: "Action"
    DISCORD_UPDATEAPPLICATIONUSERROLECONNECTION: "Action"
    DISCORD_UPDATEGUILD: "Action"
    DISCORD_UPDATEGUILDAPPLICATIONCOMMAND: "Action"
    DISCORD_UPDATEGUILDEMOJI: "Action"
    DISCORD_UPDATEGUILDMEMBER: "Action"
    DISCORD_UPDATEGUILDROLE: "Action"
    DISCORD_UPDATEGUILDSTICKER: "Action"
    DISCORD_UPDATEGUILDTEMPLATE: "Action"
    DISCORD_UPDATEGUILDWELCOMESCREEN: "Action"
    DISCORD_UPDATEGUILDWIDGETSETTINGS: "Action"
    DISCORD_UPDATEMESSAGE: "Action"
    DISCORD_UPDATEMYAPPLICATION: "Action"
    DISCORD_UPDATEMYGUILDMEMBER: "Action"
    DISCORD_UPDATEMYUSER: "Action"
    DISCORD_UPDATEORIGINALWEBHOOKMESSAGE: "Action"
    DISCORD_UPDATESELFVOICESTATE: "Action"
    DISCORD_UPDATESTAGEINSTANCE: "Action"
    DISCORD_UPDATEVOICESTATE: "Action"
    DISCORD_UPDATEWEBHOOK: "Action"
    DISCORD_UPDATEWEBHOOKBYTOKEN: "Action"
    DISCORD_UPDATEWEBHOOKMESSAGE: "Action"
    DROPBOX_GET_ABOUT_ME: "Action"
    ELEVENLABS_ADD_A_PRONUNCIATION_DICTIONARY_V_1_PRONUNCIATION_DICTIONARIES_ADD_FROM_FILE_POST: "Action"
    ELEVENLABS_ADD_PROJECT_V_1_PROJECTS_ADD_POST: "Action"
    ELEVENLABS_ADD_RULES_TO_THE_PRONUNCIATION_DICTIONARY_V_1_PRONUNCIATION_DICTIONARIES_PRONUNCIATION_DICTIONARY_ID_ADD_RULES_POST: "Action"
    ELEVENLABS_ADD_SHARING_VOICE_V_1_VOICES_ADD_PUBLIC_USE_RID_VOICE_ID_POST: "Action"
    ELEVENLABS_ADD_VOICE_V_1_VOICES_ADD_POST: "Action"
    ELEVENLABS_CONVERT_CHAPTER_V_1_PROJECTS_PROJECT_ID_CHAPTERS_CHAPTER_ID_CONVERT_POST: "Action"
    ELEVENLABS_CONVERT_PROJECT_V_1_PROJECTS_PROJECT_ID_CONVERT_POST: "Action"
    ELEVENLABS_CREATES_AUDIO_NATIVE_ENABLED_PROJECT_V_1_AUDIO_NATIVE_POST: "Action"
    ELEVENLABS_CREATE_A_PREVIOUSLY_GENERATED_VOICE_V_1_VOICE_GENERATION_CREATE_VOICE_POST: "Action"
    ELEVENLABS_DELETE_CHAPTER_V_1_PROJECTS_PROJECT_ID_CHAPTERS_CHAPTER_ID_DELETE: "Action"
    ELEVENLABS_DELETE_DUBBING_PROJECT_V_1_DUBBING_DUBBING_ID_DELETE: "Action"
    ELEVENLABS_DELETE_HISTORY_ITEM_V_1_HISTORY_HISTORY_ITEMID_DELETE: "Action"
    ELEVENLABS_DELETE_PROJECT_V_1_PROJECTS_PROJECT_ID_DELETE: "Action"
    ELEVENLABS_DELETE_SAMPLE_V_1_VOICES_VOICE_ID_SAMPLES_SAMPLE_ID_DELETE: "Action"
    ELEVENLABS_DELETE_VOICE_V_1_VOICES_VOICE_ID_DELETE: "Action"
    ELEVENLABS_DOWNLOAD_HISTORY_ITEMS_V_1_HISTORY_DOWNLOAD_POST: "Action"
    ELEVENLABS_DUB_A_VIDEO_OR_AN_AUDIOFILE_V_1_DUBBING_POST: "Action"
    ELEVENLABS_EDIT_VOICE_SETTINGS_V_1_VOICES_VOICE_ID_SETTINGS_EDIT_POST: "Action"
    ELEVENLABS_EDIT_VOICE_V_1_VOICES_VOICE_ID_EDIT_POST: "Action"
    ELEVENLABS_GENERATE_A_RANDOM_VOICE_V_1_VOICE_GENERATION_GENERATE_VOICE_POST: "Action"
    ELEVENLABS_GETS_SO_PROVIDER_ADMIN_ADMIN_ADMIN_URL_PREFIX_S_SO_PROVIDER_GET: "Action"
    ELEVENLABS_GET_AUDIO_FROM_HISTORY_ITEM_V_1_HISTORY_HISTORY_ITEMID_AUDIO_GET: "Action"
    ELEVENLABS_GET_AUDIO_FROM_SAMPLE_V_1_VOICES_VOICE_ID_SAMPLES_SAMPLE_ID_AUDIO_GET: "Action"
    ELEVENLABS_GET_A_PROFILE_PAGE_PROFILE_HANDLE_GET: "Action"
    ELEVENLABS_GET_CHAPTERS_V_1_PROJECTS_PROJECT_ID_CHAPTERS_GET: "Action"
    ELEVENLABS_GET_CHAPTER_BY_ID_V_1_PROJECTS_PROJECT_ID_CHAPTERS_CHAPTER_ID_GET: "Action"
    ELEVENLABS_GET_CHAPTER_SNAPSHOTS_V_1_PROJECTS_PROJECT_ID_CHAPTERS_CHAPTER_ID_SNAPSHOTS_GET: "Action"
    ELEVENLABS_GET_DEFAULT_VOICE_SETTINGS_V_1_VOICES_SETTINGS_DEFAULT_GET: "Action"
    ELEVENLABS_GET_DUBBED_FILE_V_1_DUBBING_DUBBING_ID_AUDIO_LANGUAGE_CODE_GET: "Action"
    ELEVENLABS_GET_DUBBING_PROJECT_METADATA_V_1_DUBBING_DUBBING_ID_GET: "Action"
    ELEVENLABS_GET_GENERATED_ITEMS_V_1_HISTORY_GET: "Action"
    ELEVENLABS_GET_HISTORY_ITEM_BY_ID_V_1_HISTORY_HISTORY_ITEMID_GET: "Action"
    ELEVENLABS_GET_METADATA_FOR_A_PRONUNCIATION_DICTIONARY_V_1_PRONUNCIATION_DICTIONARIES_PRONUNCIATION_DICTIONARY_ID_GET: "Action"
    ELEVENLABS_GET_MODELS_V_1_MODELS_GET: "Action"
    ELEVENLABS_GET_PL_S_FILE_WITH_A_PRONUNCIATION_DICTIONARY_VERSION_RULES_V_1_PRONUNCIATION_DICTIONARIES_DICTIONARY_ID_VERSION_ID_DOWNLOAD_GET: "Action"
    ELEVENLABS_GET_PROJECTS_V_1_PROJECTS_GET: "Action"
    ELEVENLABS_GET_PROJECT_BY_ID_V_1_PROJECTS_PROJECT_ID_GET: "Action"
    ELEVENLABS_GET_PROJECT_SNAPSHOTS_V_1_PROJECTS_PROJECT_ID_SNAPSHOTS_GET: "Action"
    ELEVENLABS_GET_PRONUNCIATION_DICTIONARIES_V_1_PRONUNCIATION_DICTIONARIES_GET: "Action"
    ELEVENLABS_GET_TRANSCRIPT_FOR_DUB_V_1_DUBBING_DUBBING_ID_TRANSCRIPT_LANGUAGE_CODE_GET: "Action"
    ELEVENLABS_GET_USER_INFO_V_1_USER_GET: "Action"
    ELEVENLABS_GET_USER_SUBSCRIPTION_INFO_V_1_USER_SUBSCRIPTION_GET: "Action"
    ELEVENLABS_GET_VOICES_V_1_SHARED_VOICES_GET: "Action"
    ELEVENLABS_GET_VOICES_V_1_VOICES_GET: "Action"
    ELEVENLABS_GET_VOICE_SETTINGS_V_1_VOICES_VOICE_ID_SETTINGS_GET: "Action"
    ELEVENLABS_GET_VOICE_V_1_VOICES_VOICE_ID_GET: "Action"
    ELEVENLABS_REDIRECT_TO_MINT_LI_FY_DOCS_GET: "Action"
    ELEVENLABS_REMOVE_RULES_FROM_THE_PRONUNCIATION_DICTIONARY_V_1_PRONUNCIATION_DICTIONARIES_PRONUNCIATION_DICTIONARY_ID_REMOVE_RULES_POST: "Action"
    ELEVENLABS_SPEECH_TO_SPEECH_STREAMING_V_1_SPEECH_TO_SPEECH_VOICE_ID_STREAM_POST: "Action"
    ELEVENLABS_SPEECH_TO_SPEECH_V_1_SPEECH_TO_SPEECH_VOICE_ID_POST: "Action"
    ELEVENLABS_STREAMS_ARCHIVE_WITH_PROJECT_AUDIO_V_1_PROJECTS_PROJECT_ID_SNAPSHOTS_PROJECT_SNAPSHOT_ID_ARCHIVE_POST: "Action"
    ELEVENLABS_STREAM_CHAPTER_AUDIO_V_1_PROJECTS_PROJECT_ID_CHAPTERS_CHAPTER_ID_SNAPSHOTS_CHAPTER_SNAPSHOT_ID_STREAM_POST: "Action"
    ELEVENLABS_STREAM_PROJECT_AUDIO_V_1_PROJECTS_PROJECT_ID_SNAPSHOTS_PROJECT_SNAPSHOT_ID_STREAM_POST: "Action"
    ELEVENLABS_TEXT_TO_SPEECH_V_1_TEXT_TO_SPEECH_VOICE_ID_POST: "Action"
    ELEVENLABS_TEXT_TO_SPEECH_V_1_TEXT_TO_SPEECH_VOICE_ID_STREAM_POST: "Action"
    ELEVENLABS_UPDATE_PRONUNCIATION_DICTIONARIES_V_1_PROJECTS_PROJECT_ID_UPDATE_PRONUNCIATION_DICTIONARIES_POST: "Action"
    ELEVENLABS_VOICE_GENERATION_PARAMETERS_V_1_VOICE_GENERATION_GENERATE_VOICE_PARAMETERS_GET: "Action"
    EXA_SEARCH: "Action"
    EXA_SIMILARLINK: "Action"
    FIGMA_DELETED_EV_RESOURCE: "Action"
    FIGMA_DELETE_COMMENT: "Action"
    FIGMA_DELETE_COMMENT_REACTION: "Action"
    FIGMA_DELETE_WEB_HOOK: "Action"
    FIGMA_GET_ACTIVITY_LOGS: "Action"
    FIGMA_GET_COMMENTS: "Action"
    FIGMA_GET_COMMENT_REACTIONS: "Action"
    FIGMA_GET_COMPONENT: "Action"
    FIGMA_GET_COMPONENT_SET: "Action"
    FIGMA_GET_DEV_RESOURCES: "Action"
    FIGMA_GET_FILE: "Action"
    FIGMA_GET_FILE_COMPONENTS: "Action"
    FIGMA_GET_FILE_COMPONENT_SETS: "Action"
    FIGMA_GET_FILE_NODES: "Action"
    FIGMA_GET_FILE_STYLES: "Action"
    FIGMA_GET_FILE_VERSIONS: "Action"
    FIGMA_GET_IMAGES: "Action"
    FIGMA_GET_IMAGE_FILLS: "Action"
    FIGMA_GET_LOCAL_VARIABLES: "Action"
    FIGMA_GET_ME: "Action"
    FIGMA_GET_PAYMENTS: "Action"
    FIGMA_GET_PROJECT_FILES: "Action"
    FIGMA_GET_PUBLISHED_VARIABLES: "Action"
    FIGMA_GET_STYLE: "Action"
    FIGMA_GET_TEAM_COMPONENTS: "Action"
    FIGMA_GET_TEAM_COMPONENT_SETS: "Action"
    FIGMA_GET_TEAM_PROJECTS: "Action"
    FIGMA_GET_TEAM_STYLES: "Action"
    FIGMA_GET_TEAM_WEB_HOOKS: "Action"
    FIGMA_GET_WEB_HOOK: "Action"
    FIGMA_GET_WEB_HOOK_REQUESTS: "Action"
    FIGMA_POST_COMMENT: "Action"
    FIGMA_POST_COMMENT_REACTION: "Action"
    FIGMA_POST_DEV_RESOURCES: "Action"
    FIGMA_POST_VARIABLES: "Action"
    FIGMA_POST_WEB_HOOK: "Action"
    FIGMA_PUT_DEV_RESOURCES: "Action"
    FIGMA_PUT_WEB_HOOK: "Action"
    FILEEDITTOOL_CREATE_FILE_CMD: "Action"
    FILEEDITTOOL_EDIT_FILE: "Action"
    FILEEDITTOOL_OPEN_FILE: "Action"
    FILEEDITTOOL_SCROLL: "Action"
    FILEMANAGER_CLOSE_SHELL_ACTION: "Action"
    FILEMANAGER_CREATE_FILE_ACTION: "Action"
    FILEMANAGER_CREATE_SHELL_ACTION: "Action"
    FILEMANAGER_EDIT_FILE_ACTION: "Action"
    FILEMANAGER_GOTO_LINE_ACTION: "Action"
    FILEMANAGER_OPEN_FILE_ACTION: "Action"
    FILEMANAGER_RUN_COMMAND_ACTION: "Action"
    FILEMANAGER_SCROLL_ACTION: "Action"
    FILEMANAGER_SET_ENV_VAR_ACTION: "Action"
    FILETOOL_READ_FILE: "Action"
    FILETOOL_WRITE_FILE: "Action"
    FIRECRAWL_CHECK_CRAWL_STATUS: "Action"
    FIRECRAWL_CRAWL: "Action"
    FIRECRAWL_EXTRACT: "Action"
    FIRECRAWL_SCRAPE: "Action"
    FIRECRAWL_SEARCH: "Action"
    GITCMDTOOL_GET_PATCH_CMD: "Action"
    GITCMDTOOL_GITHUB_CLONE_CMD: "Action"
    GITCMDTOOL_GIT_REPO_TREE: "Action"
    GITHUB_ACTIONS_ADD_CUSTOM_LABELS_TO_SELF_HOSTED_RUNNER_FOR_ORG: "Action"
    GITHUB_ACTIONS_ADD_CUSTOM_LABELS_TO_SELF_HOSTED_RUNNER_FOR_REPO: "Action"
    GITHUB_ACTIONS_ADD_SELECTED_REPO_TO_ORG_SECRET: "Action"
    GITHUB_ACTIONS_ADD_SELECTED_REPO_TO_ORG_VARIABLE: "Action"
    GITHUB_ACTIONS_APPROVE_WORK_FLOW_RUN: "Action"
    GITHUB_ACTIONS_CANCEL_WORK_FLOW_RUN: "Action"
    GITHUB_ACTIONS_CREATE_ENVIRONMENT_VARIABLE: "Action"
    GITHUB_ACTIONS_CREATE_ORG_VARIABLE: "Action"
    GITHUB_ACTIONS_CREATE_OR_UPDATE_ENVIRONMENT_SECRET: "Action"
    GITHUB_ACTIONS_CREATE_OR_UPDATE_ORG_SECRET: "Action"
    GITHUB_ACTIONS_CREATE_OR_UPDATE_REPO_SECRET: "Action"
    GITHUB_ACTIONS_CREATE_REGISTRATION_TOKEN_FOR_ORG: "Action"
    GITHUB_ACTIONS_CREATE_REGISTRATION_TOKEN_FOR_REPO: "Action"
    GITHUB_ACTIONS_CREATE_REMOVE_TOKEN_FOR_ORG: "Action"
    GITHUB_ACTIONS_CREATE_REMOVE_TOKEN_FOR_REPO: "Action"
    GITHUB_ACTIONS_CREATE_REPO_VARIABLE: "Action"
    GITHUB_ACTIONS_CREATE_WORK_FLOW_DISPATCH: "Action"
    GITHUB_ACTIONS_DELETE_ACTIONS_CACHE_BY_ID: "Action"
    GITHUB_ACTIONS_DELETE_ACTIONS_CACHE_BY_KEY: "Action"
    GITHUB_ACTIONS_DELETE_ARTIFACT: "Action"
    GITHUB_ACTIONS_DELETE_ENVIRONMENT_SECRET: "Action"
    GITHUB_ACTIONS_DELETE_ENVIRONMENT_VARIABLE: "Action"
    GITHUB_ACTIONS_DELETE_ORG_SECRET: "Action"
    GITHUB_ACTIONS_DELETE_ORG_VARIABLE: "Action"
    GITHUB_ACTIONS_DELETE_REPO_SECRET: "Action"
    GITHUB_ACTIONS_DELETE_REPO_VARIABLE: "Action"
    GITHUB_ACTIONS_DELETE_SELF_HOSTED_RUNNER_FROM_ORG: "Action"
    GITHUB_ACTIONS_DELETE_SELF_HOSTED_RUNNER_FROM_REPO: "Action"
    GITHUB_ACTIONS_DELETE_WORK_FLOW_RUN: "Action"
    GITHUB_ACTIONS_DELETE_WORK_FLOW_RUN_LOGS: "Action"
    GITHUB_ACTIONS_DISABLE_SELECTED_REPOSITORY_G_IT_HUB_ACTIONS_ORGANIZATION: "Action"
    GITHUB_ACTIONS_DISABLE_WORK_FLOW: "Action"
    GITHUB_ACTIONS_DOWNLOAD_ARTIFACT: "Action"
    GITHUB_ACTIONS_DOWNLOAD_JOB_LOGS_FOR_WORK_FLOW_RUN: "Action"
    GITHUB_ACTIONS_DOWNLOAD_WORK_FLOW_RUN_ATTEMPT_LOGS: "Action"
    GITHUB_ACTIONS_DOWNLOAD_WORK_FLOW_RUN_LOGS: "Action"
    GITHUB_ACTIONS_ENABLES_ELECTED_REPOSITORY_G_IT_HUB_ACTIONS_ORGANIZATION: "Action"
    GITHUB_ACTIONS_ENABLE_WORK_FLOW: "Action"
    GITHUB_ACTIONS_FORCE_CANCEL_WORK_FLOW_RUN: "Action"
    GITHUB_ACTIONS_GENERATE_RUNNER_J_IT_CONFIG_FOR_ORG: "Action"
    GITHUB_ACTIONS_GENERATE_RUNNER_J_IT_CONFIG_FOR_REPO: "Action"
    GITHUB_ACTIONS_GET_ACTIONS_CACHE_LIST: "Action"
    GITHUB_ACTIONS_GET_ACTIONS_CACHE_USAGE: "Action"
    GITHUB_ACTIONS_GET_ACTIONS_CACHE_USAGE_BY_REPO_FOR_ORG: "Action"
    GITHUB_ACTIONS_GET_ACTIONS_CACHE_USAGE_FOR_ORG: "Action"
    GITHUB_ACTIONS_GET_ALLOWED_ACTIONS_ORGANIZATION: "Action"
    GITHUB_ACTIONS_GET_ALLOWED_ACTIONS_REPOSITORY: "Action"
    GITHUB_ACTIONS_GET_ARTIFACT: "Action"
    GITHUB_ACTIONS_GET_CUSTOM_OID_C_SUB_CLAIM_FOR_REPO: "Action"
    GITHUB_ACTIONS_GET_ENVIRONMENT_PUBLIC_KEY: "Action"
    GITHUB_ACTIONS_GET_ENVIRONMENT_SECRET: "Action"
    GITHUB_ACTIONS_GET_ENVIRONMENT_VARIABLE: "Action"
    GITHUB_ACTIONS_GET_G_IT_HUB_ACTIONS_DEFAULT_WORK_FLOW_PERMISSIONS_ORGANIZATION: "Action"
    GITHUB_ACTIONS_GET_G_IT_HUB_ACTIONS_DEFAULT_WORK_FLOW_PERMISSIONS_REPOSITORY: "Action"
    GITHUB_ACTIONS_GET_G_IT_HUB_ACTIONS_PERMISSIONS_ORGANIZATION: "Action"
    GITHUB_ACTIONS_GET_G_IT_HUB_ACTIONS_PERMISSIONS_REPOSITORY: "Action"
    GITHUB_ACTIONS_GET_JOB_FOR_WORK_FLOW_RUN: "Action"
    GITHUB_ACTIONS_GET_ORG_PUBLIC_KEY: "Action"
    GITHUB_ACTIONS_GET_ORG_SECRET: "Action"
    GITHUB_ACTIONS_GET_ORG_VARIABLE: "Action"
    GITHUB_ACTIONS_GET_PENDING_DEPLOYMENTS_FOR_RUN: "Action"
    GITHUB_ACTIONS_GET_REPO_PUBLIC_KEY: "Action"
    GITHUB_ACTIONS_GET_REPO_SECRET: "Action"
    GITHUB_ACTIONS_GET_REPO_VARIABLE: "Action"
    GITHUB_ACTIONS_GET_REVIEWS_FOR_RUN: "Action"
    GITHUB_ACTIONS_GET_SELF_HOSTED_RUNNER_FOR_ORG: "Action"
    GITHUB_ACTIONS_GET_SELF_HOSTED_RUNNER_FOR_REPO: "Action"
    GITHUB_ACTIONS_GET_WORK_FLOW: "Action"
    GITHUB_ACTIONS_GET_WORK_FLOW_ACCESS_TO_REPOSITORY: "Action"
    GITHUB_ACTIONS_GET_WORK_FLOW_RUN: "Action"
    GITHUB_ACTIONS_GET_WORK_FLOW_RUN_ATTEMPT: "Action"
    GITHUB_ACTIONS_GET_WORK_FLOW_RUN_USAGE: "Action"
    GITHUB_ACTIONS_GET_WORK_FLOW_USAGE: "Action"
    GITHUB_ACTIONS_LIST_ARTIFACTS_FOR_REPO: "Action"
    GITHUB_ACTIONS_LIST_ENVIRONMENT_SECRETS: "Action"
    GITHUB_ACTIONS_LIST_ENVIRONMENT_VARIABLES: "Action"
    GITHUB_ACTIONS_LIST_JOBS_FOR_WORK_FLOW_RUN: "Action"
    GITHUB_ACTIONS_LIST_JOBS_FOR_WORK_FLOW_RUN_ATTEMPT: "Action"
    GITHUB_ACTIONS_LIST_LABELS_FOR_SELF_HOSTED_RUNNER_FOR_ORG: "Action"
    GITHUB_ACTIONS_LIST_LABELS_FOR_SELF_HOSTED_RUNNER_FOR_REPO: "Action"
    GITHUB_ACTIONS_LIST_ORG_SECRETS: "Action"
    GITHUB_ACTIONS_LIST_ORG_VARIABLES: "Action"
    GITHUB_ACTIONS_LIST_REPO_ORGANIZATION_SECRETS: "Action"
    GITHUB_ACTIONS_LIST_REPO_ORGANIZATION_VARIABLES: "Action"
    GITHUB_ACTIONS_LIST_REPO_SECRETS: "Action"
    GITHUB_ACTIONS_LIST_REPO_VARIABLES: "Action"
    GITHUB_ACTIONS_LIST_REPO_WORK_FLOWS: "Action"
    GITHUB_ACTIONS_LIST_RUNNER_APPLICATIONS_FOR_ORG: "Action"
    GITHUB_ACTIONS_LIST_RUNNER_APPLICATIONS_FOR_REPO: "Action"
    GITHUB_ACTIONS_LIST_SELECTED_REPOSITORIES_ENABLED_G_IT_HUB_ACTIONS_ORGANIZATION: "Action"
    GITHUB_ACTIONS_LIST_SELECTED_REPO_S_FOR_ORG_SECRET: "Action"
    GITHUB_ACTIONS_LIST_SELECTED_REPO_S_FOR_ORG_VARIABLE: "Action"
    GITHUB_ACTIONS_LIST_SELF_HOSTED_RUNNERS_FOR_ORG: "Action"
    GITHUB_ACTIONS_LIST_SELF_HOSTED_RUNNERS_FOR_REPO: "Action"
    GITHUB_ACTIONS_LIST_WORK_FLOW_RUNS: "Action"
    GITHUB_ACTIONS_LIST_WORK_FLOW_RUNS_FOR_REPO: "Action"
    GITHUB_ACTIONS_LIST_WORK_FLOW_RUN_ARTIFACTS: "Action"
    GITHUB_ACTIONS_REMOVE_ALL_CUSTOM_LABELS_FROM_SELF_HOSTED_RUNNER_FOR_ORG: "Action"
    GITHUB_ACTIONS_REMOVE_ALL_CUSTOM_LABELS_FROM_SELF_HOSTED_RUNNER_FOR_REPO: "Action"
    GITHUB_ACTIONS_REMOVE_CUSTOM_LABEL_FROM_SELF_HOSTED_RUNNER_FOR_ORG: "Action"
    GITHUB_ACTIONS_REMOVE_CUSTOM_LABEL_FROM_SELF_HOSTED_RUNNER_FOR_REPO: "Action"
    GITHUB_ACTIONS_REMOVE_SELECTED_REPO_FROM_ORG_SECRET: "Action"
    GITHUB_ACTIONS_REMOVE_SELECTED_REPO_FROM_ORG_VARIABLE: "Action"
    GITHUB_ACTIONS_RERUN_JOB_FOR_WORK_FLOW_RUN: "Action"
    GITHUB_ACTIONS_RERUN_WORK_FLOW: "Action"
    GITHUB_ACTIONS_RERUN_WORK_FLOW_FAILED_JOBS: "Action"
    GITHUB_ACTIONS_REVIEW_CUSTOM_GATES_FOR_RUN: "Action"
    GITHUB_ACTIONS_REVIEW_PENDING_DEPLOYMENTS_FOR_RUN: "Action"
    GITHUB_ACTIONS_SET_ALLOWED_ACTIONS_ORGANIZATION: "Action"
    GITHUB_ACTIONS_SET_ALLOWED_ACTIONS_REPOSITORY: "Action"
    GITHUB_ACTIONS_SET_CUSTOM_LABELS_FOR_SELF_HOSTED_RUNNER_FOR_ORG: "Action"
    GITHUB_ACTIONS_SET_CUSTOM_LABELS_FOR_SELF_HOSTED_RUNNER_FOR_REPO: "Action"
    GITHUB_ACTIONS_SET_CUSTOM_OID_C_SUB_CLAIM_FOR_REPO: "Action"
    GITHUB_ACTIONS_SET_G_IT_HUB_ACTIONS_DEFAULT_WORK_FLOW_PERMISSIONS_ORGANIZATION: "Action"
    GITHUB_ACTIONS_SET_G_IT_HUB_ACTIONS_DEFAULT_WORK_FLOW_PERMISSIONS_REPOSITORY: "Action"
    GITHUB_ACTIONS_SET_G_IT_HUB_ACTIONS_PERMISSIONS_ORGANIZATION: "Action"
    GITHUB_ACTIONS_SET_G_IT_HUB_ACTIONS_PERMISSIONS_REPOSITORY: "Action"
    GITHUB_ACTIONS_SET_SELECTED_REPOSITORIES_ENABLED_G_IT_HUB_ACTIONS_ORGANIZATION: "Action"
    GITHUB_ACTIONS_SET_SELECTED_REPO_S_FOR_ORG_SECRET: "Action"
    GITHUB_ACTIONS_SET_SELECTED_REPO_S_FOR_ORG_VARIABLE: "Action"
    GITHUB_ACTIONS_SET_WORK_FLOW_ACCESS_TO_REPOSITORY: "Action"
    GITHUB_ACTIONS_UPDATE_ENVIRONMENT_VARIABLE: "Action"
    GITHUB_ACTIONS_UPDATE_ORG_VARIABLE: "Action"
    GITHUB_ACTIONS_UPDATE_REPO_VARIABLE: "Action"
    GITHUB_ACTIVITY_CHECK_REPO_IS_STARRED_BY_AUTHENTICATED_USER: "Action"
    GITHUB_ACTIVITY_DELETE_REPO_SUBSCRIPTION: "Action"
    GITHUB_ACTIVITY_DELETE_THREAD_SUBSCRIPTION: "Action"
    GITHUB_ACTIVITY_GET_FEEDS: "Action"
    GITHUB_ACTIVITY_GET_REPO_SUBSCRIPTION: "Action"
    GITHUB_ACTIVITY_GET_THREAD: "Action"
    GITHUB_ACTIVITY_GET_THREAD_SUBSCRIPTION_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ACTIVITY_LIST_EVENTS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ACTIVITY_LIST_NOTIFICATIONS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ACTIVITY_LIST_ORG_EVENTS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ACTIVITY_LIST_PUBLIC_EVENTS: "Action"
    GITHUB_ACTIVITY_LIST_PUBLIC_EVENTS_FOR_REPO_NETWORK: "Action"
    GITHUB_ACTIVITY_LIST_PUBLIC_EVENTS_FOR_USER: "Action"
    GITHUB_ACTIVITY_LIST_PUBLIC_ORG_EVENTS: "Action"
    GITHUB_ACTIVITY_LIST_RECEIVED_EVENTS_FOR_USER: "Action"
    GITHUB_ACTIVITY_LIST_RECEIVED_PUBLIC_EVENTS_FOR_USER: "Action"
    GITHUB_ACTIVITY_LIST_REPO_EVENTS: "Action"
    GITHUB_ACTIVITY_LIST_REPO_NOTIFICATIONS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ACTIVITY_LIST_REPO_S_STARRED_BY_AUTHENTICATED_USER: "Action"
    GITHUB_ACTIVITY_LIST_REPO_S_STARRED_BY_USER: "Action"
    GITHUB_ACTIVITY_LIST_REPO_S_WATCHED_BY_USER: "Action"
    GITHUB_ACTIVITY_LIST_STARGAZERS_FOR_REPO: "Action"
    GITHUB_ACTIVITY_LIST_WATCHED_REPO_S_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ACTIVITY_LIST_WATCHERS_FOR_REPO: "Action"
    GITHUB_ACTIVITY_MARK_NOTIFICATIONS_AS_READ: "Action"
    GITHUB_ACTIVITY_MARK_REPO_NOTIFICATIONS_AS_READ: "Action"
    GITHUB_ACTIVITY_MARK_THREAD_AS_DONE: "Action"
    GITHUB_ACTIVITY_MARK_THREAD_AS_READ: "Action"
    GITHUB_ACTIVITY_SET_REPO_SUBSCRIPTION: "Action"
    GITHUB_ACTIVITY_SET_THREAD_SUBSCRIPTION: "Action"
    GITHUB_ACTIVITY_STAR_REPO_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ACTIVITY_UN_STAR_REPO_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_APPS_ADD_REPO_TO_INSTALLATION_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_APPS_CHECK_TOKEN: "Action"
    GITHUB_APPS_CREATE_FROM_MANIFEST: "Action"
    GITHUB_APPS_CREATE_INSTALLATION_ACCESS_TOKEN: "Action"
    GITHUB_APPS_DELETE_AUTHORIZATION: "Action"
    GITHUB_APPS_DELETE_INSTALLATION: "Action"
    GITHUB_APPS_DELETE_TOKEN: "Action"
    GITHUB_APPS_GET_AUTHENTICATED: "Action"
    GITHUB_APPS_GET_BY_SLUG: "Action"
    GITHUB_APPS_GET_INSTALLATION: "Action"
    GITHUB_APPS_GET_ORG_INSTALLATION: "Action"
    GITHUB_APPS_GET_REPO_INSTALLATION: "Action"
    GITHUB_APPS_GET_SUBSCRIPTION_PLAN_FOR_ACCOUNT: "Action"
    GITHUB_APPS_GET_SUBSCRIPTION_PLAN_FOR_ACCOUNT_STUBBED: "Action"
    GITHUB_APPS_GET_USER_INSTALLATION: "Action"
    GITHUB_APPS_GET_WEB_HOOK_CONFIG_FOR_APP: "Action"
    GITHUB_APPS_GET_WEB_HOOK_DELIVERY: "Action"
    GITHUB_APPS_LIST_ACCOUNTS_FOR_PLAN: "Action"
    GITHUB_APPS_LIST_ACCOUNTS_FOR_PLAN_STUBBED: "Action"
    GITHUB_APPS_LIST_INSTALLATIONS: "Action"
    GITHUB_APPS_LIST_INSTALLATIONS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_APPS_LIST_INSTALLATION_REPO_S_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_APPS_LIST_INSTALLATION_REQUESTS_FOR_AUTHENTICATED_APP: "Action"
    GITHUB_APPS_LIST_PLANS: "Action"
    GITHUB_APPS_LIST_PLANS_STUBBED: "Action"
    GITHUB_APPS_LIST_REPO_S_ACCESSIBLE_TO_INSTALLATION: "Action"
    GITHUB_APPS_LIST_SUBSCRIPTIONS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_APPS_LIST_SUBSCRIPTIONS_FOR_AUTHENTICATED_USER_STUBBED: "Action"
    GITHUB_APPS_LIST_WEB_HOOK_DELIVERIES: "Action"
    GITHUB_APPS_REMOVE_REPO_FROM_INSTALLATION_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_APPS_RESET_TOKEN: "Action"
    GITHUB_APPS_REVOKE_INSTALLATION_ACCESS_TOKEN: "Action"
    GITHUB_APPS_RE_DELIVER_WEB_HOOK_DELIVERY: "Action"
    GITHUB_APPS_SCOPE_TOKEN: "Action"
    GITHUB_APPS_SUSPEND_INSTALLATION: "Action"
    GITHUB_APPS_UPDATE_WEB_HOOK_CONFIG_FOR_APP: "Action"
    GITHUB_APP_SUN_SUSPEND_INSTALLATION: "Action"
    GITHUB_BILLING_GET_G_IT_HUB_ACTIONS_BILLING_ORG: "Action"
    GITHUB_BILLING_GET_G_IT_HUB_ACTIONS_BILLING_USER: "Action"
    GITHUB_BILLING_GET_G_IT_HUB_PACKAGES_BILLING_ORG: "Action"
    GITHUB_BILLING_GET_G_IT_HUB_PACKAGES_BILLING_USER: "Action"
    GITHUB_BILLING_GET_SHARED_STORAGE_BILLING_ORG: "Action"
    GITHUB_BILLING_GET_SHARED_STORAGE_BILLING_USER: "Action"
    GITHUB_CHECKS_CREATE: "Action"
    GITHUB_CHECKS_CREATE_SUITE: "Action"
    GITHUB_CHECKS_GET: "Action"
    GITHUB_CHECKS_GET_SUITE: "Action"
    GITHUB_CHECKS_LIST_ANNOTATIONS: "Action"
    GITHUB_CHECKS_LIST_FOR_REF: "Action"
    GITHUB_CHECKS_LIST_FOR_SUITE: "Action"
    GITHUB_CHECKS_LIST_SUITES_FOR_REF: "Action"
    GITHUB_CHECKS_RE_REQUEST_RUN: "Action"
    GITHUB_CHECKS_RE_REQUEST_SUITE: "Action"
    GITHUB_CHECKS_SET_SUITES_PREFERENCES: "Action"
    GITHUB_CHECKS_UPDATE: "Action"
    GITHUB_CLASSROOM_GET_AN_ASSIGNMENT: "Action"
    GITHUB_CLASSROOM_GET_ASSIGNMENT_GRADES: "Action"
    GITHUB_CLASSROOM_GET_A_CLASSROOM: "Action"
    GITHUB_CLASSROOM_LIST_ACCEPTED_AS_SIG_MENT_S_FOR_AN_ASSIGNMENT: "Action"
    GITHUB_CLASSROOM_LIST_ASSIGNMENTS_FOR_A_CLASSROOM: "Action"
    GITHUB_CLASSROOM_LIST_CLASSROOMS: "Action"
    GITHUB_CODES_OF_CONDUCT_GET_ALL_CODES_OF_CONDUCT: "Action"
    GITHUB_CODES_OF_CONDUCT_GET_CONDUCT_CODE: "Action"
    GITHUB_CODE_SCANNING_DELETE_ANALYSIS: "Action"
    GITHUB_CODE_SCANNING_GETS_ARIF: "Action"
    GITHUB_CODE_SCANNING_GET_ALERT: "Action"
    GITHUB_CODE_SCANNING_GET_ANALYSIS: "Action"
    GITHUB_CODE_SCANNING_GET_CODE_QL_DATABASE: "Action"
    GITHUB_CODE_SCANNING_GET_DEFAULT_SETUP: "Action"
    GITHUB_CODE_SCANNING_LIST_ALERTS_FOR_ORG: "Action"
    GITHUB_CODE_SCANNING_LIST_ALERTS_FOR_REPO: "Action"
    GITHUB_CODE_SCANNING_LIST_ALERT_INSTANCES: "Action"
    GITHUB_CODE_SCANNING_LIST_CODE_QL_DATABASES: "Action"
    GITHUB_CODE_SCANNING_LIST_RECENT_ANALYSES: "Action"
    GITHUB_CODE_SCANNING_UPDATE_ALERT: "Action"
    GITHUB_CODE_SCANNING_UPDATE_DEFAULT_SETUP: "Action"
    GITHUB_CODE_SCANNING_UPLOADS_ARIF: "Action"
    GITHUB_CODE_SPACES_ADD_REPOSITORY_FOR_SECRET_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_ADD_SELECTED_REPO_TO_ORG_SECRET: "Action"
    GITHUB_CODE_SPACES_CHECK_PERMISSIONS_FOR_DEV_CONTAINER: "Action"
    GITHUB_CODE_SPACES_CODE_SPACE_MACHINES_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_CREATE_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_CREATE_OR_UPDATE_ORG_SECRET: "Action"
    GITHUB_CODE_SPACES_CREATE_OR_UPDATE_REPO_SECRET: "Action"
    GITHUB_CODE_SPACES_CREATE_OR_UPDATE_SECRET_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_CREATE_WITH_PR_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_CREATE_WITH_REPO_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_DELETE_CODE_SPACES_ACCESS_USERS: "Action"
    GITHUB_CODE_SPACES_DELETE_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_DELETE_FROM_ORGANIZATION: "Action"
    GITHUB_CODE_SPACES_DELETE_ORG_SECRET: "Action"
    GITHUB_CODE_SPACES_DELETE_REPO_SECRET: "Action"
    GITHUB_CODE_SPACES_DELETE_SECRET_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_EXPORT_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_GET_CODE_SPACES_FOR_USER_IN_ORG: "Action"
    GITHUB_CODE_SPACES_GET_EXPORT_DETAILS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_GET_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_GET_ORG_PUBLIC_KEY: "Action"
    GITHUB_CODE_SPACES_GET_ORG_SECRET: "Action"
    GITHUB_CODE_SPACES_GET_PUBLIC_KEY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_GET_REPO_PUBLIC_KEY: "Action"
    GITHUB_CODE_SPACES_GET_REPO_SECRET: "Action"
    GITHUB_CODE_SPACES_GET_SECRET_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_LIST_DEV_CONTAINERS_IN_REPOSITORY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_LIST_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_LIST_IN_ORGANIZATION: "Action"
    GITHUB_CODE_SPACES_LIST_IN_REPOSITORY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_LIST_ORG_SECRETS: "Action"
    GITHUB_CODE_SPACES_LIST_REPOSITORIES_FOR_SECRET_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_LIST_REPO_SECRETS: "Action"
    GITHUB_CODE_SPACES_LIST_SECRETS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_LIST_SELECTED_REPO_S_FOR_ORG_SECRET: "Action"
    GITHUB_CODE_SPACES_PRE_FLIGHT_WITH_REPO_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_PUBLISH_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_REMOVE_REPOSITORY_FOR_SECRET_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_REMOVE_SELECTED_REPO_FROM_ORG_SECRET: "Action"
    GITHUB_CODE_SPACES_REPO_MACHINES_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_SET_CODE_SPACES_ACCESS: "Action"
    GITHUB_CODE_SPACES_SET_CODE_SPACES_ACCESS_USERS: "Action"
    GITHUB_CODE_SPACES_SET_REPOSITORIES_FOR_SECRET_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_SET_SELECTED_REPO_S_FOR_ORG_SECRET: "Action"
    GITHUB_CODE_SPACES_START_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_STOP_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_CODE_SPACES_STOP_IN_ORGANIZATION: "Action"
    GITHUB_CODE_SPACES_UPDATE_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_COPILOT_ADD_COPILOT_SEATS_FOR_TEAMS: "Action"
    GITHUB_COPILOT_ADD_COPILOT_SEATS_FOR_USERS: "Action"
    GITHUB_COPILOT_CANCEL_COPILOTS_EAT_ASSIGNMENT_FOR_TEAMS: "Action"
    GITHUB_COPILOT_CANCEL_COPILOTS_EAT_ASSIGNMENT_FOR_USERS: "Action"
    GITHUB_COPILOT_GET_COPILOTS_EAT_DETAILS_FOR_USER: "Action"
    GITHUB_COPILOT_GET_COPILOT_ORGANIZATION_DETAILS: "Action"
    GITHUB_COPILOT_LIST_COPILOT_SEATS: "Action"
    GITHUB_DEPENDENCY_GRAPH_CREATE_REPOSITORY_SNAPSHOT: "Action"
    GITHUB_DEPENDENCY_GRAPH_DIFF_RANGE: "Action"
    GITHUB_DEPENDENCY_GRAPH_EXPORTS_BOM: "Action"
    GITHUB_DEPEND_A_BOT_ADD_SELECTED_REPO_TO_ORG_SECRET: "Action"
    GITHUB_DEPEND_A_BOT_CREATE_OR_UPDATE_ORG_SECRET: "Action"
    GITHUB_DEPEND_A_BOT_CREATE_OR_UPDATE_REPO_SECRET: "Action"
    GITHUB_DEPEND_A_BOT_DELETE_ORG_SECRET: "Action"
    GITHUB_DEPEND_A_BOT_DELETE_REPO_SECRET: "Action"
    GITHUB_DEPEND_A_BOT_GET_ALERT: "Action"
    GITHUB_DEPEND_A_BOT_GET_ORG_PUBLIC_KEY: "Action"
    GITHUB_DEPEND_A_BOT_GET_ORG_SECRET: "Action"
    GITHUB_DEPEND_A_BOT_GET_REPO_PUBLIC_KEY: "Action"
    GITHUB_DEPEND_A_BOT_GET_REPO_SECRET: "Action"
    GITHUB_DEPEND_A_BOT_LIST_ALERTS_FOR_ENTERPRISE: "Action"
    GITHUB_DEPEND_A_BOT_LIST_ALERTS_FOR_ORG: "Action"
    GITHUB_DEPEND_A_BOT_LIST_ALERTS_FOR_REPO: "Action"
    GITHUB_DEPEND_A_BOT_LIST_ORG_SECRETS: "Action"
    GITHUB_DEPEND_A_BOT_LIST_REPO_SECRETS: "Action"
    GITHUB_DEPEND_A_BOT_LIST_SELECTED_REPO_S_FOR_ORG_SECRET: "Action"
    GITHUB_DEPEND_A_BOT_REMOVE_SELECTED_REPO_FROM_ORG_SECRET: "Action"
    GITHUB_DEPEND_A_BOT_SET_SELECTED_REPO_S_FOR_ORG_SECRET: "Action"
    GITHUB_DEPEND_A_BOT_UPDATE_ALERT: "Action"
    GITHUB_EMO_J_IS_GET: "Action"
    GITHUB_GET_CODE_CHANGES_IN_PR: "Action"
    GITHUB_GIST_SUNSTAR: "Action"
    GITHUB_GIST_S_CHECK_IS_STARRED: "Action"
    GITHUB_GIST_S_CREATE: "Action"
    GITHUB_GIST_S_CREATE_COMMENT: "Action"
    GITHUB_GIST_S_DELETE: "Action"
    GITHUB_GIST_S_DELETE_COMMENT: "Action"
    GITHUB_GIST_S_FORK: "Action"
    GITHUB_GIST_S_GET: "Action"
    GITHUB_GIST_S_GET_COMMENT: "Action"
    GITHUB_GIST_S_GET_REVISION: "Action"
    GITHUB_GIST_S_LIST: "Action"
    GITHUB_GIST_S_LIST_COMMENTS: "Action"
    GITHUB_GIST_S_LIST_COMMITS: "Action"
    GITHUB_GIST_S_LIST_FORKS: "Action"
    GITHUB_GIST_S_LIST_FOR_USER: "Action"
    GITHUB_GIST_S_LIST_PUBLIC: "Action"
    GITHUB_GIST_S_LIST_STARRED: "Action"
    GITHUB_GIST_S_STAR: "Action"
    GITHUB_GIST_S_UPDATE: "Action"
    GITHUB_GIST_S_UPDATE_COMMENT: "Action"
    GITHUB_G_IT_CREATE_BLOB: "Action"
    GITHUB_G_IT_CREATE_COMMIT: "Action"
    GITHUB_G_IT_CREATE_REF: "Action"
    GITHUB_G_IT_CREATE_TAG: "Action"
    GITHUB_G_IT_CREATE_TREE: "Action"
    GITHUB_G_IT_DELETE_REF: "Action"
    GITHUB_G_IT_GET_BLOB: "Action"
    GITHUB_G_IT_GET_COMMIT: "Action"
    GITHUB_G_IT_GET_REF: "Action"
    GITHUB_G_IT_GET_TAG: "Action"
    GITHUB_G_IT_GET_TREE: "Action"
    GITHUB_G_IT_IGNORE_GET_ALL_TEMPLATES: "Action"
    GITHUB_G_IT_IGNORE_GET_TEMPLATE: "Action"
    GITHUB_G_IT_LIST_MATCHING_REFS: "Action"
    GITHUB_G_IT_UPDATE_REF: "Action"
    GITHUB_INTERACTIONS_GET_RESTRICTIONS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_INTERACTIONS_GET_RESTRICTIONS_FOR_ORG: "Action"
    GITHUB_INTERACTIONS_GET_RESTRICTIONS_FOR_REPO: "Action"
    GITHUB_INTERACTIONS_REMOVE_RESTRICTIONS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_INTERACTIONS_REMOVE_RESTRICTIONS_FOR_ORG: "Action"
    GITHUB_INTERACTIONS_REMOVE_RESTRICTIONS_FOR_REPO: "Action"
    GITHUB_INTERACTIONS_SET_RESTRICTIONS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_INTERACTIONS_SET_RESTRICTIONS_FOR_ORG: "Action"
    GITHUB_INTERACTIONS_SET_RESTRICTIONS_FOR_REPO: "Action"
    GITHUB_ISSUES_ADD_ASSIGN_EES: "Action"
    GITHUB_ISSUES_ADD_LABELS: "Action"
    GITHUB_ISSUES_CHECKUSER_CAN_BE_ASSIGNED: "Action"
    GITHUB_ISSUES_CHECKUSER_CAN_BE_ASSIGNED_TO_ISSUE: "Action"
    GITHUB_ISSUES_CREATE: "Action"
    GITHUB_ISSUES_CREATE_COMMENT: "Action"
    GITHUB_ISSUES_CREATE_LABEL: "Action"
    GITHUB_ISSUES_CREATE_MILESTONE: "Action"
    GITHUB_ISSUES_DELETE_COMMENT: "Action"
    GITHUB_ISSUES_DELETE_LABEL: "Action"
    GITHUB_ISSUES_DELETE_MILESTONE: "Action"
    GITHUB_ISSUES_GET: "Action"
    GITHUB_ISSUES_GET_COMMENT: "Action"
    GITHUB_ISSUES_GET_EVENT: "Action"
    GITHUB_ISSUES_GET_LABEL: "Action"
    GITHUB_ISSUES_GET_MILESTONE: "Action"
    GITHUB_ISSUES_LIST: "Action"
    GITHUB_ISSUES_LIST_ASSIGN_EES: "Action"
    GITHUB_ISSUES_LIST_COMMENTS: "Action"
    GITHUB_ISSUES_LIST_COMMENTS_FOR_REPO: "Action"
    GITHUB_ISSUES_LIST_EVENTS: "Action"
    GITHUB_ISSUES_LIST_EVENTS_FOR_REPO: "Action"
    GITHUB_ISSUES_LIST_EVENTS_FOR_TIMELINE: "Action"
    GITHUB_ISSUES_LIST_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ISSUES_LIST_FOR_ORG: "Action"
    GITHUB_ISSUES_LIST_FOR_REPO: "Action"
    GITHUB_ISSUES_LIST_LABELS_FOR_MILESTONE: "Action"
    GITHUB_ISSUES_LIST_LABELS_FOR_REPO: "Action"
    GITHUB_ISSUES_LIST_LABELS_ON_ISSUE: "Action"
    GITHUB_ISSUES_LIST_MILESTONES: "Action"
    GITHUB_ISSUES_LOCK: "Action"
    GITHUB_ISSUES_REMOVE_ALL_LABELS: "Action"
    GITHUB_ISSUES_REMOVE_ASSIGN_EES: "Action"
    GITHUB_ISSUES_REMOVE_LABEL: "Action"
    GITHUB_ISSUES_SET_LABELS: "Action"
    GITHUB_ISSUES_UNLOCK: "Action"
    GITHUB_ISSUES_UPDATE: "Action"
    GITHUB_ISSUES_UPDATE_COMMENT: "Action"
    GITHUB_ISSUES_UPDATE_LABEL: "Action"
    GITHUB_ISSUES_UPDATE_MILESTONE: "Action"
    GITHUB_LICENSES_GET: "Action"
    GITHUB_LICENSES_GET_ALL_COMMONLY_USED: "Action"
    GITHUB_LICENSES_GET_FOR_REPO: "Action"
    GITHUB_MARKDOWN_RENDER: "Action"
    GITHUB_MARKDOWN_RENDER_RAW: "Action"
    GITHUB_META_GET: "Action"
    GITHUB_META_GET_ALL_VERSIONS: "Action"
    GITHUB_META_GET_OCTO_CAT: "Action"
    GITHUB_META_GET_ZEN: "Action"
    GITHUB_META_ROOT: "Action"
    GITHUB_MIGRATIONS_CANCEL_IMPORT: "Action"
    GITHUB_MIGRATIONS_DELETE_ARCHIVE_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_MIGRATIONS_DELETE_ARCHIVE_FOR_ORG: "Action"
    GITHUB_MIGRATIONS_DOWNLOAD_ARCHIVE_FOR_ORG: "Action"
    GITHUB_MIGRATIONS_GET_ARCHIVE_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_MIGRATIONS_GET_COMMIT_AUTHORS: "Action"
    GITHUB_MIGRATIONS_GET_IMPORT_STATUS: "Action"
    GITHUB_MIGRATIONS_GET_LARGE_FILES: "Action"
    GITHUB_MIGRATIONS_GET_STATUS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_MIGRATIONS_GET_STATUS_FOR_ORG: "Action"
    GITHUB_MIGRATIONS_LIST_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_MIGRATIONS_LIST_FOR_ORG: "Action"
    GITHUB_MIGRATIONS_LIST_REPO_S_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_MIGRATIONS_LIST_REPO_S_FOR_ORG: "Action"
    GITHUB_MIGRATIONS_MAP_COMMIT_AUTHOR: "Action"
    GITHUB_MIGRATIONS_SET_LF_S_PREFERENCE: "Action"
    GITHUB_MIGRATIONS_START_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_MIGRATIONS_START_FOR_ORG: "Action"
    GITHUB_MIGRATIONS_START_IMPORT: "Action"
    GITHUB_MIGRATIONS_UNLOCK_REPO_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_MIGRATIONS_UNLOCK_REPO_FOR_ORG: "Action"
    GITHUB_MIGRATIONS_UPDATE_IMPORT: "Action"
    GITHUB_OID_CGE_TO_I_DC_CUSTOM_SUBTEMPLATE_FOR_ORG: "Action"
    GITHUB_OID_CUP_DATE_OID_C_CUSTOM_SUBTEMPLATE_FOR_ORG: "Action"
    GITHUB_ORG_SADD_SECURITY_MANAGER_TEAM: "Action"
    GITHUB_ORG_SUNBLOCK_USER: "Action"
    GITHUB_ORG_S_ASSIGN_TEAM_TO_ORG_ROLE: "Action"
    GITHUB_ORG_S_ASSIGN_USER_TO_ORG_ROLE: "Action"
    GITHUB_ORG_S_BLOCK_USER: "Action"
    GITHUB_ORG_S_CANCEL_INVITATION: "Action"
    GITHUB_ORG_S_CHECK_BLOCKED_USER: "Action"
    GITHUB_ORG_S_CHECK_MEMBERSHIP_FOR_USER: "Action"
    GITHUB_ORG_S_CHECK_PUBLIC_MEMBERSHIP_FOR_USER: "Action"
    GITHUB_ORG_S_CONVERT_MEMBER_TO_OUTSIDE_COLLABORATOR: "Action"
    GITHUB_ORG_S_CREATE_CUSTOM_ORGANIZATION_ROLE: "Action"
    GITHUB_ORG_S_CREATE_INVITATION: "Action"
    GITHUB_ORG_S_CREATE_OR_UPDATE_CUSTOM_PROPERTIES: "Action"
    GITHUB_ORG_S_CREATE_OR_UPDATE_CUSTOM_PROPERTIES_VALUES_FOR_REPO_S: "Action"
    GITHUB_ORG_S_CREATE_OR_UPDATE_CUSTOM_PROPERTY: "Action"
    GITHUB_ORG_S_CREATE_WEB_HOOK: "Action"
    GITHUB_ORG_S_DELETE: "Action"
    GITHUB_ORG_S_DELETE_CUSTOM_ORGANIZATION_ROLE: "Action"
    GITHUB_ORG_S_DELETE_WEB_HOOK: "Action"
    GITHUB_ORG_S_ENABLE_OR_DISABLE_SECURITY_PRODUCT_ON_ALL_ORG_REPO_S: "Action"
    GITHUB_ORG_S_GET: "Action"
    GITHUB_ORG_S_GET_ALL_CUSTOM_PROPERTIES: "Action"
    GITHUB_ORG_S_GET_CUSTOM_PROPERTY: "Action"
    GITHUB_ORG_S_GET_MEMBERSHIP_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ORG_S_GET_MEMBERSHIP_FOR_USER: "Action"
    GITHUB_ORG_S_GET_ORG_ROLE: "Action"
    GITHUB_ORG_S_GET_WEB_HOOK: "Action"
    GITHUB_ORG_S_GET_WEB_HOOK_CONFIG_FOR_ORG: "Action"
    GITHUB_ORG_S_GET_WEB_HOOK_DELIVERY: "Action"
    GITHUB_ORG_S_LIST: "Action"
    GITHUB_ORG_S_LIST_APP_INSTALLATIONS: "Action"
    GITHUB_ORG_S_LIST_BLOCKED_USERS: "Action"
    GITHUB_ORG_S_LIST_CUSTOM_PROPERTIES_VALUES_FOR_REPO_S: "Action"
    GITHUB_ORG_S_LIST_FAILED_INVITATIONS: "Action"
    GITHUB_ORG_S_LIST_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ORG_S_LIST_FOR_USER: "Action"
    GITHUB_ORG_S_LIST_INVITATION_TEAMS: "Action"
    GITHUB_ORG_S_LIST_MEMBERS: "Action"
    GITHUB_ORG_S_LIST_MEMBERSHIPS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ORG_S_LIST_ORGANIZATION_FINE_GRAINED_PERMISSIONS: "Action"
    GITHUB_ORG_S_LIST_ORG_ROLES: "Action"
    GITHUB_ORG_S_LIST_ORG_ROLE_TEAMS: "Action"
    GITHUB_ORG_S_LIST_ORG_ROLE_USERS: "Action"
    GITHUB_ORG_S_LIST_OUTSIDE_COLLABORATORS: "Action"
    GITHUB_ORG_S_LIST_PAT_GRANTS: "Action"
    GITHUB_ORG_S_LIST_PAT_GRANT_REPOSITORIES: "Action"
    GITHUB_ORG_S_LIST_PAT_GRANT_REQUESTS: "Action"
    GITHUB_ORG_S_LIST_PAT_GRANT_REQUEST_REPOSITORIES: "Action"
    GITHUB_ORG_S_LIST_PENDING_INVITATIONS: "Action"
    GITHUB_ORG_S_LIST_PUBLIC_MEMBERS: "Action"
    GITHUB_ORG_S_LIST_SECURITY_MANAGER_TEAMS: "Action"
    GITHUB_ORG_S_LIST_WEB_HOOKS: "Action"
    GITHUB_ORG_S_LIST_WEB_HOOK_DELIVERIES: "Action"
    GITHUB_ORG_S_PATCH_CUSTOM_ORGANIZATION_ROLE: "Action"
    GITHUB_ORG_S_PING_WEB_HOOK: "Action"
    GITHUB_ORG_S_REMOVE_CUSTOM_PROPERTY: "Action"
    GITHUB_ORG_S_REMOVE_MEMBER: "Action"
    GITHUB_ORG_S_REMOVE_MEMBERSHIP_FOR_USER: "Action"
    GITHUB_ORG_S_REMOVE_OUTSIDE_COLLABORATOR: "Action"
    GITHUB_ORG_S_REMOVE_PUBLIC_MEMBERSHIP_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ORG_S_REMOVE_SECURITY_MANAGER_TEAM: "Action"
    GITHUB_ORG_S_REVIEW_PAT_GRANT_REQUEST: "Action"
    GITHUB_ORG_S_REVIEW_PAT_GRANT_REQUESTS_IN_BULK: "Action"
    GITHUB_ORG_S_REVOKE_ALL_ORG_ROLES_TEAM: "Action"
    GITHUB_ORG_S_REVOKE_ALL_ORG_ROLES_USER: "Action"
    GITHUB_ORG_S_REVOKE_ORG_ROLE_TEAM: "Action"
    GITHUB_ORG_S_REVOKE_ORG_ROLE_USER: "Action"
    GITHUB_ORG_S_RE_DELIVER_WEB_HOOK_DELIVERY: "Action"
    GITHUB_ORG_S_SET_MEMBERSHIP_FOR_USER: "Action"
    GITHUB_ORG_S_SET_PUBLIC_MEMBERSHIP_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ORG_S_UPDATE: "Action"
    GITHUB_ORG_S_UPDATE_MEMBERSHIP_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_ORG_S_UPDATE_PAT_ACCESS: "Action"
    GITHUB_ORG_S_UPDATE_PAT_ACCESSES: "Action"
    GITHUB_ORG_S_UPDATE_WEB_HOOK: "Action"
    GITHUB_ORG_S_UPDATE_WEB_HOOK_CONFIG_FOR_ORG: "Action"
    GITHUB_PACKAGES_DELETE_PACKAGE_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_PACKAGES_DELETE_PACKAGE_FOR_ORG: "Action"
    GITHUB_PACKAGES_DELETE_PACKAGE_FOR_USER: "Action"
    GITHUB_PACKAGES_DELETE_PACKAGE_VERSION_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_PACKAGES_DELETE_PACKAGE_VERSION_FOR_ORG: "Action"
    GITHUB_PACKAGES_DELETE_PACKAGE_VERSION_FOR_USER: "Action"
    GITHUB_PACKAGES_GET_ALL_PACKAGE_VERSIONS_FOR_PACKAGE_OWNED_BY_AUTHENTICATED_USER: "Action"
    GITHUB_PACKAGES_GET_ALL_PACKAGE_VERSIONS_FOR_PACKAGE_OWNED_BY_ORG: "Action"
    GITHUB_PACKAGES_GET_ALL_PACKAGE_VERSIONS_FOR_PACKAGE_OWNED_BY_USER: "Action"
    GITHUB_PACKAGES_GET_PACKAGE_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_PACKAGES_GET_PACKAGE_FOR_ORGANIZATION: "Action"
    GITHUB_PACKAGES_GET_PACKAGE_FOR_USER: "Action"
    GITHUB_PACKAGES_GET_PACKAGE_VERSION_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_PACKAGES_GET_PACKAGE_VERSION_FOR_ORGANIZATION: "Action"
    GITHUB_PACKAGES_GET_PACKAGE_VERSION_FOR_USER: "Action"
    GITHUB_PACKAGES_LIST_DOCKER_MIGRATION_CONFLICTING_PACKAGES_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_PACKAGES_LIST_DOCKER_MIGRATION_CONFLICTING_PACKAGES_FOR_ORGANIZATION: "Action"
    GITHUB_PACKAGES_LIST_DOCKER_MIGRATION_CONFLICTING_PACKAGES_FOR_USER: "Action"
    GITHUB_PACKAGES_LIST_PACKAGES_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_PACKAGES_LIST_PACKAGES_FOR_ORGANIZATION: "Action"
    GITHUB_PACKAGES_LIST_PACKAGES_FOR_USER: "Action"
    GITHUB_PACKAGES_RESTORE_PACKAGE_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_PACKAGES_RESTORE_PACKAGE_FOR_ORG: "Action"
    GITHUB_PACKAGES_RESTORE_PACKAGE_FOR_USER: "Action"
    GITHUB_PACKAGES_RESTORE_PACKAGE_VERSION_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_PACKAGES_RESTORE_PACKAGE_VERSION_FOR_ORG: "Action"
    GITHUB_PACKAGES_RESTORE_PACKAGE_VERSION_FOR_USER: "Action"
    GITHUB_PROJECTS_ADD_COLLABORATOR: "Action"
    GITHUB_PROJECTS_CREATE_CARD: "Action"
    GITHUB_PROJECTS_CREATE_COLUMN: "Action"
    GITHUB_PROJECTS_CREATE_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_PROJECTS_CREATE_FOR_ORG: "Action"
    GITHUB_PROJECTS_CREATE_FOR_REPO: "Action"
    GITHUB_PROJECTS_DELETE: "Action"
    GITHUB_PROJECTS_DELETE_CARD: "Action"
    GITHUB_PROJECTS_DELETE_COLUMN: "Action"
    GITHUB_PROJECTS_GET: "Action"
    GITHUB_PROJECTS_GET_CARD: "Action"
    GITHUB_PROJECTS_GET_COLUMN: "Action"
    GITHUB_PROJECTS_GET_PERMISSION_FOR_USER: "Action"
    GITHUB_PROJECTS_LIST_CARDS: "Action"
    GITHUB_PROJECTS_LIST_COLLABORATORS: "Action"
    GITHUB_PROJECTS_LIST_COLUMNS: "Action"
    GITHUB_PROJECTS_LIST_FOR_ORG: "Action"
    GITHUB_PROJECTS_LIST_FOR_REPO: "Action"
    GITHUB_PROJECTS_LIST_FOR_USER: "Action"
    GITHUB_PROJECTS_MOVE_CARD: "Action"
    GITHUB_PROJECTS_MOVE_COLUMN: "Action"
    GITHUB_PROJECTS_REMOVE_COLLABORATOR: "Action"
    GITHUB_PROJECTS_UPDATE: "Action"
    GITHUB_PROJECTS_UPDATE_CARD: "Action"
    GITHUB_PROJECTS_UPDATE_COLUMN: "Action"
    GITHUB_PULLS_CHECK_IF_MERGED: "Action"
    GITHUB_PULLS_CREATE: "Action"
    GITHUB_PULLS_CREATE_REPLY_FOR_REVIEW_COMMENT: "Action"
    GITHUB_PULLS_CREATE_REVIEW: "Action"
    GITHUB_PULLS_CREATE_REVIEW_COMMENT: "Action"
    GITHUB_PULLS_DELETE_PENDING_REVIEW: "Action"
    GITHUB_PULLS_DELETE_REVIEW_COMMENT: "Action"
    GITHUB_PULLS_DISMISS_REVIEW: "Action"
    GITHUB_PULLS_GET: "Action"
    GITHUB_PULLS_GET_REVIEW: "Action"
    GITHUB_PULLS_GET_REVIEW_COMMENT: "Action"
    GITHUB_PULLS_LIST: "Action"
    GITHUB_PULLS_LIST_COMMENTS_FOR_REVIEW: "Action"
    GITHUB_PULLS_LIST_COMMITS: "Action"
    GITHUB_PULLS_LIST_FILES: "Action"
    GITHUB_PULLS_LIST_REQUESTED_REVIEWERS: "Action"
    GITHUB_PULLS_LIST_REVIEWS: "Action"
    GITHUB_PULLS_LIST_REVIEW_COMMENTS: "Action"
    GITHUB_PULLS_LIST_REVIEW_COMMENTS_FOR_REPO: "Action"
    GITHUB_PULLS_MERGE: "Action"
    GITHUB_PULLS_REMOVE_REQUESTED_REVIEWERS: "Action"
    GITHUB_PULLS_REQUEST_REVIEWERS: "Action"
    GITHUB_PULLS_SUBMIT_REVIEW: "Action"
    GITHUB_PULLS_UPDATE: "Action"
    GITHUB_PULLS_UPDATE_BRANCH: "Action"
    GITHUB_PULLS_UPDATE_REVIEW: "Action"
    GITHUB_PULLS_UPDATE_REVIEW_COMMENT: "Action"
    GITHUB_RATE_LIMIT_GET: "Action"
    GITHUB_REACTIONS_CREATE_FOR_COMMIT_COMMENT: "Action"
    GITHUB_REACTIONS_CREATE_FOR_ISSUE: "Action"
    GITHUB_REACTIONS_CREATE_FOR_ISSUE_COMMENT: "Action"
    GITHUB_REACTIONS_CREATE_FOR_PULL_REQUEST_REVIEW_COMMENT: "Action"
    GITHUB_REACTIONS_CREATE_FOR_RELEASE: "Action"
    GITHUB_REACTIONS_CREATE_FOR_TEAM_DISCUSSION_COMMENT_IN_ORG: "Action"
    GITHUB_REACTIONS_CREATE_FOR_TEAM_DISCUSSION_COMMENT_LEGACY: "Action"
    GITHUB_REACTIONS_CREATE_FOR_TEAM_DISCUSSION_IN_ORG: "Action"
    GITHUB_REACTIONS_CREATE_FOR_TEAM_DISCUSSION_LEGACY: "Action"
    GITHUB_REACTIONS_DELETE_FOR_COMMIT_COMMENT: "Action"
    GITHUB_REACTIONS_DELETE_FOR_ISSUE: "Action"
    GITHUB_REACTIONS_DELETE_FOR_ISSUE_COMMENT: "Action"
    GITHUB_REACTIONS_DELETE_FOR_PULL_REQUEST_COMMENT: "Action"
    GITHUB_REACTIONS_DELETE_FOR_RELEASE: "Action"
    GITHUB_REACTIONS_DELETE_FOR_TEAM_DISCUSSION: "Action"
    GITHUB_REACTIONS_DELETE_FOR_TEAM_DISCUSSION_COMMENT: "Action"
    GITHUB_REACTIONS_LIST_FOR_COMMIT_COMMENT: "Action"
    GITHUB_REACTIONS_LIST_FOR_ISSUE: "Action"
    GITHUB_REACTIONS_LIST_FOR_ISSUE_COMMENT: "Action"
    GITHUB_REACTIONS_LIST_FOR_PULL_REQUEST_REVIEW_COMMENT: "Action"
    GITHUB_REACTIONS_LIST_FOR_RELEASE: "Action"
    GITHUB_REACTIONS_LIST_FOR_TEAM_DISCUSSION_COMMENT_IN_ORG: "Action"
    GITHUB_REACTIONS_LIST_FOR_TEAM_DISCUSSION_COMMENT_LEGACY: "Action"
    GITHUB_REACTIONS_LIST_FOR_TEAM_DISCUSSION_IN_ORG: "Action"
    GITHUB_REACTIONS_LIST_FOR_TEAM_DISCUSSION_LEGACY: "Action"
    GITHUB_REPO_SADD_APP_ACCESS_RESTRICTIONS: "Action"
    GITHUB_REPO_SADD_COLLABORATOR: "Action"
    GITHUB_REPO_SADD_STATUS_CHECK_CONTEXTS: "Action"
    GITHUB_REPO_SADD_TEAM_ACCESS_RESTRICTIONS: "Action"
    GITHUB_REPO_SADD_USER_ACCESS_RESTRICTIONS: "Action"
    GITHUB_REPO_S_ACCEPT_INVITATION_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_REPO_S_CANCEL_PAGES_DEPLOYMENT: "Action"
    GITHUB_REPO_S_CHECK_AUTOMATED_SECURITY_FIXES: "Action"
    GITHUB_REPO_S_CHECK_COLLABORATOR: "Action"
    GITHUB_REPO_S_CHECK_PRIVATE_VULNERABILITY_REPORTING: "Action"
    GITHUB_REPO_S_CHECK_VULNERABILITY_ALERTS: "Action"
    GITHUB_REPO_S_CODE_OWNERS_ERRORS: "Action"
    GITHUB_REPO_S_COMPARE_COMMITS: "Action"
    GITHUB_REPO_S_CREATE_AUTO_LINK: "Action"
    GITHUB_REPO_S_CREATE_COMMIT_COMMENT: "Action"
    GITHUB_REPO_S_CREATE_COMMIT_SIGNATURE_PROTECTION: "Action"
    GITHUB_REPO_S_CREATE_COMMIT_STATUS: "Action"
    GITHUB_REPO_S_CREATE_DEPLOYMENT: "Action"
    GITHUB_REPO_S_CREATE_DEPLOYMENT_BRANCH_POLICY: "Action"
    GITHUB_REPO_S_CREATE_DEPLOYMENT_PROTECTION_RULE: "Action"
    GITHUB_REPO_S_CREATE_DEPLOYMENT_STATUS: "Action"
    GITHUB_REPO_S_CREATE_DEPLOY_KEY: "Action"
    GITHUB_REPO_S_CREATE_DISPATCH_EVENT: "Action"
    GITHUB_REPO_S_CREATE_FORK: "Action"
    GITHUB_REPO_S_CREATE_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_REPO_S_CREATE_IN_ORG: "Action"
    GITHUB_REPO_S_CREATE_ORG_RULE_SET: "Action"
    GITHUB_REPO_S_CREATE_OR_UPDATE_CUSTOM_PROPERTIES_VALUES: "Action"
    GITHUB_REPO_S_CREATE_OR_UPDATE_ENVIRONMENT: "Action"
    GITHUB_REPO_S_CREATE_OR_UPDATE_FILE_CONTENTS: "Action"
    GITHUB_REPO_S_CREATE_PAGES_DEPLOYMENT: "Action"
    GITHUB_REPO_S_CREATE_PAGES_SITE: "Action"
    GITHUB_REPO_S_CREATE_RELEASE: "Action"
    GITHUB_REPO_S_CREATE_REPO_RULE_SET: "Action"
    GITHUB_REPO_S_CREATE_TAG_PROTECTION: "Action"
    GITHUB_REPO_S_CREATE_USING_TEMPLATE: "Action"
    GITHUB_REPO_S_CREATE_WEB_HOOK: "Action"
    GITHUB_REPO_S_DECLINE_INVITATION_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_REPO_S_DELETE: "Action"
    GITHUB_REPO_S_DELETE_ACCESS_RESTRICTIONS: "Action"
    GITHUB_REPO_S_DELETE_ADMIN_BRANCH_PROTECTION: "Action"
    GITHUB_REPO_S_DELETE_AN_ENVIRONMENT: "Action"
    GITHUB_REPO_S_DELETE_AUTO_LINK: "Action"
    GITHUB_REPO_S_DELETE_BRANCH_PROTECTION: "Action"
    GITHUB_REPO_S_DELETE_COMMIT_COMMENT: "Action"
    GITHUB_REPO_S_DELETE_COMMIT_SIGNATURE_PROTECTION: "Action"
    GITHUB_REPO_S_DELETE_DEPLOYMENT: "Action"
    GITHUB_REPO_S_DELETE_DEPLOYMENT_BRANCH_POLICY: "Action"
    GITHUB_REPO_S_DELETE_DEPLOY_KEY: "Action"
    GITHUB_REPO_S_DELETE_FILE: "Action"
    GITHUB_REPO_S_DELETE_INVITATION: "Action"
    GITHUB_REPO_S_DELETE_ORG_RULE_SET: "Action"
    GITHUB_REPO_S_DELETE_PAGES_SITE: "Action"
    GITHUB_REPO_S_DELETE_PULL_REQUEST_REVIEW_PROTECTION: "Action"
    GITHUB_REPO_S_DELETE_RELEASE: "Action"
    GITHUB_REPO_S_DELETE_RELEASE_ASSET: "Action"
    GITHUB_REPO_S_DELETE_REPO_RULE_SET: "Action"
    GITHUB_REPO_S_DELETE_TAG_PROTECTION: "Action"
    GITHUB_REPO_S_DELETE_WEB_HOOK: "Action"
    GITHUB_REPO_S_DISABLE_AUTOMATED_SECURITY_FIXES: "Action"
    GITHUB_REPO_S_DISABLE_DEPLOYMENT_PROTECTION_RULE: "Action"
    GITHUB_REPO_S_DISABLE_PRIVATE_VULNERABILITY_REPORTING: "Action"
    GITHUB_REPO_S_DISABLE_VULNERABILITY_ALERTS: "Action"
    GITHUB_REPO_S_DOWNLOAD_TAR_BALL_ARCHIVE: "Action"
    GITHUB_REPO_S_DOWNLOAD_ZIP_BALL_ARCHIVE: "Action"
    GITHUB_REPO_S_ENABLE_AUTOMATED_SECURITY_FIXES: "Action"
    GITHUB_REPO_S_ENABLE_PRIVATE_VULNERABILITY_REPORTING: "Action"
    GITHUB_REPO_S_ENABLE_VULNERABILITY_ALERTS: "Action"
    GITHUB_REPO_S_GENERATE_RELEASE_NOTES: "Action"
    GITHUB_REPO_S_GET: "Action"
    GITHUB_REPO_S_GET_ACCESS_RESTRICTIONS: "Action"
    GITHUB_REPO_S_GET_ADMIN_BRANCH_PROTECTION: "Action"
    GITHUB_REPO_S_GET_ALL_DEPLOYMENT_PROTECTION_RULES: "Action"
    GITHUB_REPO_S_GET_ALL_ENVIRONMENTS: "Action"
    GITHUB_REPO_S_GET_ALL_STATUS_CHECK_CONTEXTS: "Action"
    GITHUB_REPO_S_GET_ALL_TOPICS: "Action"
    GITHUB_REPO_S_GET_APPS_WITH_ACCESS_TO_PROTECTED_BRANCH: "Action"
    GITHUB_REPO_S_GET_AUTO_LINK: "Action"
    GITHUB_REPO_S_GET_BRANCH: "Action"
    GITHUB_REPO_S_GET_BRANCH_PROTECTION: "Action"
    GITHUB_REPO_S_GET_BRANCH_RULES: "Action"
    GITHUB_REPO_S_GET_CLONES: "Action"
    GITHUB_REPO_S_GET_CODE_FREQUENCY_STATS: "Action"
    GITHUB_REPO_S_GET_COLLABORATOR_PERMISSION_LEVEL: "Action"
    GITHUB_REPO_S_GET_COMBINED_STATUS_FOR_REF: "Action"
    GITHUB_REPO_S_GET_COMMIT: "Action"
    GITHUB_REPO_S_GET_COMMIT_ACTIVITY_STATS: "Action"
    GITHUB_REPO_S_GET_COMMIT_COMMENT: "Action"
    GITHUB_REPO_S_GET_COMMIT_SIGNATURE_PROTECTION: "Action"
    GITHUB_REPO_S_GET_COMMUNITY_PROFILE_METRICS: "Action"
    GITHUB_REPO_S_GET_CONTENT: "Action"
    GITHUB_REPO_S_GET_CONTRIBUTORS_STATS: "Action"
    GITHUB_REPO_S_GET_CUSTOM_DEPLOYMENT_PROTECTION_RULE: "Action"
    GITHUB_REPO_S_GET_CUSTOM_PROPERTIES_VALUES: "Action"
    GITHUB_REPO_S_GET_DEPLOYMENT: "Action"
    GITHUB_REPO_S_GET_DEPLOYMENT_BRANCH_POLICY: "Action"
    GITHUB_REPO_S_GET_DEPLOYMENT_STATUS: "Action"
    GITHUB_REPO_S_GET_DEPLOY_KEY: "Action"
    GITHUB_REPO_S_GET_ENVIRONMENT: "Action"
    GITHUB_REPO_S_GET_LATEST_PAGES_BUILD: "Action"
    GITHUB_REPO_S_GET_LATEST_RELEASE: "Action"
    GITHUB_REPO_S_GET_ORG_RULE_SET: "Action"
    GITHUB_REPO_S_GET_ORG_RULE_SETS: "Action"
    GITHUB_REPO_S_GET_ORG_RULE_SUITE: "Action"
    GITHUB_REPO_S_GET_ORG_RULE_SUITES: "Action"
    GITHUB_REPO_S_GET_PAGES: "Action"
    GITHUB_REPO_S_GET_PAGES_BUILD: "Action"
    GITHUB_REPO_S_GET_PAGES_DEPLOYMENT: "Action"
    GITHUB_REPO_S_GET_PAGES_HEALTH_CHECK: "Action"
    GITHUB_REPO_S_GET_PARTICIPATION_STATS: "Action"
    GITHUB_REPO_S_GET_PULL_REQUEST_REVIEW_PROTECTION: "Action"
    GITHUB_REPO_S_GET_PUNCH_CARD_STATS: "Action"
    GITHUB_REPO_S_GET_README: "Action"
    GITHUB_REPO_S_GET_README_IN_DIRECTORY: "Action"
    GITHUB_REPO_S_GET_RELEASE: "Action"
    GITHUB_REPO_S_GET_RELEASE_ASSET: "Action"
    GITHUB_REPO_S_GET_RELEASE_BY_TAG: "Action"
    GITHUB_REPO_S_GET_REPO_RULE_SET: "Action"
    GITHUB_REPO_S_GET_REPO_RULE_SETS: "Action"
    GITHUB_REPO_S_GET_REPO_RULE_SUITE: "Action"
    GITHUB_REPO_S_GET_REPO_RULE_SUITES: "Action"
    GITHUB_REPO_S_GET_STATUS_CHECKS_PROTECTION: "Action"
    GITHUB_REPO_S_GET_TEAMS_WITH_ACCESS_TO_PROTECTED_BRANCH: "Action"
    GITHUB_REPO_S_GET_TOP_PATHS: "Action"
    GITHUB_REPO_S_GET_TOP_REFERRER_S: "Action"
    GITHUB_REPO_S_GET_USERS_WITH_ACCESS_TO_PROTECTED_BRANCH: "Action"
    GITHUB_REPO_S_GET_VIEWS: "Action"
    GITHUB_REPO_S_GET_WEB_HOOK: "Action"
    GITHUB_REPO_S_GET_WEB_HOOK_CONFIG_FOR_REPO: "Action"
    GITHUB_REPO_S_GET_WEB_HOOK_DELIVERY: "Action"
    GITHUB_REPO_S_LIST_ACTIVITIES: "Action"
    GITHUB_REPO_S_LIST_AUTO_LINKS: "Action"
    GITHUB_REPO_S_LIST_BRANCHES: "Action"
    GITHUB_REPO_S_LIST_BRANCHES_FOR_HEAD_COMMIT: "Action"
    GITHUB_REPO_S_LIST_COLLABORATORS: "Action"
    GITHUB_REPO_S_LIST_COMMENTS_FOR_COMMIT: "Action"
    GITHUB_REPO_S_LIST_COMMITS: "Action"
    GITHUB_REPO_S_LIST_COMMIT_COMMENTS_FOR_REPO: "Action"
    GITHUB_REPO_S_LIST_COMMIT_STATUSES_FOR_REF: "Action"
    GITHUB_REPO_S_LIST_CONTRIBUTORS: "Action"
    GITHUB_REPO_S_LIST_CUSTOM_DEPLOYMENT_RULE_INTEGRATION_S: "Action"
    GITHUB_REPO_S_LIST_DEPLOYMENTS: "Action"
    GITHUB_REPO_S_LIST_DEPLOYMENT_BRANCH_POLICIES: "Action"
    GITHUB_REPO_S_LIST_DEPLOYMENT_STATUSES: "Action"
    GITHUB_REPO_S_LIST_DEPLOY_KEYS: "Action"
    GITHUB_REPO_S_LIST_FORKS: "Action"
    GITHUB_REPO_S_LIST_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_REPO_S_LIST_FOR_ORG: "Action"
    GITHUB_REPO_S_LIST_FOR_USER: "Action"
    GITHUB_REPO_S_LIST_INVITATIONS: "Action"
    GITHUB_REPO_S_LIST_INVITATIONS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_REPO_S_LIST_LANGUAGES: "Action"
    GITHUB_REPO_S_LIST_PAGES_BUILDS: "Action"
    GITHUB_REPO_S_LIST_PUBLIC: "Action"
    GITHUB_REPO_S_LIST_PULL_REQUESTS_ASSOCIATED_WITH_COMMIT: "Action"
    GITHUB_REPO_S_LIST_RELEASES: "Action"
    GITHUB_REPO_S_LIST_RELEASE_ASSETS: "Action"
    GITHUB_REPO_S_LIST_TAGS: "Action"
    GITHUB_REPO_S_LIST_TAG_PROTECTION: "Action"
    GITHUB_REPO_S_LIST_TEAMS: "Action"
    GITHUB_REPO_S_LIST_WEB_HOOKS: "Action"
    GITHUB_REPO_S_LIST_WEB_HOOK_DELIVERIES: "Action"
    GITHUB_REPO_S_MERGE: "Action"
    GITHUB_REPO_S_MERGE_UPSTREAM: "Action"
    GITHUB_REPO_S_PING_WEB_HOOK: "Action"
    GITHUB_REPO_S_REMOVE_APP_ACCESS_RESTRICTIONS: "Action"
    GITHUB_REPO_S_REMOVE_COLLABORATOR: "Action"
    GITHUB_REPO_S_REMOVE_STATUS_CHECK_CONTEXTS: "Action"
    GITHUB_REPO_S_REMOVE_STATUS_CHECK_PROTECTION: "Action"
    GITHUB_REPO_S_REMOVE_TEAM_ACCESS_RESTRICTIONS: "Action"
    GITHUB_REPO_S_REMOVE_USER_ACCESS_RESTRICTIONS: "Action"
    GITHUB_REPO_S_RENAME_BRANCH: "Action"
    GITHUB_REPO_S_REPLACE_ALL_TOPICS: "Action"
    GITHUB_REPO_S_REQUEST_PAGES_BUILD: "Action"
    GITHUB_REPO_S_RE_DELIVER_WEB_HOOK_DELIVERY: "Action"
    GITHUB_REPO_S_SET_ADMIN_BRANCH_PROTECTION: "Action"
    GITHUB_REPO_S_SET_APP_ACCESS_RESTRICTIONS: "Action"
    GITHUB_REPO_S_SET_STATUS_CHECK_CONTEXTS: "Action"
    GITHUB_REPO_S_SET_TEAM_ACCESS_RESTRICTIONS: "Action"
    GITHUB_REPO_S_SET_USER_ACCESS_RESTRICTIONS: "Action"
    GITHUB_REPO_S_TEST_PUSH_WEB_HOOK: "Action"
    GITHUB_REPO_S_TRANSFER: "Action"
    GITHUB_REPO_S_UPDATE: "Action"
    GITHUB_REPO_S_UPDATE_BRANCH_PROTECTION: "Action"
    GITHUB_REPO_S_UPDATE_COMMIT_COMMENT: "Action"
    GITHUB_REPO_S_UPDATE_DEPLOYMENT_BRANCH_POLICY: "Action"
    GITHUB_REPO_S_UPDATE_INFORMATION_ABOUT_PAGES_SITE: "Action"
    GITHUB_REPO_S_UPDATE_INVITATION: "Action"
    GITHUB_REPO_S_UPDATE_ORG_RULE_SET: "Action"
    GITHUB_REPO_S_UPDATE_PULL_REQUEST_REVIEW_PROTECTION: "Action"
    GITHUB_REPO_S_UPDATE_RELEASE: "Action"
    GITHUB_REPO_S_UPDATE_RELEASE_ASSET: "Action"
    GITHUB_REPO_S_UPDATE_REPO_RULE_SET: "Action"
    GITHUB_REPO_S_UPDATE_STATUS_CHECK_PROTECTION: "Action"
    GITHUB_REPO_S_UPDATE_WEB_HOOK: "Action"
    GITHUB_REPO_S_UPDATE_WEB_HOOK_CONFIG_FOR_REPO: "Action"
    GITHUB_REPO_S_UPLOAD_RELEASE_ASSET: "Action"
    GITHUB_SEARCH_CODE: "Action"
    GITHUB_SEARCH_COMMITS: "Action"
    GITHUB_SEARCH_ISSUES_AND_PULL_REQUESTS: "Action"
    GITHUB_SEARCH_LABELS: "Action"
    GITHUB_SEARCH_REPO_S: "Action"
    GITHUB_SEARCH_TOPICS: "Action"
    GITHUB_SEARCH_USERS: "Action"
    GITHUB_SECRET_SCANNING_GET_ALERT: "Action"
    GITHUB_SECRET_SCANNING_LIST_ALERTS_FOR_ENTERPRISE: "Action"
    GITHUB_SECRET_SCANNING_LIST_ALERTS_FOR_ORG: "Action"
    GITHUB_SECRET_SCANNING_LIST_ALERTS_FOR_REPO: "Action"
    GITHUB_SECRET_SCANNING_LIST_LOCATIONS_FOR_ALERT: "Action"
    GITHUB_SECRET_SCANNING_UPDATE_ALERT: "Action"
    GITHUB_SECURITY_ADVISORIES_CREATE_FORK: "Action"
    GITHUB_SECURITY_ADVISORIES_CREATE_PRIVATE_VULNERABILITY_REPORT: "Action"
    GITHUB_SECURITY_ADVISORIES_CREATE_REPOSITORY_ADVISORY: "Action"
    GITHUB_SECURITY_ADVISORIES_CREATE_REPOSITORY_ADVISORY_CVE_REQUEST: "Action"
    GITHUB_SECURITY_ADVISORIES_GET_GLOBAL_ADVISORY: "Action"
    GITHUB_SECURITY_ADVISORIES_GET_REPOSITORY_ADVISORY: "Action"
    GITHUB_SECURITY_ADVISORIES_LIST_GLOBAL_ADVISORIES: "Action"
    GITHUB_SECURITY_ADVISORIES_LIST_ORG_REPOSITORY_ADVISORIES: "Action"
    GITHUB_SECURITY_ADVISORIES_LIST_REPOSITORY_ADVISORIES: "Action"
    GITHUB_SECURITY_ADVISORIES_UPDATE_REPOSITORY_ADVISORY: "Action"
    GITHUB_TEAMS_ADD_MEMBER_LEGACY: "Action"
    GITHUB_TEAMS_ADD_OR_UPDATE_MEMBERSHIP_FOR_USER_IN_ORG: "Action"
    GITHUB_TEAMS_ADD_OR_UPDATE_MEMBERSHIP_FOR_USER_LEGACY: "Action"
    GITHUB_TEAMS_ADD_OR_UPDATE_PROJECT_PERMISSIONS_IN_ORG: "Action"
    GITHUB_TEAMS_ADD_OR_UPDATE_PROJECT_PERMISSIONS_LEGACY: "Action"
    GITHUB_TEAMS_ADD_OR_UPDATE_REPO_PERMISSIONS_IN_ORG: "Action"
    GITHUB_TEAMS_ADD_OR_UPDATE_REPO_PERMISSIONS_LEGACY: "Action"
    GITHUB_TEAMS_CHECK_PERMISSIONS_FOR_PROJECT_IN_ORG: "Action"
    GITHUB_TEAMS_CHECK_PERMISSIONS_FOR_PROJECT_LEGACY: "Action"
    GITHUB_TEAMS_CHECK_PERMISSIONS_FOR_REPO_IN_ORG: "Action"
    GITHUB_TEAMS_CHECK_PERMISSIONS_FOR_REPO_LEGACY: "Action"
    GITHUB_TEAMS_CREATE: "Action"
    GITHUB_TEAMS_CREATE_DISCUSSION_COMMENT_IN_ORG: "Action"
    GITHUB_TEAMS_CREATE_DISCUSSION_COMMENT_LEGACY: "Action"
    GITHUB_TEAMS_CREATE_DISCUSSION_IN_ORG: "Action"
    GITHUB_TEAMS_CREATE_DISCUSSION_LEGACY: "Action"
    GITHUB_TEAMS_DELETE_DISCUSSION_COMMENT_IN_ORG: "Action"
    GITHUB_TEAMS_DELETE_DISCUSSION_COMMENT_LEGACY: "Action"
    GITHUB_TEAMS_DELETE_DISCUSSION_IN_ORG: "Action"
    GITHUB_TEAMS_DELETE_DISCUSSION_LEGACY: "Action"
    GITHUB_TEAMS_DELETE_IN_ORG: "Action"
    GITHUB_TEAMS_DELETE_LEGACY: "Action"
    GITHUB_TEAMS_GET_BY_NAME: "Action"
    GITHUB_TEAMS_GET_DISCUSSION_COMMENT_IN_ORG: "Action"
    GITHUB_TEAMS_GET_DISCUSSION_COMMENT_LEGACY: "Action"
    GITHUB_TEAMS_GET_DISCUSSION_IN_ORG: "Action"
    GITHUB_TEAMS_GET_DISCUSSION_LEGACY: "Action"
    GITHUB_TEAMS_GET_LEGACY: "Action"
    GITHUB_TEAMS_GET_MEMBERSHIP_FOR_USER_IN_ORG: "Action"
    GITHUB_TEAMS_GET_MEMBERSHIP_FOR_USER_LEGACY: "Action"
    GITHUB_TEAMS_GET_MEMBER_LEGACY: "Action"
    GITHUB_TEAMS_LIST: "Action"
    GITHUB_TEAMS_LIST_CHILD_IN_ORG: "Action"
    GITHUB_TEAMS_LIST_CHILD_LEGACY: "Action"
    GITHUB_TEAMS_LIST_DISCUSSIONS_IN_ORG: "Action"
    GITHUB_TEAMS_LIST_DISCUSSIONS_LEGACY: "Action"
    GITHUB_TEAMS_LIST_DISCUSSION_COMMENTS_IN_ORG: "Action"
    GITHUB_TEAMS_LIST_DISCUSSION_COMMENTS_LEGACY: "Action"
    GITHUB_TEAMS_LIST_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_TEAMS_LIST_MEMBERS_IN_ORG: "Action"
    GITHUB_TEAMS_LIST_MEMBERS_LEGACY: "Action"
    GITHUB_TEAMS_LIST_PENDING_INVITATIONS_IN_ORG: "Action"
    GITHUB_TEAMS_LIST_PENDING_INVITATIONS_LEGACY: "Action"
    GITHUB_TEAMS_LIST_PROJECTS_IN_ORG: "Action"
    GITHUB_TEAMS_LIST_PROJECTS_LEGACY: "Action"
    GITHUB_TEAMS_LIST_REPO_S_IN_ORG: "Action"
    GITHUB_TEAMS_LIST_REPO_S_LEGACY: "Action"
    GITHUB_TEAMS_REMOVE_MEMBERSHIP_FOR_USER_IN_ORG: "Action"
    GITHUB_TEAMS_REMOVE_MEMBERSHIP_FOR_USER_LEGACY: "Action"
    GITHUB_TEAMS_REMOVE_MEMBER_LEGACY: "Action"
    GITHUB_TEAMS_REMOVE_PROJECT_IN_ORG: "Action"
    GITHUB_TEAMS_REMOVE_PROJECT_LEGACY: "Action"
    GITHUB_TEAMS_REMOVE_REPO_IN_ORG: "Action"
    GITHUB_TEAMS_REMOVE_REPO_LEGACY: "Action"
    GITHUB_TEAMS_UPDATE_DISCUSSION_COMMENT_IN_ORG: "Action"
    GITHUB_TEAMS_UPDATE_DISCUSSION_COMMENT_LEGACY: "Action"
    GITHUB_TEAMS_UPDATE_DISCUSSION_IN_ORG: "Action"
    GITHUB_TEAMS_UPDATE_DISCUSSION_LEGACY: "Action"
    GITHUB_TEAMS_UPDATE_IN_ORG: "Action"
    GITHUB_TEAMS_UPDATE_LEGACY: "Action"
    GITHUB_USERS_ADD_EMAIL_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_ADD_SOCIAL_ACCOUNT_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_BLOCK: "Action"
    GITHUB_USERS_CHECK_BLOCKED: "Action"
    GITHUB_USERS_CHECK_FOLLOWING_FOR_USER: "Action"
    GITHUB_USERS_CHECK_PERSON_IS_FOLLOWED_BY_AUTHENTICATED: "Action"
    GITHUB_USERS_CREATES_SH_SIGNING_KEY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_CREATE_GPG_KEY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_CREATE_PUBLIC_SSH_KEY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_DELETE_EMAIL_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_DELETE_GPG_KEY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_DELETE_PUBLIC_SSH_KEY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_DELETE_SOCIAL_ACCOUNT_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_DELETE_SSH_SIGNING_KEY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_FOLLOW: "Action"
    GITHUB_USERS_GETS_SH_SIGNING_KEY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_GET_AUTHENTICATED: "Action"
    GITHUB_USERS_GET_BY_USERNAME: "Action"
    GITHUB_USERS_GET_CONTEXT_FOR_USER: "Action"
    GITHUB_USERS_GET_GPG_KEY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_GET_PUBLIC_SSH_KEY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_LIST: "Action"
    GITHUB_USERS_LIST_BLOCKED_BY_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_LIST_EMAILS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_LIST_FOLLOWED_BY_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_LIST_FOLLOWERS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_LIST_FOLLOWERS_FOR_USER: "Action"
    GITHUB_USERS_LIST_FOLLOWING_FOR_USER: "Action"
    GITHUB_USERS_LIST_GPG_KEYS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_LIST_GPG_KEYS_FOR_USER: "Action"
    GITHUB_USERS_LIST_PUBLIC_EMAILS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_LIST_PUBLIC_KEYS_FOR_USER: "Action"
    GITHUB_USERS_LIST_PUBLIC_SSH_KEYS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_LIST_SOCIAL_ACCOUNTS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_LIST_SOCIAL_ACCOUNTS_FOR_USER: "Action"
    GITHUB_USERS_LIST_SSH_SIGNING_KEYS_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_LIST_SSH_SIGNING_KEYS_FOR_USER: "Action"
    GITHUB_USERS_SET_PRIMARY_EMAIL_VISIBILITY_FOR_AUTHENTICATED_USER: "Action"
    GITHUB_USERS_UNBLOCK: "Action"
    GITHUB_USERS_UPDATE_AUTHENTICATED: "Action"
    GITHUB_USER_SUN_FOLLOW: "Action"
    GMAIL_ADD_LABEL_TO_EMAIL: "Action"
    GMAIL_CREATE_EMAIL_DRAFT: "Action"
    GMAIL_FETCH_EMAILS_WITH_LABEL: "Action"
    GMAIL_FETCH_LAST_THREE_MESSAGES: "Action"
    GMAIL_FETCH_MESSAGE_BY_THREAD_ID: "Action"
    GMAIL_FIND_EMAIL_ID: "Action"
    GMAIL_LIST_LABELS: "Action"
    GMAIL_REPLY_TO_THREAD: "Action"
    GMAIL_SEND_EMAIL: "Action"
    GOOGLECALENDAR_CREATE_EVENT: "Action"
    GOOGLECALENDAR_DELETE_EVENT: "Action"
    GOOGLECALENDAR_DUPLICATE_CALENDAR: "Action"
    GOOGLECALENDAR_FIND_EVENT: "Action"
    GOOGLECALENDAR_FIND_FREE_SLOTS: "Action"
    GOOGLECALENDAR_GET_CURRENT_DATE_TIME: "Action"
    GOOGLECALENDAR_LIST_CALENDARS: "Action"
    GOOGLECALENDAR_PATCH_CALENDAR: "Action"
    GOOGLECALENDAR_QUICK_ADD: "Action"
    GOOGLECALENDAR_REMOVE_ATTENDEE: "Action"
    GOOGLECALENDAR_UPDATE_EVENT: "Action"
    GOOGLEDOCS_CREATE_DOCUMENT: "Action"
    GOOGLEDOCS_GET_DOCUMENT_BY_ID: "Action"
    GOOGLEDOCS_UPDATE_EXISTING_DOCUMENT: "Action"
    GOOGLEDRIVE_ADD_FILE_SHARING_PREFERENCE: "Action"
    GOOGLEDRIVE_COPY_FILE: "Action"
    GOOGLEDRIVE_CREATE_FILE_FROM_TEXT: "Action"
    GOOGLEDRIVE_CREATE_FOLDER: "Action"
    GOOGLEDRIVE_DELETE_FOLDER_OR_FILE: "Action"
    GOOGLEDRIVE_EDIT_FILE: "Action"
    GOOGLEDRIVE_EXPORT_FILE: "Action"
    GOOGLEDRIVE_FIND_FILE: "Action"
    GOOGLEDRIVE_FIND_FOLDER: "Action"
    GOOGLEMEET_CREATE_MEET: "Action"
    GOOGLEMEET_GET_CONFERENCE_RECORD_FOR_MEET: "Action"
    GOOGLEMEET_GET_MEET: "Action"
    GOOGLEMEET_GET_RECORDINGS_BY_CONFERENCE_RECORD_ID: "Action"
    GOOGLEMEET_GET_TRANSCRIPTS_BY_CONFERENCE_RECORD_ID: "Action"
    GOOGLESHEETS_BATCH_GET: "Action"
    GOOGLESHEETS_BATCH_UPDATE: "Action"
    GOOGLESHEETS_CLEAR_VALUES: "Action"
    GOOGLESHEETS_CREATE_GOOGLE_SHEET1: "Action"
    GOOGLESHEETS_GET_SPREADSHEET_INFO: "Action"
    GOOGLESHEETS_LOOKUP_SPREADSHEET_ROW: "Action"
    GOOGLETASKS_CLEAR_TASKS: "Action"
    GOOGLETASKS_CREATE_TASK_LIST: "Action"
    GOOGLETASKS_DELETE_TASK: "Action"
    GOOGLETASKS_DELETE_TASK_LIST: "Action"
    GOOGLETASKS_GET_TASK: "Action"
    GOOGLETASKS_GET_TASK_LIST: "Action"
    GOOGLETASKS_INSERT_TASK: "Action"
    GOOGLETASKS_LIST_TASK_LISTS: "Action"
    GOOGLETASKS_PATCH_TASK: "Action"
    GOOGLETASKS_PATCH_TASK_LIST: "Action"
    GREPTILE_CODE_QUERY: "Action"
    HACKERNEWS_GET_FRONTPAGE: "Action"
    HACKERNEWS_GET_TODAYS_POSTS: "Action"
    HEROKU_CREATE_HEROKU_APP: "Action"
    HEROKU_DELETE_HEROKU_APP: "Action"
    HEROKU_GET_ACCOUNT_DELINQUENCY_INFO: "Action"
    HEROKU_GET_ACCOUNT_FEATURE_INFO: "Action"
    HEROKU_GET_ACCOUNT_FEATURE_LIST: "Action"
    HEROKU_GET_ACCOUNT_INFO: "Action"
    HEROKU_GET_HEROKU_APP_INFO: "Action"
    HEROKU_GET_HEROKU_APP_LIST: "Action"
    HEROKU_UPDATE_ACCOUNT_FEATURE: "Action"
    HEROKU_UPDATE_ACCOUNT_INFO: "Action"
    HISTORYFETCHERTOOL_GET_WORKSPACE_HISTORY: "Action"
    INDUCEDAI_EXTRACT_DATA: "Action"
    INDUCEDAI_GET_AUTONOMOUS_TASK_STATUS: "Action"
    INDUCEDAI_GET_DATA_EXTRACTION_STATUS: "Action"
    INDUCEDAI_PERFORM_AUTONOMOUS_TASK: "Action"
    INDUCEDAI_STOP_AUTONOMOUS_TASK: "Action"
    LINEAR_CREATE_LINEAR_ISSUE: "Action"
    LINEAR_LIST_LINEAR_PROJECTS: "Action"
    LINEAR_LIST_LINEAR_TEAMS: "Action"
    LISTENNOTES_DELETE_PODCAST_BY_ID: "Action"
    LISTENNOTES_GET_BEST_PODCASTS: "Action"
    LISTENNOTES_GET_CURATE_D_PODCASTS: "Action"
    LISTENNOTES_GET_CURATE_D_PODCAST_BY_ID: "Action"
    LISTENNOTES_GET_EPISODES_IN_BATCH: "Action"
    LISTENNOTES_GET_EPISODE_BY_ID: "Action"
    LISTENNOTES_GET_EPISODE_RECOMMENDATIONS: "Action"
    LISTENNOTES_GET_GENRES: "Action"
    LISTENNOTES_GET_LANGUAGES: "Action"
    LISTENNOTES_GET_PLAYLISTS: "Action"
    LISTENNOTES_GET_PLAYLIST_BY_ID: "Action"
    LISTENNOTES_GET_PODCASTS_BY_DOMAIN_NAME: "Action"
    LISTENNOTES_GET_PODCASTS_IN_BATCH: "Action"
    LISTENNOTES_GET_PODCAST_AUDIENCE: "Action"
    LISTENNOTES_GET_PODCAST_BY_ID: "Action"
    LISTENNOTES_GET_PODCAST_RECOMMENDATIONS: "Action"
    LISTENNOTES_GET_REGIONS: "Action"
    LISTENNOTES_GET_RELATED_SEARCHES: "Action"
    LISTENNOTES_GET_TRENDING_SEARCHES: "Action"
    LISTENNOTES_JUST_LISTEN: "Action"
    LISTENNOTES_REFRESH_RSS: "Action"
    LISTENNOTES_SEARCH: "Action"
    LISTENNOTES_SEARCH_EPISODE_TITLES: "Action"
    LISTENNOTES_SPELL_CHECK: "Action"
    LISTENNOTES_SUBMIT_PODCAST: "Action"
    LISTENNOTES_TYPE_AHEAD: "Action"
    MATHEMATICAL_CALCULATOR: "Action"
    MULTIONAI_CREATE_SESSION: "Action"
    MULTIONAI_DELETE_SESSION: "Action"
    MULTIONAI_LIST_SESSIONS: "Action"
    MULTIONAI_RETRIEVE_DATA: "Action"
    MULTIONAI_SCREENSHOT: "Action"
    MULTIONAI_STEP_SESSION: "Action"
    MULTIONAI_WEB_BROWSE: "Action"
    NASA_ORGANIZATION_GET_INFORMATION: "Action"
    NASA_ORGANIZATION_GET_LIST_BY_NAME: "Action"
    NASA_ORGANIZATION_LIST_TYPES: "Action"
    NASA_PROJECT_FIND_MATCHING_PROJECTS: "Action"
    NASA_PROJECT_GET_INFO: "Action"
    NASA_PROJECT_LIST_AVAILABLE_IDS: "Action"
    NASA_RESOURCE_GET_SPECIFICATION: "Action"
    NOTION_ADD_PAGE_CONTENT: "Action"
    NOTION_ARCHIVE_NOTION_PAGE: "Action"
    NOTION_CREATE_COMMENT: "Action"
    NOTION_CREATE_DATABASE: "Action"
    NOTION_CREATE_NOTION_PAGE: "Action"
    NOTION_DELETE_BLOCK: "Action"
    NOTION_FETCH_COMMENTS: "Action"
    NOTION_FETCH_DATABASE: "Action"
    NOTION_FETCH_NOTION_BLOCK: "Action"
    NOTION_FETCH_NOTION_CHILD_BLOCK: "Action"
    NOTION_FETCH_ROW: "Action"
    NOTION_GET_ABOUT_ME: "Action"
    NOTION_GET_ABOUT_USER: "Action"
    NOTION_INSERT_ROW_DATABASE: "Action"
    NOTION_LIST_USERS: "Action"
    NOTION_QUERY_DATABASE: "Action"
    NOTION_SEARCH_NOTION_PAGE: "Action"
    NOTION_UPDATE_ROW_DATABASE: "Action"
    NOTION_UPDATE_SCHEMA_DATABASE: "Action"
    OKTA_APPLICATION_ACTIVATE_CLIENT_SECRET: "Action"
    OKTA_APPLICATION_ACTIVATE_DEFAULT_PROVISIONING_CONNECTION: "Action"
    OKTA_APPLICATION_ACTIVATE_INACTIVE: "Action"
    OKTA_APPLICATION_ADD_CLIENT_SECRET: "Action"
    OKTA_APPLICATION_ASSIGN_GROUP_TO: "Action"
    OKTA_APPLICATION_ASSIGN_POLICY_TO_APPLICATION: "Action"
    OKTA_APPLICATION_ASSIGN_USER_TO_APPLICATION: "Action"
    OKTA_APPLICATION_CLONE_APPLICATION_KEY_CREDENTIAL: "Action"
    OKTA_APPLICATION_CREATE_NEW: "Action"
    OKTA_APPLICATION_DEACTIVATE_CLIENT_SECRET_BY_ID: "Action"
    OKTA_APPLICATION_DEACTIVATE_DEFAULT_PROVISIONING_CONNECTION: "Action"
    OKTA_APPLICATION_DEACTIVATE_LIFECYCLE: "Action"
    OKTA_APPLICATION_DELETE_CSR_BY_ID: "Action"
    OKTA_APPLICATION_GENERATE_CSR_FOR_APPLICATION: "Action"
    OKTA_APPLICATION_GENERATE_X_509_CERTIFICATE: "Action"
    OKTA_APPLICATION_GET_BY_ID: "Action"
    OKTA_APPLICATION_GET_CLIENT_SECRET: "Action"
    OKTA_APPLICATION_GET_CREDENTIALS_CSR_S: "Action"
    OKTA_APPLICATION_GET_DEFAULT_PROVISIONING_CONNECTION: "Action"
    OKTA_APPLICATION_GET_FEATURE: "Action"
    OKTA_APPLICATION_GET_GROUP_ASSIGNMENT: "Action"
    OKTA_APPLICATION_GET_KEY_CREDENTIAL: "Action"
    OKTA_APPLICATION_GET_SINGLE_SCOPE_CONSENT_GRANT: "Action"
    OKTA_APPLICATION_GET_SPECIFIC_USER_ASSIGNMENT: "Action"
    OKTA_APPLICATION_GET_TOKEN: "Action"
    OKTA_APPLICATION_GRANT_CONSENT_TO_SCOPE: "Action"
    OKTA_APPLICATION_LIST_APPS: "Action"
    OKTA_APPLICATION_LIST_ASSIGNED_USERS: "Action"
    OKTA_APPLICATION_LIST_CLIENT_SECRETS: "Action"
    OKTA_APPLICATION_LIST_CSR_S_FOR_APPLICATION: "Action"
    OKTA_APPLICATION_LIST_FEATURES: "Action"
    OKTA_APPLICATION_LIST_GROUPS_ASSIGNED: "Action"
    OKTA_APPLICATION_LIST_KEY_CREDENTIALS: "Action"
    OKTA_APPLICATION_LIST_SCOPE_CONSENT_GRANTS: "Action"
    OKTA_APPLICATION_LIST_TOKENS: "Action"
    OKTA_APPLICATION_PREVIEWS_AM_LAPP_METADATA: "Action"
    OKTA_APPLICATION_PUBLISH_CSR_LIFECYCLE: "Action"
    OKTA_APPLICATION_REMOVE_GROUP_ASSIGNMENT: "Action"
    OKTA_APPLICATION_REMOVE_INACTIVE: "Action"
    OKTA_APPLICATION_REMOVE_SECRET: "Action"
    OKTA_APPLICATION_REMOVE_USER_FROM: "Action"
    OKTA_APPLICATION_REVOKE_ALL_TOKENS: "Action"
    OKTA_APPLICATION_REVOKE_PERMISSION: "Action"
    OKTA_APPLICATION_REVOKE_TOKEN: "Action"
    OKTA_APPLICATION_SET_DEFAULT_PROVISIONING_CONNECTION: "Action"
    OKTA_APPLICATION_UPDATE_APPLICATION_IN_ORG: "Action"
    OKTA_APPLICATION_UPDATE_FEATURE: "Action"
    OKTA_APPLICATION_UPDATE_LOGO: "Action"
    OKTA_APPLICATION_UPDATE_PROFILE_FOR_USER: "Action"
    OKTA_AUTHENTIC_AT_OR_ACTIVATE_LIFECYCLE_SUCCESS: "Action"
    OKTA_AUTHENTIC_AT_OR_CREATE_NEW: "Action"
    OKTA_AUTHENTIC_AT_OR_DEACTIVATE_LIFECYCLE_SUCCESS: "Action"
    OKTA_AUTHENTIC_AT_OR_GET_SUCCESS: "Action"
    OKTA_AUTHENTIC_AT_OR_LIST_ALL_AVAILABLE: "Action"
    OKTA_AUTHENTIC_AT_OR_UPDATE_AUTHENTIC_AT_OR: "Action"
    OKTA_AUTHORIZATION_SERVER_ACTIVATE_LIFECYCLE_SUCCESS: "Action"
    OKTA_AUTHORIZATION_SERVER_ACTIVATE_POLICY_LIFECYCLE: "Action"
    OKTA_AUTHORIZATION_SERVER_ACTIVATE_POLICY_RULE: "Action"
    OKTA_AUTHORIZATION_SERVER_CREATE_CLAIMS: "Action"
    OKTA_AUTHORIZATION_SERVER_CREATE_NEW_SERVER: "Action"
    OKTA_AUTHORIZATION_SERVER_CREATE_POLICY: "Action"
    OKTA_AUTHORIZATION_SERVER_CREATE_POLICY_RULE: "Action"
    OKTA_AUTHORIZATION_SERVER_CREATE_SCOPE: "Action"
    OKTA_AUTHORIZATION_SERVER_DEACTIVATE_LIFECYCLE: "Action"
    OKTA_AUTHORIZATION_SERVER_DEACTIVATE_POLICY_LIFECYCLE: "Action"
    OKTA_AUTHORIZATION_SERVER_DEACTIVATE_POLICY_RULE: "Action"
    OKTA_AUTHORIZATION_SERVER_DELETE_AU_TH_TOKEN: "Action"
    OKTA_AUTHORIZATION_SERVER_DELETE_CLAIM: "Action"
    OKTA_AUTHORIZATION_SERVER_DELETE_CLIENT_TOKEN: "Action"
    OKTA_AUTHORIZATION_SERVER_DELETE_POLICY_BY_ID: "Action"
    OKTA_AUTHORIZATION_SERVER_DELETE_POLICY_RULE: "Action"
    OKTA_AUTHORIZATION_SERVER_DELETE_SCOPE: "Action"
    OKTA_AUTHORIZATION_SERVER_DELETE_SUCCESS: "Action"
    OKTA_AUTHORIZATION_SERVER_ENUMERATE_POLICY_RULES: "Action"
    OKTA_AUTHORIZATION_SERVER_GET_BY_ID: "Action"
    OKTA_AUTHORIZATION_SERVER_GET_CLAIMS: "Action"
    OKTA_AUTHORIZATION_SERVER_GET_CLAIMS_2: "Action"
    OKTA_AUTHORIZATION_SERVER_GET_CLIENT_AU_TH_TOKEN: "Action"
    OKTA_AUTHORIZATION_SERVER_GET_CLIENT_TOKENS: "Action"
    OKTA_AUTHORIZATION_SERVER_GET_POLICIES: "Action"
    OKTA_AUTHORIZATION_SERVER_GET_POLICIES_SUCCESS: "Action"
    OKTA_AUTHORIZATION_SERVER_GET_POLICY_RULE_BY_ID: "Action"
    OKTA_AUTHORIZATION_SERVER_GET_SCOPES: "Action"
    OKTA_AUTHORIZATION_SERVER_GET_SCOPES_2: "Action"
    OKTA_AUTHORIZATION_SERVER_LIST_CLIENTS: "Action"
    OKTA_AUTHORIZATION_SERVER_LIST_CREDENTIALS_KEYS: "Action"
    OKTA_AUTHORIZATION_SERVER_LIST_SERVERS: "Action"
    OKTA_AUTHORIZATION_SERVER_ROTATE_KEY_LIFECYCLE: "Action"
    OKTA_AUTHORIZATION_SERVER_UPDATE_BY_ID: "Action"
    OKTA_AUTHORIZATION_SERVER_UPDATE_CLAIM_SUCCESS: "Action"
    OKTA_AUTHORIZATION_SERVER_UPDATE_POLICY_RULE_CONFIGURATION: "Action"
    OKTA_AUTHORIZATION_SERVER_UPDATE_POLICY_SUCCESS: "Action"
    OKTA_AUTHORIZATION_SERVER_UPDATE_SCOPE_SUCCESS: "Action"
    OKTA_BRAND_CREATE_EMAIL_TEMPLATE_CUSTOMIZATION: "Action"
    OKTA_BRAND_DELETE_EMAIL_CUSTOMIZATION: "Action"
    OKTA_BRAND_DELETE_EMAIL_TEMPLATE_CUSTOMIZATION_S: "Action"
    OKTA_BRAND_DELETE_THEME_BACKGROUND_IMAGE: "Action"
    OKTA_BRAND_DELETE_THEME_FAVICON: "Action"
    OKTA_BRAND_DELETE_THEME_LOGO: "Action"
    OKTA_BRAND_GET_ALL_BRANDS: "Action"
    OKTA_BRAND_GET_BY_ID: "Action"
    OKTA_BRAND_GET_EMAIL_CUSTOMIZATION_PREVIEW: "Action"
    OKTA_BRAND_GET_EMAIL_TEMPLATE: "Action"
    OKTA_BRAND_GET_EMAIL_TEMPLATE_CUSTOMIZATION_BY_ID: "Action"
    OKTA_BRAND_GET_EMAIL_TEMPLATE_DEFAULT_CONTENT: "Action"
    OKTA_BRAND_GET_EMAIL_TEMPLATE_DEFAULT_CONTENT_PREVIEW: "Action"
    OKTA_BRAND_GET_EMAIL_TEMPLATE_DEFAULT_CONTENT_PREVIEW_2: "Action"
    OKTA_BRAND_GET_THEMES: "Action"
    OKTA_BRAND_GET_THEME_BY_ID: "Action"
    OKTA_BRAND_LIST_EMAIL_TEMPLATES: "Action"
    OKTA_BRAND_LIST_EMAIL_TEMPLATE_CUSTOMIZATION_S: "Action"
    OKTA_BRAND_UPDATE_BY_BRAN_DID: "Action"
    OKTA_BRAND_UPDATE_EMAIL_CUSTOMIZATION: "Action"
    OKTA_BRAND_UPDATE_THEME: "Action"
    OKTA_BRAND_UPDATE_THEME_BACKGROUND_IMAGE: "Action"
    OKTA_BRAND_UPDATE_THEME_FAVICON: "Action"
    OKTA_BRAND_UPDATE_THEME_LOGO: "Action"
    OKTA_DOMAIN_CREATE_CERTIFICATE: "Action"
    OKTA_DOMAIN_CREATE_NEW_DOMAIN: "Action"
    OKTA_DOMAIN_GET_BY_ID: "Action"
    OKTA_DOMAIN_LIST_VERIFIED_CUSTOM: "Action"
    OKTA_DOMAIN_REMOVE_BY_ID: "Action"
    OKTA_DOMAIN_VERIFY_BY_ID: "Action"
    OKTA_EVENT_HOOK_ACTIVATE_LIFECYCLE_SUCCESS: "Action"
    OKTA_EVENT_HOOK_CREATE_SUCCESS: "Action"
    OKTA_EVENT_HOOK_DEACTIVATE_LIFECYCLE_EVENT: "Action"
    OKTA_EVENT_HOOK_GET_SUCCESS_EVENT: "Action"
    OKTA_EVENT_HOOK_LIST_SUCCESS_EVENTS: "Action"
    OKTA_EVENT_HOOK_REMOVE_SUCCESS_EVENT: "Action"
    OKTA_EVENT_HOOK_UPDATE_SUCCESS_EVENT: "Action"
    OKTA_EVENT_HOOK_VERIFY_LIFECYCLE_SUCCESS: "Action"
    OKTA_FEATURE_CREATE_LIFECYCLE_SUCCESS: "Action"
    OKTA_FEATURE_GET_SUCCESS: "Action"
    OKTA_FEATURE_GET_SUCCESS_BY_ID: "Action"
    OKTA_FEATURE_LIST_DEPENDENCIES: "Action"
    OKTA_FEATURE_LIST_DEPENDENTS: "Action"
    OKTA_GROUP_ACTIVATE_RULE_LIFECYCLE: "Action"
    OKTA_GROUP_ADD_APP_INSTANCE_TARGET_TO_APP_ADMIN_ROLE_GIVEN_TO_GROUP: "Action"
    OKTA_GROUP_ADD_RULE: "Action"
    OKTA_GROUP_ADD_USER_TO_GROUP: "Action"
    OKTA_GROUP_ASSIGN_ROLE_TO_GROUP: "Action"
    OKTA_GROUP_CREATE_NEW_GROUP: "Action"
    OKTA_GROUP_DEACTIVATE_RULE_LIFECYCLE: "Action"
    OKTA_GROUP_DELETE_TARGET_GROUP_ROLES_CATALOG_APPS: "Action"
    OKTA_GROUP_ENUMERATE_GROUP_MEMBERS: "Action"
    OKTA_GROUP_GET_ALL_RULES: "Action"
    OKTA_GROUP_GET_GROUP_RULE_BY_ID: "Action"
    OKTA_GROUP_GET_ROLE_LIST: "Action"
    OKTA_GROUP_GET_ROLE_SUCCESS: "Action"
    OKTA_GROUP_GET_ROLE_TARGETS_CATALOG_APPS: "Action"
    OKTA_GROUP_GET_RULES: "Action"
    OKTA_GROUP_LIST: "Action"
    OKTA_GROUP_LIST_ASSIGNED_APPS: "Action"
    OKTA_GROUP_LIST_ROLE_TARGETS_GROUPS: "Action"
    OKTA_GROUP_REMOVE_APP_INSTANCE_TARGET_TO_APP_ADMIN_ROLE_GIVEN_TO_GROUP: "Action"
    OKTA_GROUP_REMOVE_OPERATION: "Action"
    OKTA_GROUP_REMOVE_RULE_BY_ID: "Action"
    OKTA_GROUP_REMOVE_TARGET_GROUP: "Action"
    OKTA_GROUP_REMOVE_USER_FROM: "Action"
    OKTA_GROUP_SCHEMA_GET: "Action"
    OKTA_GROUP_UN_ASSIGN_ROLE: "Action"
    OKTA_GROUP_UPDATE_PROFILE: "Action"
    OKTA_GROUP_UPDATE_ROLES_CATALOG_APPS: "Action"
    OKTA_GROUP_UPDATE_RULE: "Action"
    OKTA_GROUP_UPDATE_TARGET_GROUPS_ROLE: "Action"
    OKTA_IDENTITY_PROVIDER_ACTIVATE_IDP_LIFECYCLE: "Action"
    OKTA_IDENTITY_PROVIDER_ADD_NEW_IDP: "Action"
    OKTA_IDENTITY_PROVIDER_ADD_X_509_CERTIFICATE_PUBLIC_KEY: "Action"
    OKTA_IDENTITY_PROVIDER_CLONE_SIGNING_KEY_CREDENTIAL: "Action"
    OKTA_IDENTITY_PROVIDER_DEACTIVATE_IDP: "Action"
    OKTA_IDENTITY_PROVIDER_DELETE_KEY_CREDENTIAL: "Action"
    OKTA_IDENTITY_PROVIDER_ENUMERATE_IDP_KEYS: "Action"
    OKTA_IDENTITY_PROVIDER_GENERATE_CSR: "Action"
    OKTA_IDENTITY_PROVIDER_GENERATE_NEW_SIGNING_KEY_CREDENTIAL: "Action"
    OKTA_IDENTITY_PROVIDER_GET_BY_IDP: "Action"
    OKTA_IDENTITY_PROVIDER_GET_CSR_BY_IDP: "Action"
    OKTA_IDENTITY_PROVIDER_GET_KEY_CREDENTIAL_BY_IDP: "Action"
    OKTA_IDENTITY_PROVIDER_GET_LINKED_USER_BY_ID: "Action"
    OKTA_IDENTITY_PROVIDER_GET_SIGNING_KEY_CREDENTIAL_BY_IDP: "Action"
    OKTA_IDENTITY_PROVIDER_GET_SOCIAL_AU_TH_TOKENS: "Action"
    OKTA_IDENTITY_PROVIDER_GET_USER: "Action"
    OKTA_IDENTITY_PROVIDER_LINK_USER_TO_IDP_WITHOUT_TRANSACTION: "Action"
    OKTA_IDENTITY_PROVIDER_LIST: "Action"
    OKTA_IDENTITY_PROVIDER_LIST_CSR_S_FOR_CERTIFICATE_SIGNING_REQUESTS: "Action"
    OKTA_IDENTITY_PROVIDER_LIST_SIGNING_KEY_CREDENTIALS: "Action"
    OKTA_IDENTITY_PROVIDER_REMOVE_IDP: "Action"
    OKTA_IDENTITY_PROVIDER_REVOKE_CSR_FOR_IDENTITY_PROVIDER: "Action"
    OKTA_IDENTITY_PROVIDER_UPDATE_CONFIGURATION: "Action"
    OKTA_IDENTITY_PROVIDER_UPDATE_CSR_LIFECYCLE_PUBLISH: "Action"
    OKTA_IDENTITY_PROVIDE_RUN_LINK_USER: "Action"
    OKTA_INLINE_HOOK_ACTIVATE_LIFECYCLE: "Action"
    OKTA_INLINE_HOOK_CREATE_SUCCESS: "Action"
    OKTA_INLINE_HOOK_DEACTIVATE_LIFECYCLE: "Action"
    OKTA_INLINE_HOOK_DELETE_MATCHING_BY_ID: "Action"
    OKTA_INLINE_HOOK_GET_BY_ID: "Action"
    OKTA_INLINE_HOOK_GET_SUCCESS: "Action"
    OKTA_INLINE_HOOK_UPDATE_BY_ID: "Action"
    OKTA_LINKED_OBJECT_CREATE_LINKED_OBJECT: "Action"
    OKTA_LINKED_OBJECT_DELETE_USER_LINKED_OBJECT: "Action"
    OKTA_LINKED_OBJECT_GET_USER_LINKED_OBJECTS: "Action"
    OKTA_LINKED_OBJECT_GET_USER_LINKED_OBJECTS_2: "Action"
    OKTA_LOG_GET_LIST_EVENTS: "Action"
    OKTA_NETWORK_ZONE_ACTIVATE_LIFECYCLE: "Action"
    OKTA_NETWORK_ZONE_CREATE_NEW: "Action"
    OKTA_NETWORK_ZONE_DEACTIVATE_ZONE_LIFECYCLE: "Action"
    OKTA_NETWORK_ZONE_GET_BY_ID: "Action"
    OKTA_NETWORK_ZONE_LIST_ZONES: "Action"
    OKTA_NETWORK_ZONE_REMOVE_ZONE: "Action"
    OKTA_NETWORK_ZONE_UPDATE_ZONE: "Action"
    OKTA_ORG_EXTENDO_KTA_SUPPORT: "Action"
    OKTA_ORG_EXTENDO_KTA_SUPPORT_2: "Action"
    OKTA_ORG_GETO_KTA_COMMUNICATION_SETTINGS: "Action"
    OKTA_ORG_GETO_KTA_SUPPORT_SETTINGS: "Action"
    OKTA_ORG_GET_CONTACT_USER: "Action"
    OKTA_ORG_GET_ORG_PREFERENCES: "Action"
    OKTA_ORG_GET_SETTINGS: "Action"
    OKTA_ORG_GRAN_TO_KTA_SUPPORT_ACCESS: "Action"
    OKTA_ORG_HIDE_END_USER_FOOTER: "Action"
    OKTA_ORG_LIST_CONTACT_TYPES: "Action"
    OKTA_ORG_MAKE_OK_TAU_I_FOOTER_VISIBLE: "Action"
    OKTA_ORG_UPDATE_CONTACT_USER: "Action"
    OKTA_ORG_UPDATE_ORGANIZATION_LOGO: "Action"
    OKTA_ORG_UPDATE_SETTING: "Action"
    OKTA_ORG_UPDATE_SETTINGS: "Action"
    OKTA_OR_GOP_TO_U_TO_KTA_COMMUNICATION_EMAILS: "Action"
    OKTA_OR_GO_PTI_NO_KTA_COMMUNICATION_EMAILS: "Action"
    OKTA_POLICY_ACTIVATE_LIFECYCLE: "Action"
    OKTA_POLICY_ACTIVATE_RULE_LIFECYCLE: "Action"
    OKTA_POLICY_CREATE_NEW_POLICY: "Action"
    OKTA_POLICY_CREATE_RULE: "Action"
    OKTA_POLICY_DEACTIVATE_LIFECYCLE: "Action"
    OKTA_POLICY_DEACTIVATE_RULE_LIFECYCLE: "Action"
    OKTA_POLICY_ENUMERATE_RULES: "Action"
    OKTA_POLICY_GET_ALL_WITH_TYPE: "Action"
    OKTA_POLICY_GET_POLICY: "Action"
    OKTA_POLICY_GET_POLICY_RULE: "Action"
    OKTA_POLICY_REMOVE_POLICY_OPERATION: "Action"
    OKTA_POLICY_REMOVE_RULE: "Action"
    OKTA_POLICY_UPDATE_OPERATION: "Action"
    OKTA_POLICY_UPDATE_RULE: "Action"
    OKTA_PROFILE_MAPPING_GET_BY_ID: "Action"
    OKTA_PROFILE_MAPPING_LIST_WITH_PAGINATION: "Action"
    OKTA_PROFILE_MAPPING_UPDATE_PROPERTY_MAPPINGS: "Action"
    OKTA_SESSION_CLOSE: "Action"
    OKTA_SESSION_CREATE_SESSION_WITH_TOKEN: "Action"
    OKTA_SESSION_GET_DETAILS: "Action"
    OKTA_SESSION_REFRESH_LIFECYCLE: "Action"
    OKTA_SUBSCRIPTION_CUSTOM_ROLE_NOTIFICATION_UNSUBSCRIBE: "Action"
    OKTA_SUBSCRIPTION_GET_ROLE_SUBSCRIPTIONS_BY_NOTIFICATION_TYPE: "Action"
    OKTA_SUBSCRIPTION_LIST_ROLE_SUBSCRIPTIONS: "Action"
    OKTA_SUBSCRIPTION_ROLE_NOTIFICATION_SUBSCRIBE: "Action"
    OKTA_SUBSCRIPTION_UNSUBSCRIBE_USER_SUBSCRIPTION_BY_NOTIFICATION_TYPE: "Action"
    OKTA_SUBSCRIPTION_USER_NOTIFICATION_SUBSCRIBE: "Action"
    OKTA_TEMPLATE_ADD_NEW_CUSTOMS_MS: "Action"
    OKTA_TEMPLATE_ENUMERATES_MS_TEMPLATES: "Action"
    OKTA_TEMPLATE_GET_BY_ID: "Action"
    OKTA_TEMPLATE_PARTIAL_SMS_UPDATE: "Action"
    OKTA_TEMPLATE_REMOVES_MS: "Action"
    OKTA_TEMPLATE_UPDATES_MS_TEMPLATE: "Action"
    OKTA_THREAT_INSIGHT_GET_CURRENT_CONFIGURATION: "Action"
    OKTA_THREAT_INSIGHT_UPDATE_CONFIGURATION: "Action"
    OKTA_TRUSTED_ORIGIN_ACTIVATE_LIFECYCLE_SUCCESS: "Action"
    OKTA_TRUSTED_ORIGIN_CREATE_SUCCESS: "Action"
    OKTA_TRUSTED_ORIGIN_DEACTIVATE_LIFECYCLE_SUCCESS: "Action"
    OKTA_TRUSTED_ORIGIN_DELETE_SUCCESS: "Action"
    OKTA_TRUSTED_ORIGIN_GET_LIST: "Action"
    OKTA_TRUSTED_ORIGIN_GET_SUCCESS_BY_ID: "Action"
    OKTA_TRUSTED_ORIGIN_UPDATE_SUCCESS: "Action"
    OKTA_USER_ACTIVATE_LIFECYCLE: "Action"
    OKTA_USER_ADD_APP_INSTANCE_TARGET_TO_APP_ADMINISTRATOR_ROLE_GIVEN_TO_USER: "Action"
    OKTA_USER_ASSIGN_ROLE: "Action"
    OKTA_USER_CHANGE_PASSWORD_VALIDATION: "Action"
    OKTA_USER_CREATE_NEW_USER: "Action"
    OKTA_USER_DEACTIVATE_LIFECYCLE: "Action"
    OKTA_USER_DELETE_LINKED_OBJECTS: "Action"
    OKTA_USER_DELETE_PERMANENTLY: "Action"
    OKTA_USER_DELETE_TARGET_APP: "Action"
    OKTA_USER_EXPIRE_PASSWORD_AND_GET_TEMPORARY_PASSWORD: "Action"
    OKTA_USER_EXPIRE_PASSWORD_AND_TEMPORARY_PASSWORD: "Action"
    OKTA_USER_FACTOR_ACTIVATE_FACTOR_LIFECYCLE: "Action"
    OKTA_USER_FACTOR_ENROLL_SUPPORTED_FACTOR: "Action"
    OKTA_USER_FACTOR_ENUMERATE_ENROLLED: "Action"
    OKTA_USER_FACTOR_ENUMERATE_SECURITY_QUESTIONS: "Action"
    OKTA_USER_FACTOR_ENUMERATE_SUPPORTED_FACTORS: "Action"
    OKTA_USER_FACTOR_GET_FACTOR: "Action"
    OKTA_USER_FACTOR_POLL_FACTOR_TRANSACTION_STATUS: "Action"
    OKTA_USER_FACTOR_VERIFY_OT_P: "Action"
    OKTA_USER_FACTO_RUN_ENROLL_FACTOR: "Action"
    OKTA_USER_FORGOT_PASSWORD: "Action"
    OKTA_USER_GENERATE_PASSWORD_RESET_TOKEN: "Action"
    OKTA_USER_GETO_KTA_USER: "Action"
    OKTA_USER_GET_ASSIGNED_ROLE: "Action"
    OKTA_USER_GET_CLIENT_REFRESH_TOKEN: "Action"
    OKTA_USER_GET_GRANT_BY_ID: "Action"
    OKTA_USER_GET_LINKED_OBJECTS: "Action"
    OKTA_USER_GET_MEMBER_GROUPS: "Action"
    OKTA_USER_GET_SUBSCRIPTION_BY_NOTIFICATION: "Action"
    OKTA_USER_LIST_ACTIVE_USERS: "Action"
    OKTA_USER_LIST_APP_TARGETS_FOR_ROLE: "Action"
    OKTA_USER_LIST_ASSIGNED_APP_LINKS: "Action"
    OKTA_USER_LIST_ASSIGNED_ROLES: "Action"
    OKTA_USER_LIST_CLIENTS: "Action"
    OKTA_USER_LIST_GRANTS: "Action"
    OKTA_USER_LIST_GRANTS_FOR_CLIENT: "Action"
    OKTA_USER_LIST_I_DPS_FOR_USER: "Action"
    OKTA_USER_LIST_REFRESH_TOKENS_FOR_USER_AND_CLIENT: "Action"
    OKTA_USER_LIST_ROLE_TARGETS_GROUPS: "Action"
    OKTA_USER_LIST_SUBSCRIPTIONS: "Action"
    OKTA_USER_REACTIVATE_USER: "Action"
    OKTA_USER_REMOVE_APP_INSTANCE_TARGET_TO_APP_ADMINISTRATOR_ROLE_GIVEN_TO: "Action"
    OKTA_USER_REMOVE_TARGET_GROUP: "Action"
    OKTA_USER_RESET_FACTORS_OPERATION: "Action"
    OKTA_USER_REVOKE_ALL_SESSIONS: "Action"
    OKTA_USER_REVOKE_ALL_TOKENS: "Action"
    OKTA_USER_REVOKE_GRANT: "Action"
    OKTA_USER_REVOKE_GRANTS: "Action"
    OKTA_USER_REVOKE_GRANTS_FOR_USER_AND_CLIENT: "Action"
    OKTA_USER_REVOKE_TOKEN_FOR_CLIENT: "Action"
    OKTA_USER_SCHEMA_GET_SCHEMA_BY_ID: "Action"
    OKTA_USER_SCHEMA_GET_USER_SCHEMA: "Action"
    OKTA_USER_SUSPEND_LIFECYCLE: "Action"
    OKTA_USER_TYPEREPLACE_EXISTING_TYPE: "Action"
    OKTA_USER_TYPE_CREATE_NEW_USER_TYPE: "Action"
    OKTA_USER_TYPE_DELETE_PERMANENTLY: "Action"
    OKTA_USER_TYPE_GET_ALL_USER_TYPES: "Action"
    OKTA_USER_TYPE_GET_BY_ID: "Action"
    OKTA_USER_TYPE_UPDATE_EXISTING_TYPE: "Action"
    OKTA_USER_UNLOCK_USER_STATUS: "Action"
    OKTA_USER_UPDATE_LINKED_OBJECT: "Action"
    OKTA_USER_UPDATE_PROFILE: "Action"
    OKTA_USER_UPDATE_PROFILE_2: "Action"
    OKTA_USER_UPDATE_RECOVERY_QUESTION: "Action"
    OKTA_USER_UPDATE_ROLES_CATALOG_APPS: "Action"
    OKTA_USER_UPDATE_ROLES_CATALOG_APPS_2: "Action"
    OKTA_USER_UPDATE_ROLES_CATALOG_APPS_3: "Action"
    OKTA_USE_RUN_ASSIGN_ROLE: "Action"
    OKTA_USE_RUN_SUSPEND_LIFECYCLE: "Action"
    PAGERDUTY_ASSOCIATE_SERVICE_TO_INCIDENT_WORK_FLOW_TRIGGER: "Action"
    PAGERDUTY_CONVERT_SERVICE_EVENT_RULES_TO_EVENT_ORCHESTRATION: "Action"
    PAGERDUTY_CREATE_ADDON: "Action"
    PAGERDUTY_CREATE_AUTOMATION_ACTION: "Action"
    PAGERDUTY_CREATE_AUTOMATION_ACTIONS_RUNNER: "Action"
    PAGERDUTY_CREATE_AUTOMATION_ACTIONS_RUNNER_TEAM_ASSOCIATION: "Action"
    PAGERDUTY_CREATE_AUTOMATION_ACTION_INVOCATION: "Action"
    PAGERDUTY_CREATE_AUTOMATION_ACTION_SERVICE_AS_SO_CATION: "Action"
    PAGERDUTY_CREATE_AUTOMATION_ACTION_TEAM_ASSOCIATION: "Action"
    PAGERDUTY_CREATE_BUSINESS_SERVICE: "Action"
    PAGERDUTY_CREATE_BUSINESS_SERVICE_ACCOUNT_SUBSCRIPTION: "Action"
    PAGERDUTY_CREATE_BUSINESS_SERVICE_NOTIFICATION_SUBSCRIBERS: "Action"
    PAGERDUTY_CREATE_CACHE_VAR_ON_GLOBAL_OR_CH: "Action"
    PAGERDUTY_CREATE_CACHE_VAR_ON_SERVICE_OR_CH: "Action"
    PAGERDUTY_CREATE_CHANGE_EVENT: "Action"
    PAGERDUTY_CREATE_CUSTOM_FIELDS_FIELD: "Action"
    PAGERDUTY_CREATE_CUSTOM_FIELDS_FIELD_OPTION: "Action"
    PAGERDUTY_CREATE_ENTITY_TYPE_BY_ID_CHANGE_TAGS: "Action"
    PAGERDUTY_CREATE_ESCALATION_POLICY: "Action"
    PAGERDUTY_CREATE_EXTENSION: "Action"
    PAGERDUTY_CREATE_INCIDENT: "Action"
    PAGERDUTY_CREATE_INCIDENT_NOTE: "Action"
    PAGERDUTY_CREATE_INCIDENT_NOTIFICATION_SUBSCRIBERS: "Action"
    PAGERDUTY_CREATE_INCIDENT_RESPONDER_REQUEST: "Action"
    PAGERDUTY_CREATE_INCIDENT_SNOOZE: "Action"
    PAGERDUTY_CREATE_INCIDENT_STATUS_UPDATE: "Action"
    PAGERDUTY_CREATE_INCIDENT_WORK_FLOW_INSTANCE: "Action"
    PAGERDUTY_CREATE_INCIDENT_WORK_FLOW_TRIGGER: "Action"
    PAGERDUTY_CREATE_MAINTENANCE_WINDOW: "Action"
    PAGERDUTY_CREATE_RESPONSE_PLAY: "Action"
    PAGERDUTY_CREATE_RULE_SET: "Action"
    PAGERDUTY_CREATE_RULE_SET_EVENT_RULE: "Action"
    PAGERDUTY_CREATE_SCHEDULE: "Action"
    PAGERDUTY_CREATE_SCHEDULE_OVERRIDE: "Action"
    PAGERDUTY_CREATE_SCHEDULE_PREVIEW: "Action"
    PAGERDUTY_CREATE_SERVICE: "Action"
    PAGERDUTY_CREATE_SERVICE_DEPENDENCY: "Action"
    PAGERDUTY_CREATE_SERVICE_EVENT_RULE: "Action"
    PAGERDUTY_CREATE_SERVICE_INTEGRATION: "Action"
    PAGERDUTY_CREATE_STATUS_PAGE_POST: "Action"
    PAGERDUTY_CREATE_STATUS_PAGE_POSTMORTEM: "Action"
    PAGERDUTY_CREATE_STATUS_PAGE_POST_UPDATE: "Action"
    PAGERDUTY_CREATE_STATUS_PAGE_SUBSCRIPTION: "Action"
    PAGERDUTY_CREATE_TAGS: "Action"
    PAGERDUTY_CREATE_TEAM: "Action"
    PAGERDUTY_CREATE_TEAM_NOTIFICATION_SUBSCRIPTIONS: "Action"
    PAGERDUTY_CREATE_TEMPLATE: "Action"
    PAGERDUTY_CREATE_USER_CONTACT_METHOD: "Action"
    PAGERDUTY_CREATE_USER_HAND_OFF_NOTIFICATION_RULE: "Action"
    PAGERDUTY_CREATE_USER_NOTIFICATION_RULE: "Action"
    PAGERDUTY_CREATE_USER_NOTIFICATION_SUBSCRIPTIONS: "Action"
    PAGERDUTY_CREATE_USER_STATUS_UPDATE_NOTIFICATION_RULE: "Action"
    PAGERDUTY_CREATE_WEB_HOOK_SUBSCRIPTION: "Action"
    PAGERDUTY_DELETE_ADDON: "Action"
    PAGERDUTY_DELETE_ALERT_GROUPING_SETTING: "Action"
    PAGERDUTY_DELETE_AUTOMATION_ACTION: "Action"
    PAGERDUTY_DELETE_AUTOMATION_ACTIONS_RUNNER: "Action"
    PAGERDUTY_DELETE_AUTOMATION_ACTIONS_RUNNER_TEAM_ASSOCIATION: "Action"
    PAGERDUTY_DELETE_AUTOMATION_ACTION_SERVICE_ASSOCIATION: "Action"
    PAGERDUTY_DELETE_AUTOMATION_ACTION_TEAM_ASSOCIATION: "Action"
    PAGERDUTY_DELETE_BUSINESS_SERVICE: "Action"
    PAGERDUTY_DELETE_BUSINESS_SERVICE_PRIORITY_THRESHOLDS: "Action"
    PAGERDUTY_DELETE_CACHE_VAR_ON_GLOBAL_OR_CH: "Action"
    PAGERDUTY_DELETE_CACHE_VAR_ON_SERVICE_OR_CH: "Action"
    PAGERDUTY_DELETE_CUSTOM_FIELDS_FIELD: "Action"
    PAGERDUTY_DELETE_CUSTOM_FIELDS_FIELD_OPTION: "Action"
    PAGERDUTY_DELETE_ESCALATION_POLICY: "Action"
    PAGERDUTY_DELETE_EXTENSION: "Action"
    PAGERDUTY_DELETE_INCIDENT_WORK_FLOW: "Action"
    PAGERDUTY_DELETE_INCIDENT_WORK_FLOW_TRIGGER: "Action"
    PAGERDUTY_DELETE_MAINTENANCE_WINDOW: "Action"
    PAGERDUTY_DELETE_ORCHESTRATION: "Action"
    PAGERDUTY_DELETE_ORCHESTRATION_INTEGRATION: "Action"
    PAGERDUTY_DELETE_RESPONSE_PLAY: "Action"
    PAGERDUTY_DELETE_RULE_SET: "Action"
    PAGERDUTY_DELETE_RULE_SET_EVENT_RULE: "Action"
    PAGERDUTY_DELETE_SCHEDULE: "Action"
    PAGERDUTY_DELETE_SCHEDULE_OVERRIDE: "Action"
    PAGERDUTY_DELETE_SERVICE: "Action"
    PAGERDUTY_DELETE_SERVICE_DEPENDENCY: "Action"
    PAGERDUTY_DELETE_SERVICE_EVENT_RULE: "Action"
    PAGERDUTY_DELETE_SERVICE_FROM_INCIDENT_WORK_FLOW_TRIGGER: "Action"
    PAGERDUTY_DELETE_STATUS_PAGE_POST: "Action"
    PAGERDUTY_DELETE_STATUS_PAGE_POSTMORTEM: "Action"
    PAGERDUTY_DELETE_STATUS_PAGE_POST_UPDATE: "Action"
    PAGERDUTY_DELETE_STATUS_PAGE_SUBSCRIPTION: "Action"
    PAGERDUTY_DELETE_TAG: "Action"
    PAGERDUTY_DELETE_TEAM: "Action"
    PAGERDUTY_DELETE_TEAM_ESCALATION_POLICY: "Action"
    PAGERDUTY_DELETE_TEAM_USER: "Action"
    PAGERDUTY_DELETE_TEMPLATE: "Action"
    PAGERDUTY_DELETE_USER: "Action"
    PAGERDUTY_DELETE_USER_CONTACT_METHOD: "Action"
    PAGERDUTY_DELETE_USER_HAND_OFF_NOTIFICATION_RULE: "Action"
    PAGERDUTY_DELETE_USER_NOTIFICATION_RULE: "Action"
    PAGERDUTY_DELETE_USER_SESSION: "Action"
    PAGERDUTY_DELETE_USER_SESSIONS: "Action"
    PAGERDUTY_DELETE_USER_STATUS_UPDATE_NOTIFICATION_RULE: "Action"
    PAGERDUTY_DELETE_WEB_HOOK_SUBSCRIPTION: "Action"
    PAGERDUTY_ENABLE_EXTENSION: "Action"
    PAGERDUTY_ENABLE_WEB_HOOK_SUBSCRIPTION: "Action"
    PAGERDUTY_GET_ABILITY: "Action"
    PAGERDUTY_GET_ADDON: "Action"
    PAGERDUTY_GET_ALERT_GROUPING_SETTING: "Action"
    PAGERDUTY_GET_ALL_AUTOMATION_ACTIONS: "Action"
    PAGERDUTY_GET_ANALYTICS_INCIDENTS: "Action"
    PAGERDUTY_GET_ANALYTICS_INCIDENTS_BY_ID: "Action"
    PAGERDUTY_GET_ANALYTICS_INCIDENT_RESPONSES_BY_ID: "Action"
    PAGERDUTY_GET_ANALYTICS_METRICS_INCIDENTS_ALL: "Action"
    PAGERDUTY_GET_ANALYTICS_METRICS_INCIDENTS_ESCALATION_POLICY: "Action"
    PAGERDUTY_GET_ANALYTICS_METRICS_INCIDENTS_ESCALATION_POLICY_ALL: "Action"
    PAGERDUTY_GET_ANALYTICS_METRICS_INCIDENTS_SERVICE: "Action"
    PAGERDUTY_GET_ANALYTICS_METRICS_INCIDENTS_SERVICE_ALL: "Action"
    PAGERDUTY_GET_ANALYTICS_METRICS_INCIDENTS_TEAM: "Action"
    PAGERDUTY_GET_ANALYTICS_METRICS_INCIDENTS_TEAM_ALL: "Action"
    PAGERDUTY_GET_ANALYTICS_METRICS_RESPONDERS_ALL: "Action"
    PAGERDUTY_GET_ANALYTICS_METRICS_RESPONDER_STEAM: "Action"
    PAGERDUTY_GET_ANALYTICS_RESPONDER_INCIDENTS: "Action"
    PAGERDUTY_GET_AUTOMATION_ACTION: "Action"
    PAGERDUTY_GET_AUTOMATION_ACTIONS_ACTION_SERVICE_ASSOCIATION: "Action"
    PAGERDUTY_GET_AUTOMATION_ACTIONS_ACTION_SERVICE_ASSOCIATIONS: "Action"
    PAGERDUTY_GET_AUTOMATION_ACTIONS_ACTION_TEAM_ASSOCIATION: "Action"
    PAGERDUTY_GET_AUTOMATION_ACTIONS_ACTION_TEAM_ASSOCIATIONS: "Action"
    PAGERDUTY_GET_AUTOMATION_ACTIONS_INVOCATION: "Action"
    PAGERDUTY_GET_AUTOMATION_ACTIONS_RUNNER: "Action"
    PAGERDUTY_GET_AUTOMATION_ACTIONS_RUNNERS: "Action"
    PAGERDUTY_GET_AUTOMATION_ACTIONS_RUNNER_TEAM_ASSOCIATION: "Action"
    PAGERDUTY_GET_AUTOMATION_ACTIONS_RUNNER_TEAM_ASSOCIATIONS: "Action"
    PAGERDUTY_GET_BUSINESS_SERVICE: "Action"
    PAGERDUTY_GET_BUSINESS_SERVICE_IMPACTS: "Action"
    PAGERDUTY_GET_BUSINESS_SERVICE_PRIORITY_THRESHOLDS: "Action"
    PAGERDUTY_GET_BUSINESS_SERVICE_SERVICE_DEPENDENCIES: "Action"
    PAGERDUTY_GET_BUSINESS_SERVICE_SUBSCRIBERS: "Action"
    PAGERDUTY_GET_BUSINESS_SERVICE_SUPPORTING_SERVICE_IMPACTS: "Action"
    PAGERDUTY_GET_BUSINESS_SERVICE_TOP_LEVEL_IMPACTORS: "Action"
    PAGERDUTY_GET_CACHE_VAR_ON_GLOBAL_OR_CH: "Action"
    PAGERDUTY_GET_CACHE_VAR_ON_SERVICE_OR_CH: "Action"
    PAGERDUTY_GET_CHANGE_EVENT: "Action"
    PAGERDUTY_GET_CURRENT_USER: "Action"
    PAGERDUTY_GET_CUSTOM_FIELDS_FIELD: "Action"
    PAGERDUTY_GET_ENTITY_TYPE_BY_ID_TAGS: "Action"
    PAGERDUTY_GET_ESCALATION_POLICY: "Action"
    PAGERDUTY_GET_EXTENSION: "Action"
    PAGERDUTY_GET_EXTENSION_SCHEMA: "Action"
    PAGERDUTY_GET_INCIDENT: "Action"
    PAGERDUTY_GET_INCIDENT_ALERT: "Action"
    PAGERDUTY_GET_INCIDENT_FIELD_VALUES: "Action"
    PAGERDUTY_GET_INCIDENT_IMPACTED_BUSINESS_SERVICES: "Action"
    PAGERDUTY_GET_INCIDENT_NOTIFICATION_SUBSCRIBERS: "Action"
    PAGERDUTY_GET_INCIDENT_WORK_FLOW: "Action"
    PAGERDUTY_GET_INCIDENT_WORK_FLOW_ACTION: "Action"
    PAGERDUTY_GET_INCIDENT_WORK_FLOW_TRIGGER: "Action"
    PAGERDUTY_GET_LOG_ENTRY: "Action"
    PAGERDUTY_GET_MAINTENANCE_WINDOW: "Action"
    PAGERDUTY_GET_ORCHESTRATION: "Action"
    PAGERDUTY_GET_ORCHESTRATION_INTEGRATION: "Action"
    PAGERDUTY_GET_OR_CHPA_THUN_ROUTED: "Action"
    PAGERDUTY_GET_OUTLIER_INCIDENT: "Action"
    PAGERDUTY_GET_PAST_INCIDENTS: "Action"
    PAGERDUTY_GET_PAUSED_INCIDENT_REPORT_ALERTS: "Action"
    PAGERDUTY_GET_PAUSED_INCIDENT_REPORT_COUNTS: "Action"
    PAGERDUTY_GET_POSTMORTEM: "Action"
    PAGERDUTY_GET_POST_UPDATE: "Action"
    PAGERDUTY_GET_RELATED_INCIDENTS: "Action"
    PAGERDUTY_GET_RESPONSE_PLAY: "Action"
    PAGERDUTY_GET_RULE_SET: "Action"
    PAGERDUTY_GET_RULE_SET_EVENT_RULE: "Action"
    PAGERDUTY_GET_SCHEDULE: "Action"
    PAGERDUTY_GET_SERVICE: "Action"
    PAGERDUTY_GET_SERVICE_EVENT_RULE: "Action"
    PAGERDUTY_GET_SERVICE_INTEGRATION: "Action"
    PAGERDUTY_GET_STATUS_DASHBOARD_BY_ID: "Action"
    PAGERDUTY_GET_STATUS_DASHBOARD_BY_URL_SLUG: "Action"
    PAGERDUTY_GET_STATUS_DASHBOARD_SERVICE_IMPACTS_BY_ID: "Action"
    PAGERDUTY_GET_STATUS_DASHBOARD_SERVICE_IMPACTS_BY_URL_SLUG: "Action"
    PAGERDUTY_GET_STATUS_PAGE_IMPACT: "Action"
    PAGERDUTY_GET_STATUS_PAGE_POST: "Action"
    PAGERDUTY_GET_STATUS_PAGE_SERVICE: "Action"
    PAGERDUTY_GET_STATUS_PAGE_SEVERITY: "Action"
    PAGERDUTY_GET_STATUS_PAGE_STATUS: "Action"
    PAGERDUTY_GET_STATUS_PAGE_SUBSCRIPTION: "Action"
    PAGERDUTY_GET_TAG: "Action"
    PAGERDUTY_GET_TAGS_BY_ENTITY_TYPE: "Action"
    PAGERDUTY_GET_TEAM: "Action"
    PAGERDUTY_GET_TEAM_NOTIFICATION_SUBSCRIPTIONS: "Action"
    PAGERDUTY_GET_TECHNICAL_SERVICE_SERVICE_DEPENDENCIES: "Action"
    PAGERDUTY_GET_TEMPLATE: "Action"
    PAGERDUTY_GET_TEMPLATES: "Action"
    PAGERDUTY_GET_TEMPLATE_FIELDS: "Action"
    PAGERDUTY_GET_USER: "Action"
    PAGERDUTY_GET_USER_CONTACT_METHOD: "Action"
    PAGERDUTY_GET_USER_CONTACT_METHODS: "Action"
    PAGERDUTY_GET_USER_HAND_OFF_NOTIFICATION_RULES: "Action"
    PAGERDUTY_GET_USER_HAND_OFF_NOT_IFI_ACTION_RULE: "Action"
    PAGERDUTY_GET_USER_LICENSE: "Action"
    PAGERDUTY_GET_USER_NOTIFICATION_RULE: "Action"
    PAGERDUTY_GET_USER_NOTIFICATION_RULES: "Action"
    PAGERDUTY_GET_USER_NOTIFICATION_SUBSCRIPTIONS: "Action"
    PAGERDUTY_GET_USER_SESSION: "Action"
    PAGERDUTY_GET_USER_SESSIONS: "Action"
    PAGERDUTY_GET_USER_STATUS_UPDATE_NOTIFICATION_RULE: "Action"
    PAGERDUTY_GET_USER_STATUS_UPDATE_NOTIFICATION_RULES: "Action"
    PAGERDUTY_GET_VENDOR: "Action"
    PAGERDUTY_GET_WEB_HOOK_SUBSCRIPTION: "Action"
    PAGERDUTY_GE_TORCH_ACTIVE_STATUS: "Action"
    PAGERDUTY_GE_TORCH_PATH_GLOBAL: "Action"
    PAGERDUTY_GE_TORCH_PATH_ROUTER: "Action"
    PAGERDUTY_GE_TORCH_PATH_SERVICE: "Action"
    PAGERDUTY_LIST_ABILITIES: "Action"
    PAGERDUTY_LIST_ADDON: "Action"
    PAGERDUTY_LIST_ALERT_GROUPING_SETTINGS: "Action"
    PAGERDUTY_LIST_AUDIT_RECORDS: "Action"
    PAGERDUTY_LIST_AUTOMATION_ACTION_INVOCATIONS: "Action"
    PAGERDUTY_LIST_BUSINESS_SERVICES: "Action"
    PAGERDUTY_LIST_CACHE_VAR_ON_GLOBAL_OR_CH: "Action"
    PAGERDUTY_LIST_CACHE_VAR_ON_SERVICE_OR_CH: "Action"
    PAGERDUTY_LIST_CHANGE_EVENTS: "Action"
    PAGERDUTY_LIST_CUSTOM_FIELDS_FIELDS: "Action"
    PAGERDUTY_LIST_CUSTOM_FIELDS_FIELD_OPTIONS: "Action"
    PAGERDUTY_LIST_ESCALATION_POLICIES: "Action"
    PAGERDUTY_LIST_ESCALATION_POLICY_AUDIT_RECORDS: "Action"
    PAGERDUTY_LIST_EVENT_ORCHESTRATIONS: "Action"
    PAGERDUTY_LIST_EXTENSIONS: "Action"
    PAGERDUTY_LIST_EXTENSION_SCHEMAS: "Action"
    PAGERDUTY_LIST_INCIDENTS: "Action"
    PAGERDUTY_LIST_INCIDENT_ALERTS: "Action"
    PAGERDUTY_LIST_INCIDENT_LOG_ENTRIES: "Action"
    PAGERDUTY_LIST_INCIDENT_NOTES: "Action"
    PAGERDUTY_LIST_INCIDENT_RELATED_CHANGE_EVENTS: "Action"
    PAGERDUTY_LIST_INCIDENT_WORK_FLOWS: "Action"
    PAGERDUTY_LIST_INCIDENT_WORK_FLOW_ACTIONS: "Action"
    PAGERDUTY_LIST_INCIDENT_WORK_FLOW_TRIGGERS: "Action"
    PAGERDUTY_LIST_LICENSES: "Action"
    PAGERDUTY_LIST_LICENSE_ALLOCATIONS: "Action"
    PAGERDUTY_LIST_LOG_ENTRIES: "Action"
    PAGERDUTY_LIST_MAINTENANCE_WINDOWS: "Action"
    PAGERDUTY_LIST_NOTIFICATIONS: "Action"
    PAGERDUTY_LIST_ON_CALLS: "Action"
    PAGERDUTY_LIST_ORCHESTRATION_INTEGRATIONS: "Action"
    PAGERDUTY_LIST_PRIORITIES: "Action"
    PAGERDUTY_LIST_RESOURCE_STANDARDS: "Action"
    PAGERDUTY_LIST_RESOURCE_STANDARDS_MANY_SERVICES: "Action"
    PAGERDUTY_LIST_RESPONSE_PLAYS: "Action"
    PAGERDUTY_LIST_RULE_SETS: "Action"
    PAGERDUTY_LIST_RULE_SET_EVENT_RULES: "Action"
    PAGERDUTY_LIST_SCHEDULES: "Action"
    PAGERDUTY_LIST_SCHEDULES_AUDIT_RECORDS: "Action"
    PAGERDUTY_LIST_SCHEDULE_OVERRIDES: "Action"
    PAGERDUTY_LIST_SCHEDULE_USERS: "Action"
    PAGERDUTY_LIST_SERVICES: "Action"
    PAGERDUTY_LIST_SERVICE_AUDIT_RECORDS: "Action"
    PAGERDUTY_LIST_SERVICE_CHANGE_EVENTS: "Action"
    PAGERDUTY_LIST_SERVICE_EVENT_RULES: "Action"
    PAGERDUTY_LIST_STANDARDS: "Action"
    PAGERDUTY_LIST_STATUS_DASHBOARDS: "Action"
    PAGERDUTY_LIST_STATUS_PAGES: "Action"
    PAGERDUTY_LIST_STATUS_PAGESE_VERITIES: "Action"
    PAGERDUTY_LIST_STATUS_PAGE_IMPACTS: "Action"
    PAGERDUTY_LIST_STATUS_PAGE_POSTS: "Action"
    PAGERDUTY_LIST_STATUS_PAGE_POST_UPDATES: "Action"
    PAGERDUTY_LIST_STATUS_PAGE_SERVICES: "Action"
    PAGERDUTY_LIST_STATUS_PAGE_STATUSES: "Action"
    PAGERDUTY_LIST_STATUS_PAGE_SUBSCRIPTIONS: "Action"
    PAGERDUTY_LIST_TAGS: "Action"
    PAGERDUTY_LIST_TEAMS: "Action"
    PAGERDUTY_LIST_TEAMS_AUDIT_RECORDS: "Action"
    PAGERDUTY_LIST_TEAM_USERS: "Action"
    PAGERDUTY_LIST_USERS: "Action"
    PAGERDUTY_LIST_USERS_AUDIT_RECORDS: "Action"
    PAGERDUTY_LIST_VENDORS: "Action"
    PAGERDUTY_LIST_WEB_HOOK_SUBSCRIPTIONS: "Action"
    PAGERDUTY_MERGE_INCIDENTS: "Action"
    PAGERDUTY_MIGRATE_ORCHESTRATION_INTEGRATION: "Action"
    PAGERDUTY_POST_ALERT_GROUPING_SETTINGS: "Action"
    PAGERDUTY_POST_INCIDENT_WORK_FLOW: "Action"
    PAGERDUTY_POST_ORCHESTRATION: "Action"
    PAGERDUTY_POST_ORCHESTRATION_INTEGRATION: "Action"
    PAGERDUTY_PUT_ALERT_GROUPING_SETTING: "Action"
    PAGERDUTY_PUT_BUSINESS_SERVICE_PRIORITY_THRESHOLDS: "Action"
    PAGERDUTY_PUT_INCIDENT_MANUAL_BUSINESS_SERVICE_ASSOCIATION: "Action"
    PAGERDUTY_PUT_INCIDENT_WORK_FLOW: "Action"
    PAGERDUTY_REMOVE_BUSINESS_SERVICE_ACCOUNT_SUBSCRIPTION: "Action"
    PAGERDUTY_REMOVE_BUSINESS_SERVICE_NOTIFICATION_SUBSCRIBER: "Action"
    PAGERDUTY_REMOVE_INCIDENT_NOTIFICATION_SUBSCRIBERS: "Action"
    PAGERDUTY_REMOVE_TEAM_NOTIFICATION_SUBSCRIPTIONS: "Action"
    PAGERDUTY_RENDER_TEMPLATE: "Action"
    PAGERDUTY_RUN_RESPONSE_PLAY: "Action"
    PAGERDUTY_SET_INCIDENT_FIELD_VALUES: "Action"
    PAGERDUTY_TEST_WEB_HOOK_SUBSCRIPTION: "Action"
    PAGERDUTY_UNSUBSCRIBE_USER_NOTIFICATION_SUBSCRIPTIONS: "Action"
    PAGERDUTY_UPDATE_ADDON: "Action"
    PAGERDUTY_UPDATE_AUTOMATION_ACTION: "Action"
    PAGERDUTY_UPDATE_AUTOMATION_ACTIONS_RUNNER: "Action"
    PAGERDUTY_UPDATE_BUSINESS_SERVICE: "Action"
    PAGERDUTY_UPDATE_CACHE_VAR_ON_GLOBAL_OR_CH: "Action"
    PAGERDUTY_UPDATE_CACHE_VAR_ON_SERVICE_OR_CH: "Action"
    PAGERDUTY_UPDATE_CUSTOM_FIELDS_FIELD: "Action"
    PAGERDUTY_UPDATE_CUSTOM_FIELDS_FIELD_OPTION: "Action"
    PAGERDUTY_UPDATE_ESCALATION_POLICY: "Action"
    PAGERDUTY_UPDATE_EXTENSION: "Action"
    PAGERDUTY_UPDATE_INCIDENT: "Action"
    PAGERDUTY_UPDATE_INCIDENTS: "Action"
    PAGERDUTY_UPDATE_INCIDENT_ALERT: "Action"
    PAGERDUTY_UPDATE_INCIDENT_ALERTS: "Action"
    PAGERDUTY_UPDATE_INCIDENT_WORK_FLOW_TRIGGER: "Action"
    PAGERDUTY_UPDATE_LOG_ENTRY_CHANNEL: "Action"
    PAGERDUTY_UPDATE_MAINTENANCE_WINDOW: "Action"
    PAGERDUTY_UPDATE_ORCHESTRATION: "Action"
    PAGERDUTY_UPDATE_ORCHESTRATION_INTEGRATION: "Action"
    PAGERDUTY_UPDATE_OR_CHPA_THUN_ROUTED: "Action"
    PAGERDUTY_UPDATE_OR_CH_ACTIVE_STATUS: "Action"
    PAGERDUTY_UPDATE_OR_CH_PATH_GLOBAL: "Action"
    PAGERDUTY_UPDATE_OR_CH_PATH_ROUTER: "Action"
    PAGERDUTY_UPDATE_OR_CH_PATH_SERVICE: "Action"
    PAGERDUTY_UPDATE_RESPONSE_PLAY: "Action"
    PAGERDUTY_UPDATE_RULE_SET: "Action"
    PAGERDUTY_UPDATE_RULE_SET_EVENT_RULE: "Action"
    PAGERDUTY_UPDATE_SCHEDULE: "Action"
    PAGERDUTY_UPDATE_SERVICE: "Action"
    PAGERDUTY_UPDATE_SERVICE_EVENT_RULE: "Action"
    PAGERDUTY_UPDATE_SERVICE_INTEGRATION: "Action"
    PAGERDUTY_UPDATE_STANDARD: "Action"
    PAGERDUTY_UPDATE_STATUS_PAGE_POST: "Action"
    PAGERDUTY_UPDATE_STATUS_PAGE_POSTMORTEM: "Action"
    PAGERDUTY_UPDATE_STATUS_PAGE_POST_UPDATE: "Action"
    PAGERDUTY_UPDATE_TEAM: "Action"
    PAGERDUTY_UPDATE_TEAM_ESCALATION_POLICY: "Action"
    PAGERDUTY_UPDATE_TEAM_USER: "Action"
    PAGERDUTY_UPDATE_TEMPLATE: "Action"
    PAGERDUTY_UPDATE_USER_CONTACT_METHOD: "Action"
    PAGERDUTY_UPDATE_USER_HAND_OFF_NOTIFICATION: "Action"
    PAGERDUTY_UPDATE_USER_NOTIFICATION_RULE: "Action"
    PAGERDUTY_UPDATE_USER_STATUS_UPDATE_NOTIFICATION_RULE: "Action"
    PAGERDUTY_UPDATE_WEB_HOOK_SUBSCRIPTION: "Action"
    PERPLEXITYAI_PERPLEXITY_AI_SEARCH: "Action"
    RAGTOOL_ADD_CONTENT_TO_RAG_TOOL: "Action"
    RAGTOOL_RAG_TOOL_QUERY: "Action"
    SCHEDULER_SCHEDULE_JOB_ACTION: "Action"
    SEARCHTOOL_FIND_FILE_CMD: "Action"
    SEARCHTOOL_SEARCH_DIR_CMD: "Action"
    SEARCHTOOL_SEARCH_FILE_CMD: "Action"
    SERPAPI_DUCK_DUCK_GO_SEARCH: "Action"
    SERPAPI_EVENT_SEARCH: "Action"
    SERPAPI_FINANCE_SEARCH: "Action"
    SERPAPI_IMAGE_SEARCH: "Action"
    SERPAPI_NEWS_SEARCH: "Action"
    SERPAPI_SCHOLAR_SEARCH: "Action"
    SERPAPI_SEARCH: "Action"
    SERPAPI_SHOPPING_SEARCH: "Action"
    SERPAPI_TRENDS_SEARCH: "Action"
    SHELL_CREATE_SHELL: "Action"
    SHELL_EXEC_COMMAND: "Action"
    SLACKBOT_API_TEST: "Action"
    SLACKBOT_APPS_EVENT_AUTHORIZATIONS_GET_LIST: "Action"
    SLACKBOT_APPS_PERMISSIONS_ADDITIONAL_SCOPES_REQUEST: "Action"
    SLACKBOT_APPS_PERMISSIONS_LIST_PERMISSIONS: "Action"
    SLACKBOT_APPS_PERMISSIONS_RESOURCES_GET_RESOURCES_LIST: "Action"
    SLACKBOT_APPS_PERMISSIONS_SCOPES_GET_LIST: "Action"
    SLACKBOT_APPS_PERMISSIONS_USERS_LIST_USER_GRANTS: "Action"
    SLACKBOT_APPS_PERMISSIONS_USERS_REQUEST_MODAL: "Action"
    SLACKBOT_APPS_UNINSTALL: "Action"
    SLACKBOT_AU_TH_REVOKE: "Action"
    SLACKBOT_AU_TH_TEST: "Action"
    SLACKBOT_BOTS_INFO: "Action"
    SLACKBOT_CALLS_ADD: "Action"
    SLACKBOT_CALLS_END: "Action"
    SLACKBOT_CALLS_INFO: "Action"
    SLACKBOT_CALLS_PARTICIPANTS_ADD_NEW_PARTICIPANT: "Action"
    SLACKBOT_CALLS_PARTICIPANTS_REGISTER_REMOVED: "Action"
    SLACKBOT_CALLS_UPDATE: "Action"
    SLACKBOT_CHAT_DELETE: "Action"
    SLACKBOT_CHAT_DELETE_SCHEDULED_MESSAGE: "Action"
    SLACKBOT_CHAT_GET_PERMALINK: "Action"
    SLACKBOT_CHAT_ME_MESSAGE: "Action"
    SLACKBOT_CHAT_POST_EPHEMERAL: "Action"
    SLACKBOT_CHAT_POST_MESSAGE: "Action"
    SLACKBOT_CHAT_SCHEDULED_MESSAGES_LIST: "Action"
    SLACKBOT_CHAT_SCHEDULE_MESSAGE: "Action"
    SLACKBOT_CHAT_UNFURL: "Action"
    SLACKBOT_CHAT_UPDATE: "Action"
    SLACKBOT_CONVERSATIONS_ARCHIVE: "Action"
    SLACKBOT_CONVERSATIONS_CLOSE: "Action"
    SLACKBOT_CONVERSATIONS_CREATE: "Action"
    SLACKBOT_CONVERSATIONS_HISTORY: "Action"
    SLACKBOT_CONVERSATIONS_INFO: "Action"
    SLACKBOT_CONVERSATIONS_INVITE: "Action"
    SLACKBOT_CONVERSATIONS_JOIN: "Action"
    SLACKBOT_CONVERSATIONS_KICK: "Action"
    SLACKBOT_CONVERSATIONS_LEAVE: "Action"
    SLACKBOT_CONVERSATIONS_LIST: "Action"
    SLACKBOT_CONVERSATIONS_MARK: "Action"
    SLACKBOT_CONVERSATIONS_MEMBERS: "Action"
    SLACKBOT_CONVERSATIONS_OPEN: "Action"
    SLACKBOT_CONVERSATIONS_RENAME: "Action"
    SLACKBOT_CONVERSATIONS_REPLIES: "Action"
    SLACKBOT_CONVERSATIONS_SET_PURPOSE: "Action"
    SLACKBOT_CONVERSATIONS_SET_TOPIC: "Action"
    SLACKBOT_CONVERSATION_SUN_ARCHIVE: "Action"
    SLACKBOT_DIALOG_OPEN: "Action"
    SLACKBOT_DND_END_DND: "Action"
    SLACKBOT_DND_END_SNOOZE: "Action"
    SLACKBOT_DND_INFO: "Action"
    SLACKBOT_DND_SET_SNOOZE: "Action"
    SLACKBOT_DND_TEAM_INFO: "Action"
    SLACKBOT_E_MOJI_LIST: "Action"
    SLACKBOT_FILES_COMMENTS_DELETE_COMMENT: "Action"
    SLACKBOT_FILES_DELETE: "Action"
    SLACKBOT_FILES_INFO: "Action"
    SLACKBOT_FILES_LIST: "Action"
    SLACKBOT_FILES_REMOTE_ADD_FROM_REMOTE: "Action"
    SLACKBOT_FILES_REMOTE_DELETE_FILE: "Action"
    SLACKBOT_FILES_REMOTE_GET_INFO: "Action"
    SLACKBOT_FILES_REMOTE_LIST_REMOTE_FILES: "Action"
    SLACKBOT_FILES_REMOTE_SHARE_REMOTE_FILE: "Action"
    SLACKBOT_FILES_REMOTE_UPDATE_REMOTE_FILE: "Action"
    SLACKBOT_FILES_REVOKE_PUBLIC_URL: "Action"
    SLACKBOT_FILES_SHARED_PUBLIC_URL: "Action"
    SLACKBOT_FILES_UPLOAD: "Action"
    SLACKBOT_MIGRATION_EXCHANGE: "Action"
    SLACKBOT_OAUTH_ACCESS: "Action"
    SLACKBOT_OAUTH_TOKEN: "Action"
    SLACKBOT_OAUTH_V_2_EXCHANGE_TOKEN: "Action"
    SLACKBOT_PINS_ADD: "Action"
    SLACKBOT_PINS_LIST: "Action"
    SLACKBOT_PINS_REMOVE: "Action"
    SLACKBOT_REACTIONS_ADD: "Action"
    SLACKBOT_REACTIONS_GET: "Action"
    SLACKBOT_REACTIONS_LIST: "Action"
    SLACKBOT_REACTIONS_REMOVE: "Action"
    SLACKBOT_REMINDERS_COMPLETE: "Action"
    SLACKBOT_REMINDERS_DELETE: "Action"
    SLACKBOT_REMINDERS_INFO: "Action"
    SLACKBOT_REMINDERS_LIST: "Action"
    SLACKBOT_RT_M_CONNECT: "Action"
    SLACKBOT_STARS_ADD: "Action"
    SLACKBOT_STARS_LIST: "Action"
    SLACKBOT_STARS_REMOVE: "Action"
    SLACKBOT_TEAMBILL_ABLE_INFO: "Action"
    SLACKBOT_TEAM_ACCESS_LOGS: "Action"
    SLACKBOT_TEAM_INFO: "Action"
    SLACKBOT_TEAM_INTEGRATION_LOGS: "Action"
    SLACKBOT_TEAM_PROFILE_GET_PROFILE: "Action"
    SLACKBOT_USERS_CONVERSATIONS: "Action"
    SLACKBOT_USERS_DELETE_PHOTO: "Action"
    SLACKBOT_USERS_GET_PRESENCE: "Action"
    SLACKBOT_USERS_IDENTITY: "Action"
    SLACKBOT_USERS_INFO: "Action"
    SLACKBOT_USERS_LIST: "Action"
    SLACKBOT_USERS_LOOKUP_BY_EMAIL: "Action"
    SLACKBOT_USERS_PROFILE_GET_PROFILE_INFO: "Action"
    SLACKBOT_USERS_PROFILE_SET_PROFILE_INFO: "Action"
    SLACKBOT_USERS_SET_ACTIVE: "Action"
    SLACKBOT_USERS_SET_PHOTO: "Action"
    SLACKBOT_USERS_SET_PRESENCE: "Action"
    SLACKBOT_USER_GROUPS_CREATE: "Action"
    SLACKBOT_USER_GROUPS_DISABLE: "Action"
    SLACKBOT_USER_GROUPS_ENABLE: "Action"
    SLACKBOT_USER_GROUPS_LIST: "Action"
    SLACKBOT_USER_GROUPS_UPDATE: "Action"
    SLACKBOT_USER_GROUPS_USERS_LIST_ALL_USERS: "Action"
    SLACKBOT_USER_GROUPS_USERS_UPDATE_LIST: "Action"
    SLACKBOT_VIEWS_OPEN: "Action"
    SLACKBOT_VIEWS_PUBLISH: "Action"
    SLACKBOT_VIEWS_PUSH: "Action"
    SLACKBOT_VIEWS_UPDATE: "Action"
    SLACKBOT_WORK_FLOWS_STEP_COMPLETED: "Action"
    SLACKBOT_WORK_FLOWS_STEP_FAILED: "Action"
    SLACKBOT_WORK_FLOWS_UPDATE_STEP: "Action"
    SLACK_ADMIN_APPS_APPROVED_LIST: "Action"
    SLACK_ADMIN_APPS_APPROVE_APP_INSTALLATION: "Action"
    SLACK_ADMIN_APPS_REQUESTS_LIST: "Action"
    SLACK_ADMIN_APPS_RESTRICTED_GET_LIST: "Action"
    SLACK_ADMIN_APPS_RESTRICT_APP: "Action"
    SLACK_ADMIN_CONVERSATIONS_ARCHIVE_CHANNEL: "Action"
    SLACK_ADMIN_CONVERSATIONS_CONVERT_TO_PRIVATE_CHANNEL: "Action"
    SLACK_ADMIN_CONVERSATIONS_CREATE_CHANNEL_BASED_CONVERSATION: "Action"
    SLACK_ADMIN_CONVERSATIONS_DELETE_CHANNEL: "Action"
    SLACK_ADMIN_CONVERSATIONS_DISCONNECT_SHARED_CHANNEL: "Action"
    SLACK_ADMIN_CONVERSATIONS_E_KM_LIST_ORIGINAL_CONNECTED_CHANNEL_INFO: "Action"
    SLACK_ADMIN_CONVERSATIONS_GET_CONVERSATION_PREF_S: "Action"
    SLACK_ADMIN_CONVERSATIONS_GET_TEAMS_LIST: "Action"
    SLACK_ADMIN_CONVERSATIONS_INVITE_USER_TO_CHANNEL: "Action"
    SLACK_ADMIN_CONVERSATIONS_RENAME_CHANNEL: "Action"
    SLACK_ADMIN_CONVERSATIONS_RESTRICT_ACCESS_ADD_GROUP_IDP_GROUPS: "Action"
    SLACK_ADMIN_CONVERSATIONS_RESTRICT_ACCESS_LIST_GROUPS: "Action"
    SLACK_ADMIN_CONVERSATIONS_RESTRICT_ACCESS_REMOVE_IDP_GROUP: "Action"
    SLACK_ADMIN_CONVERSATIONS_SEARCH_CHANNELS: "Action"
    SLACK_ADMIN_CONVERSATIONS_SET_CONVERSATION_PREF_S: "Action"
    SLACK_ADMIN_CONVERSATIONS_SET_TEAMS_WORK_SPACE_CONNECTION: "Action"
    SLACK_ADMIN_CONVERSATION_SUN_ARCHIVE_CHANNEL: "Action"
    SLACK_ADMIN_E_MOJI_ADDE_MOJI: "Action"
    SLACK_ADMIN_E_MOJI_ALIAS_ADD: "Action"
    SLACK_ADMIN_E_MOJI_LIST_ENTERPRISE_E_MOJI: "Action"
    SLACK_ADMIN_E_MOJI_REMOVE_ENTERPRISE_E_MOJI: "Action"
    SLACK_ADMIN_E_MOJI_RENAME_E_MOJI: "Action"
    SLACK_ADMIN_INVITE_REQUESTS_APPROVED_LIST: "Action"
    SLACK_ADMIN_INVITE_REQUESTS_APPROVE_REQUEST: "Action"
    SLACK_ADMIN_INVITE_REQUESTS_DENIED_LIST: "Action"
    SLACK_ADMIN_INVITE_REQUESTS_DENY_REQUEST: "Action"
    SLACK_ADMIN_INVITE_REQUESTS_LIST_PENDING_WORK_SPACE_INVITE_REQUESTS: "Action"
    SLACK_ADMIN_TEAMS_ADMINS_GET_ALL: "Action"
    SLACK_ADMIN_TEAMS_CREATE_ENTERPRISE_TEAM: "Action"
    SLACK_ADMIN_TEAMS_LIST_ALL: "Action"
    SLACK_ADMIN_TEAMS_OWNERS_LIST_OWNERS: "Action"
    SLACK_ADMIN_TEAMS_SETTINGS_GET_INFO: "Action"
    SLACK_ADMIN_TEAMS_SETTINGS_SET_DEFAULT_CHANNELS: "Action"
    SLACK_ADMIN_TEAMS_SETTINGS_SET_DESCRIPTION: "Action"
    SLACK_ADMIN_TEAMS_SETTINGS_SET_DISCOVER_ABILITY_OF_WORK_SPACE: "Action"
    SLACK_ADMIN_TEAMS_SETTINGS_SET_ICON: "Action"
    SLACK_ADMIN_TEAMS_SETTINGS_SET_NAME: "Action"
    SLACK_ADMIN_USERS_ADD_WORK_SPACE_USER: "Action"
    SLACK_ADMIN_USERS_INVITE_USER_TO_WORK_SPACE: "Action"
    SLACK_ADMIN_USERS_LIST_WORK_SPACE_USERS: "Action"
    SLACK_ADMIN_USERS_REMOVE_USER_FROM_WORK_SPACE: "Action"
    SLACK_ADMIN_USERS_SESSION_INVALIDATE_SESSION: "Action"
    SLACK_ADMIN_USERS_SESSION_RESET_SESSIONS: "Action"
    SLACK_ADMIN_USERS_SET_ADMIN_USER: "Action"
    SLACK_ADMIN_USERS_SET_EXPIRATION_GUEST: "Action"
    SLACK_ADMIN_USERS_SET_REGULAR_USER: "Action"
    SLACK_ADMIN_USERS_SET_WORK_SPACE_OWNER: "Action"
    SLACK_ADMIN_USER_GROUPS_ADD_DEFAULT_CHANNELS: "Action"
    SLACK_ADMIN_USER_GROUPS_ASSOCIATE_DEFAULT_WORK_SPACES: "Action"
    SLACK_ADMIN_USER_GROUPS_LIST_CHANNELS_GET: "Action"
    SLACK_ADMIN_USER_GROUPS_REMOVE_CHANNELS: "Action"
    SLACK_API_TEST: "Action"
    SLACK_APPS_EVENT_AUTHORIZATIONS_GET_LIST: "Action"
    SLACK_APPS_PERMISSIONS_ADDITIONAL_SCOPES_REQUEST: "Action"
    SLACK_APPS_PERMISSIONS_LIST_PERMISSIONS: "Action"
    SLACK_APPS_PERMISSIONS_RESOURCES_GET_RESOURCES_LIST: "Action"
    SLACK_APPS_PERMISSIONS_SCOPES_GET_LIST: "Action"
    SLACK_APPS_PERMISSIONS_USERS_LIST_USER_GRANTS: "Action"
    SLACK_APPS_PERMISSIONS_USERS_REQUEST_MODAL: "Action"
    SLACK_APPS_UNINSTALL: "Action"
    SLACK_AU_TH_REVOKE: "Action"
    SLACK_AU_TH_TEST: "Action"
    SLACK_BOTS_INFO: "Action"
    SLACK_CALLS_ADD: "Action"
    SLACK_CALLS_END: "Action"
    SLACK_CALLS_INFO: "Action"
    SLACK_CALLS_PARTICIPANTS_ADD_NEW_PARTICIPANT: "Action"
    SLACK_CALLS_PARTICIPANTS_REGISTER_REMOVED: "Action"
    SLACK_CALLS_UPDATE: "Action"
    SLACK_CHAT_DELETE: "Action"
    SLACK_CHAT_DELETE_SCHEDULED_MESSAGE: "Action"
    SLACK_CHAT_GET_PERMALINK: "Action"
    SLACK_CHAT_ME_MESSAGE: "Action"
    SLACK_CHAT_POST_EPHEMERAL: "Action"
    SLACK_CHAT_POST_MESSAGE: "Action"
    SLACK_CHAT_SCHEDULED_MESSAGES_LIST: "Action"
    SLACK_CHAT_SCHEDULE_MESSAGE: "Action"
    SLACK_CHAT_UNFURL: "Action"
    SLACK_CHAT_UPDATE: "Action"
    SLACK_CONVERSATIONS_ARCHIVE: "Action"
    SLACK_CONVERSATIONS_CLOSE: "Action"
    SLACK_CONVERSATIONS_CREATE: "Action"
    SLACK_CONVERSATIONS_HISTORY: "Action"
    SLACK_CONVERSATIONS_INFO: "Action"
    SLACK_CONVERSATIONS_INVITE: "Action"
    SLACK_CONVERSATIONS_JOIN: "Action"
    SLACK_CONVERSATIONS_KICK: "Action"
    SLACK_CONVERSATIONS_LEAVE: "Action"
    SLACK_CONVERSATIONS_LIST: "Action"
    SLACK_CONVERSATIONS_MARK: "Action"
    SLACK_CONVERSATIONS_MEMBERS: "Action"
    SLACK_CONVERSATIONS_OPEN: "Action"
    SLACK_CONVERSATIONS_RENAME: "Action"
    SLACK_CONVERSATIONS_REPLIES: "Action"
    SLACK_CONVERSATIONS_SET_PURPOSE: "Action"
    SLACK_CONVERSATIONS_SET_TOPIC: "Action"
    SLACK_CONVERSATION_SUN_ARCHIVE: "Action"
    SLACK_DIALOG_OPEN: "Action"
    SLACK_DND_END_DND: "Action"
    SLACK_DND_END_SNOOZE: "Action"
    SLACK_DND_INFO: "Action"
    SLACK_DND_SET_SNOOZE: "Action"
    SLACK_DND_TEAM_INFO: "Action"
    SLACK_E_MOJI_LIST: "Action"
    SLACK_FILES_COMMENTS_DELETE_COMMENT: "Action"
    SLACK_FILES_DELETE: "Action"
    SLACK_FILES_INFO: "Action"
    SLACK_FILES_LIST: "Action"
    SLACK_FILES_REMOTE_ADD_FROM_REMOTE: "Action"
    SLACK_FILES_REMOTE_DELETE_FILE: "Action"
    SLACK_FILES_REMOTE_GET_INFO: "Action"
    SLACK_FILES_REMOTE_LIST_REMOTE_FILES: "Action"
    SLACK_FILES_REMOTE_SHARE_REMOTE_FILE: "Action"
    SLACK_FILES_REMOTE_UPDATE_REMOTE_FILE: "Action"
    SLACK_FILES_REVOKE_PUBLIC_URL: "Action"
    SLACK_FILES_SHARED_PUBLIC_URL: "Action"
    SLACK_FILES_UPLOAD: "Action"
    SLACK_MIGRATION_EXCHANGE: "Action"
    SLACK_OAUTH_ACCESS: "Action"
    SLACK_OAUTH_TOKEN: "Action"
    SLACK_OAUTH_V_2_EXCHANGE_TOKEN: "Action"
    SLACK_PINS_ADD: "Action"
    SLACK_PINS_LIST: "Action"
    SLACK_PINS_REMOVE: "Action"
    SLACK_REACTIONS_ADD: "Action"
    SLACK_REACTIONS_GET: "Action"
    SLACK_REACTIONS_LIST: "Action"
    SLACK_REACTIONS_REMOVE: "Action"
    SLACK_REMINDERS_ADD: "Action"
    SLACK_REMINDERS_COMPLETE: "Action"
    SLACK_REMINDERS_DELETE: "Action"
    SLACK_REMINDERS_INFO: "Action"
    SLACK_REMINDERS_LIST: "Action"
    SLACK_RT_M_CONNECT: "Action"
    SLACK_SEARCH_MESSAGES: "Action"
    SLACK_STARS_ADD: "Action"
    SLACK_STARS_LIST: "Action"
    SLACK_STARS_REMOVE: "Action"
    SLACK_TEAMBILL_ABLE_INFO: "Action"
    SLACK_TEAM_ACCESS_LOGS: "Action"
    SLACK_TEAM_INFO: "Action"
    SLACK_TEAM_INTEGRATION_LOGS: "Action"
    SLACK_TEAM_PROFILE_GET_PROFILE: "Action"
    SLACK_USERS_CONVERSATIONS: "Action"
    SLACK_USERS_DELETE_PHOTO: "Action"
    SLACK_USERS_GET_PRESENCE: "Action"
    SLACK_USERS_IDENTITY: "Action"
    SLACK_USERS_INFO: "Action"
    SLACK_USERS_LIST: "Action"
    SLACK_USERS_LOOKUP_BY_EMAIL: "Action"
    SLACK_USERS_PROFILE_GET_PROFILE_INFO: "Action"
    SLACK_USERS_PROFILE_SET_PROFILE_INFO: "Action"
    SLACK_USERS_SET_ACTIVE: "Action"
    SLACK_USERS_SET_PHOTO: "Action"
    SLACK_USERS_SET_PRESENCE: "Action"
    SLACK_USER_GROUPS_CREATE: "Action"
    SLACK_USER_GROUPS_DISABLE: "Action"
    SLACK_USER_GROUPS_ENABLE: "Action"
    SLACK_USER_GROUPS_LIST: "Action"
    SLACK_USER_GROUPS_UPDATE: "Action"
    SLACK_USER_GROUPS_USERS_LIST_ALL_USERS: "Action"
    SLACK_USER_GROUPS_USERS_UPDATE_LIST: "Action"
    SLACK_VIEWS_OPEN: "Action"
    SLACK_VIEWS_PUBLISH: "Action"
    SLACK_VIEWS_PUSH: "Action"
    SLACK_VIEWS_UPDATE: "Action"
    SLACK_WORK_FLOWS_STEP_COMPLETED: "Action"
    SLACK_WORK_FLOWS_STEP_FAILED: "Action"
    SLACK_WORK_FLOWS_UPDATE_STEP: "Action"
    SNOWFLAKE_DESCRIBE_TABLE: "Action"
    SNOWFLAKE_EXPLORE_COLUMNS: "Action"
    SNOWFLAKE_RUN_QUERY: "Action"
    SNOWFLAKE_SHOW_TABLES: "Action"
    SOUNDCLOUD_LIKESPLAYLIST: "Action"
    SOUNDCLOUD_LIKESTRACKACTION: "Action"
    SOUNDCLOUD_LIKESUNLIKEPLAYLIST: "Action"
    SOUNDCLOUD_LIKESUNLIKETRACK: "Action"
    SOUNDCLOUD_MEDELETEFOLLOWEDUSER: "Action"
    SOUNDCLOUD_MEFOLLOWUSER: "Action"
    SOUNDCLOUD_MEGETACTIVITIES: "Action"
    SOUNDCLOUD_MEGETFOLLOWEDUSER: "Action"
    SOUNDCLOUD_MEGETFOLLOWEDUSERS: "Action"
    SOUNDCLOUD_MEGETFOLLOWERBYID: "Action"
    SOUNDCLOUD_MEGETFOLLOWERSLIST: "Action"
    SOUNDCLOUD_MEGETLIKEDPLAYLISTS: "Action"
    SOUNDCLOUD_MEGETRECENTACTIVITIES: "Action"
    SOUNDCLOUD_MEGETRECENTTRACKS: "Action"
    SOUNDCLOUD_MEGETUSERINFORMATION: "Action"
    SOUNDCLOUD_MELISTFOLLOWEDTRACKS: "Action"
    SOUNDCLOUD_MELISTLIKEDTRACKS: "Action"
    SOUNDCLOUD_MELISTPLAYLISTSINFOTRACKSOWNER: "Action"
    SOUNDCLOUD_MELISTUSERTRACKS: "Action"
    SOUNDCLOUD_MISCELLANEOUSRESOLVESOUNDCLOUDURLS: "Action"
    SOUNDCLOUD_OAUTHAUTHORIZEUSER: "Action"
    SOUNDCLOUD_OAUTHPROVISIONACCESSTOKEN: "Action"
    SOUNDCLOUD_PLAYLISTSCREATENEWPLAYLIST: "Action"
    SOUNDCLOUD_PLAYLISTSDELETEPLAYLIST: "Action"
    SOUNDCLOUD_PLAYLISTSGETPLAYLISTBYID: "Action"
    SOUNDCLOUD_PLAYLISTSGETTRACKS: "Action"
    SOUNDCLOUD_PLAYLISTSLISTREPOSTERS: "Action"
    SOUNDCLOUD_PLAYLISTSUPDATEPLAYLISTBYID: "Action"
    SOUNDCLOUD_REPOSTSPLAYLISTASAUTHENTICATEDUSER: "Action"
    SOUNDCLOUD_REPOSTSREMOVEREPOST: "Action"
    SOUNDCLOUD_REPOSTSREMOVEREPOSTONPLAYLIST: "Action"
    SOUNDCLOUD_REPOSTSTRACKASAUTHENTICATEDUSER: "Action"
    SOUNDCLOUD_SEARCHBYQUERY: "Action"
    SOUNDCLOUD_SEARCHBYQUERY2: "Action"
    SOUNDCLOUD_SEARCHUSERQUERY: "Action"
    SOUNDCLOUD_TRACKSCREATECOMMENT: "Action"
    SOUNDCLOUD_TRACKSDELETETRACK: "Action"
    SOUNDCLOUD_TRACKSGETBYID: "Action"
    SOUNDCLOUD_TRACKSGETCOMMENTS: "Action"
    SOUNDCLOUD_TRACKSGETFAVORITERS: "Action"
    SOUNDCLOUD_TRACKSGETRELATEDTRACKS: "Action"
    SOUNDCLOUD_TRACKSGETSTREAMABLEURLS: "Action"
    SOUNDCLOUD_TRACKSLISTREPOSTERS: "Action"
    SOUNDCLOUD_TRACKSUPDATETRACKINFORMATION: "Action"
    SOUNDCLOUD_TRACKSUPLOADNEWTRACK: "Action"
    SOUNDCLOUD_USERSGETFOLLOWERS: "Action"
    SOUNDCLOUD_USERSGETFOLLOWINGBYID: "Action"
    SOUNDCLOUD_USERSGETUSER: "Action"
    SOUNDCLOUD_USERSGETUSERFOLLOWINGS: "Action"
    SOUNDCLOUD_USERSGETUSERPLAYLISTS: "Action"
    SOUNDCLOUD_USERSGETUSERTRACKS: "Action"
    SOUNDCLOUD_USERSGETUSERWEBPROFILES: "Action"
    SOUNDCLOUD_USERSLISTFAVORITES: "Action"
    SOUNDCLOUD_USERSLISTLIKEDPLAYLISTS: "Action"
    SOUNDCLOUD_USERSLISTLIKEDTRACKS: "Action"
    SPIDERTOOL_CRAWL: "Action"
    SPIDERTOOL_SCRAPE: "Action"
    SPLITWISE_ADD_USER_TO_GROUP: "Action"
    SPLITWISE_DELETE_GROUP: "Action"
    SPLITWISE_GET_CURRENT_USER_INFO: "Action"
    SPLITWISE_GET_GROUP: "Action"
    SPLITWISE_GET_USER_INFO: "Action"
    SPLITWISE_REMOVE_USER_FROM_GROUP: "Action"
    SPLITWISE_RESTORE_GROUP: "Action"
    SPOTIFY_ADD_TO_QUEUE: "Action"
    SPOTIFY_ADD_TRACKS_TO_PLAYLIST: "Action"
    SPOTIFY_CHANGE_PLAYLIST_DETAILS: "Action"
    SPOTIFY_CHECKUSERS_SAVED_ALBUMS: "Action"
    SPOTIFY_CHECKUSERS_SAVED_AUDIOBOOKS: "Action"
    SPOTIFY_CHECKUSERS_SAVED_EPISODES: "Action"
    SPOTIFY_CHECKUSERS_SAVED_SHOWS: "Action"
    SPOTIFY_CHECKUSERS_SAVED_TRACKS: "Action"
    SPOTIFY_CHECK_CURRENT_USER_FOLLOWS: "Action"
    SPOTIFY_CHECK_IF_USER_FOLLOWS_PLAYLIST: "Action"
    SPOTIFY_CREATE_PLAYLIST: "Action"
    SPOTIFY_FOLLOW_ARTISTS_USERS: "Action"
    SPOTIFY_FOLLOW_PLAYLIST: "Action"
    SPOTIFY_GET_AN_ALBUM: "Action"
    SPOTIFY_GET_AN_ALBUMS_TRACKS: "Action"
    SPOTIFY_GET_AN_ARTIST: "Action"
    SPOTIFY_GET_AN_ARTISTS_ALBUMS: "Action"
    SPOTIFY_GET_AN_ARTISTS_RELATED_ARTISTS: "Action"
    SPOTIFY_GET_AN_ARTISTS_TOP_TRACKS: "Action"
    SPOTIFY_GET_AN_AUDIOBOOK: "Action"
    SPOTIFY_GET_AN_EPISODE: "Action"
    SPOTIFY_GET_AUDIOBOOK_CHAPTERS: "Action"
    SPOTIFY_GET_AUDIO_ANALYSIS: "Action"
    SPOTIFY_GET_AUDIO_FEATURES: "Action"
    SPOTIFY_GET_AVAILABLE_MARKETS: "Action"
    SPOTIFY_GET_A_CATEGORIES_PLAYLISTS: "Action"
    SPOTIFY_GET_A_CATEGORY: "Action"
    SPOTIFY_GET_A_CHAPTER: "Action"
    SPOTIFY_GET_A_LIST_OF_CURRENT_USERS_PLAYLISTS: "Action"
    SPOTIFY_GET_A_SHOW: "Action"
    SPOTIFY_GET_A_SHOWS_EPISODES: "Action"
    SPOTIFY_GET_A_USERS_AVAILABLE_DEVICES: "Action"
    SPOTIFY_GET_CATEGORIES: "Action"
    SPOTIFY_GET_CURRENT_USERS_PROFILE: "Action"
    SPOTIFY_GET_FEATURED_PLAYLISTS: "Action"
    SPOTIFY_GET_FOLLOWED: "Action"
    SPOTIFY_GET_INFORMATION_ABOUT_THE_USERS_CURRENT_PLAYBACK: "Action"
    SPOTIFY_GET_LIST_USERS_PLAYLISTS: "Action"
    SPOTIFY_GET_MULTIPLE_ALBUMS: "Action"
    SPOTIFY_GET_MULTIPLE_ARTISTS: "Action"
    SPOTIFY_GET_MULTIPLE_AUDIOBOOKS: "Action"
    SPOTIFY_GET_MULTIPLE_EPISODES: "Action"
    SPOTIFY_GET_MULTIPLE_SHOWS: "Action"
    SPOTIFY_GET_NEW_RELEASES: "Action"
    SPOTIFY_GET_PLAYLIST: "Action"
    SPOTIFY_GET_PLAYLISTS_TRACKS: "Action"
    SPOTIFY_GET_PLAYLIST_COVER: "Action"
    SPOTIFY_GET_QUEUE: "Action"
    SPOTIFY_GET_RECENTLY_PLAYED: "Action"
    SPOTIFY_GET_RECOMMENDATIONS: "Action"
    SPOTIFY_GET_RECOMMENDATION_GENRES: "Action"
    SPOTIFY_GET_SEVERAL_AUDIO_FEATURES: "Action"
    SPOTIFY_GET_SEVERAL_CHAPTERS: "Action"
    SPOTIFY_GET_SEVERAL_TRACKS: "Action"
    SPOTIFY_GET_THE_USERS_CURRENTLY_PLAYING_TRACK: "Action"
    SPOTIFY_GET_TRACK: "Action"
    SPOTIFY_GET_USERS_PROFILE: "Action"
    SPOTIFY_GET_USERS_SAVED_ALBUMS: "Action"
    SPOTIFY_GET_USERS_SAVED_AUDIOBOOKS: "Action"
    SPOTIFY_GET_USERS_SAVED_EPISODES: "Action"
    SPOTIFY_GET_USERS_SAVED_SHOWS: "Action"
    SPOTIFY_GET_USERS_SAVED_TRACKS: "Action"
    SPOTIFY_GET_USERS_TOP_ARTISTS: "Action"
    SPOTIFY_GET_USERS_TOP_TRACKS: "Action"
    SPOTIFY_PAUSE_A_USERS_PLAYBACK: "Action"
    SPOTIFY_REMOVE_ALBUMS_USER: "Action"
    SPOTIFY_REMOVE_AUDIOBOOKS_USER: "Action"
    SPOTIFY_REMOVE_EPISODES_USER: "Action"
    SPOTIFY_REMOVE_SHOWS_USER: "Action"
    SPOTIFY_REMOVE_TRACKS_PLAYLIST: "Action"
    SPOTIFY_REMOVE_TRACKS_USER: "Action"
    SPOTIFY_REORDER_OR_REPLACE_PLAYLISTS_TRACKS: "Action"
    SPOTIFY_SAVE_ALBUMS_USER: "Action"
    SPOTIFY_SAVE_AUDIOBOOKS_USER: "Action"
    SPOTIFY_SAVE_EPISODES_USER: "Action"
    SPOTIFY_SAVE_SHOWS_USER: "Action"
    SPOTIFY_SAVE_TRACKS_USER: "Action"
    SPOTIFY_SEARCH: "Action"
    SPOTIFY_SEEK_TO_POSITION_IN_CURRENTLY_PLAYING_TRACK: "Action"
    SPOTIFY_SET_REPEAT_MODE_ON_USERS_PLAYBACK: "Action"
    SPOTIFY_SET_VOLUME_FOR_USERS_PLAYBACK: "Action"
    SPOTIFY_SKIP_USERS_PLAYBACK_TO_NEXT_TRACK: "Action"
    SPOTIFY_SKIP_USERS_PLAYBACK_TO_PREVIOUS_TRACK: "Action"
    SPOTIFY_START_A_USERS_PLAYBACK: "Action"
    SPOTIFY_TOGGLE_SHUFFLE_FOR_USERS_PLAYBACK: "Action"
    SPOTIFY_TRANSFER_A_USERS_PLAYBACK: "Action"
    SPOTIFY_UN_FOLLOW_ARTISTS_USERS: "Action"
    SPOTIFY_UN_FOLLOW_PLAYLIST: "Action"
    SPOTIFY_UPLOAD_CUSTOM_PLAYLIST_COVER: "Action"
    SQLTOOL_SQL_QUERY: "Action"
    STRAVA_CREATE_ACTIVITY: "Action"
    STRAVA_CREATE_UPLOAD: "Action"
    STRAVA_EXPLORE_SEGMENTS: "Action"
    STRAVA_GET_ACTIVITY_BY_ID: "Action"
    STRAVA_GET_ACTIVITY_STREAMS: "Action"
    STRAVA_GET_CLUB_ACTIVITIES_BY_ID: "Action"
    STRAVA_GET_CLUB_ADMINS_BY_ID: "Action"
    STRAVA_GET_CLUB_BY_ID: "Action"
    STRAVA_GET_CLUB_MEMBERS_BY_ID: "Action"
    STRAVA_GET_COMMENTS_BY_ACTIVITY_ID: "Action"
    STRAVA_GET_EFFORTS_BY_SEGMENT_ID: "Action"
    STRAVA_GET_GEAR_BY_ID: "Action"
    STRAVA_GET_KU_DOERS_BY_ACTIVITY_ID: "Action"
    STRAVA_GET_LAPS_BY_ACTIVITY_ID: "Action"
    STRAVA_GET_LOGGED_IN_ATHLETE: "Action"
    STRAVA_GET_LOGGED_IN_ATHLETE_ACTIVITIES: "Action"
    STRAVA_GET_LOGGED_IN_ATHLETE_CLUBS: "Action"
    STRAVA_GET_LOGGED_IN_ATHLETE_STARRED_SEGMENTS: "Action"
    STRAVA_GET_LOGGED_IN_ATHLETE_ZONES: "Action"
    STRAVA_GET_ROUTES_BY_ATHLETE_ID: "Action"
    STRAVA_GET_ROUTE_ASG_PX: "Action"
    STRAVA_GET_ROUTE_BY_ID: "Action"
    STRAVA_GET_ROUTE_STREAMS: "Action"
    STRAVA_GET_ROUT_EAST_CX: "Action"
    STRAVA_GET_SEGMENT_BY_ID: "Action"
    STRAVA_GET_SEGMENT_EFFORT_BY_ID: "Action"
    STRAVA_GET_SEGMENT_EFFORT_STREAMS: "Action"
    STRAVA_GET_SEGMENT_STREAMS: "Action"
    STRAVA_GET_STATS: "Action"
    STRAVA_GET_UPLOAD_BY_ID: "Action"
    STRAVA_GET_ZONES_BY_ACTIVITY_ID: "Action"
    STRAVA_STAR_SEGMENT: "Action"
    STRAVA_UPDATE_ACTIVITY_BY_ID: "Action"
    STRAVA_UPDATE_LOGGED_IN_ATHLETE: "Action"
    TASKADE_COMPLETE_TASK_IN_PROJECT: "Action"
    TASKADE_COPY_PROJECT_TO_FOLDER: "Action"
    TASKADE_CREATE_PROJECTIN_FOLDER: "Action"
    TASKADE_CREATE_PROJECT_IN_WORKSPACE: "Action"
    TASKADE_CREATE_TASK_IN_PROJECT: "Action"
    TASKADE_CREATE_UPDATE_DATE_FOR_TASK: "Action"
    TASKADE_DELETE_TASK_OFA_TASK: "Action"
    TASKADE_DELETE_TASK_PROJECT: "Action"
    TASKADE_ENABLE_PUBLIC_ACCESS_AGENT: "Action"
    TASKADE_ENABLE_SHARE_LINK_IN_PROJECT: "Action"
    TASKADE_GET_ALL_FOLDERS_FOR_WORKSPACE: "Action"
    TASKADE_GET_ALL_PROJECTSIN_FOLDER: "Action"
    TASKADE_GET_ALL_PROJECTS_MINE: "Action"
    TASKADE_GET_ALL_TASKS_FOR_PROJECT: "Action"
    TASKADE_GET_ALL_WORKSPACES_FOR_USER: "Action"
    TASKADE_GET_SHARE_LINK_FOR_PROJECT: "Action"
    TASKADE_GET_TASK_WITH_ID: "Action"
    TASKADE_REMOVE_ASSIGNEE_FROM_TASK: "Action"
    TASKADE_TASK_ASSIGNMENT: "Action"
    TAVILY_TAVILY_SEARCH: "Action"
    TRELLO_ACTION_GET_BOARD_BY_ID_ACTION: "Action"
    TRELLO_ACTION_GET_BOARD_BY_ID_ACTION_BY_FIELD: "Action"
    TRELLO_ACTION_GET_BY_ID: "Action"
    TRELLO_ACTION_GET_BY_ID_ACTION_FIELD: "Action"
    TRELLO_ACTION_GET_CARD_BY_ID_ACTION: "Action"
    TRELLO_ACTION_GET_CARD_BY_ID_ACTION_BY_FIELD: "Action"
    TRELLO_ACTION_GET_DISPLAY_BY_ID_ACTION: "Action"
    TRELLO_ACTION_GET_ENTITIES_BY_ID_ACTION: "Action"
    TRELLO_ACTION_GET_LIST_BY_ID_ACTION: "Action"
    TRELLO_ACTION_GET_LIST_BY_ID_ACTION_BY_FIELD: "Action"
    TRELLO_ACTION_GET_MEMBER_BY_FIELD: "Action"
    TRELLO_ACTION_GET_MEMBER_BY_ID_ACTION: "Action"
    TRELLO_ACTION_GET_MEMBER_BY_ID_ACTION_BY_FIELD: "Action"
    TRELLO_ACTION_GET_MEMBER_CREATOR_BY_ID_ACTION: "Action"
    TRELLO_ACTION_GET_ORGANIZATION_BY_ID_ACTION: "Action"
    TRELLO_ACTION_GET_ORG_BY_ID_ACTION_BY_FIELD: "Action"
    TRELLO_ACTION_REMOVE_BY_ID_ACTION: "Action"
    TRELLO_ACTION_UPDATE_BY_ID_ACTION: "Action"
    TRELLO_ACTION_UPDATE_TEXT_BY_ID_ACTION: "Action"
    TRELLO_BATCH_GET_DATA: "Action"
    TRELLO_BOARD_ADD_CHECKLISTS: "Action"
    TRELLO_BOARD_ADD_LABELS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_ADD_POWER_UPS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_CREATE_BOARD: "Action"
    TRELLO_BOARD_CREATE_LISTS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_FILTER_CARDS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_GENERATE_CALENDAR_KEY_BY_ID: "Action"
    TRELLO_BOARD_GENERATE_EMAIL_KEY: "Action"
    TRELLO_BOARD_GET_BOARD_STARS_BY_ID: "Action"
    TRELLO_BOARD_GET_BY_ID: "Action"
    TRELLO_BOARD_GET_BY_ID_FIELD: "Action"
    TRELLO_BOARD_GET_CARDS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_GET_CARDS_BY_ID_BOARD_BY_ID_CARD: "Action"
    TRELLO_BOARD_GET_CHECKLISTS_BY_ID: "Action"
    TRELLO_BOARD_GET_DELTAS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_GET_LABELS_BY_ID_BOARD_BY_ID_LABEL: "Action"
    TRELLO_BOARD_GET_LISTS_BY_FILTER: "Action"
    TRELLO_BOARD_GET_LISTS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_GET_MEMBERSHIPS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_GET_MEMBERSHIPS_BY_ID_BOARD_BY_ID_MEMBERSHIP: "Action"
    TRELLO_BOARD_GET_MEMBERS_BY_FILTER: "Action"
    TRELLO_BOARD_GET_MEMBERS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_GET_MEMBERS_CARDS_BY_ID_BOARD_BY_ID_MEMBER: "Action"
    TRELLO_BOARD_GET_MEMBERS_INVITED_BY_FIELD: "Action"
    TRELLO_BOARD_GET_MEMBERS_INVITED_BY_ID_BOARD: "Action"
    TRELLO_BOARD_GET_MY_PREF_S_BY_ID: "Action"
    TRELLO_BOARD_GET_ORGANIZATION_BY_ID: "Action"
    TRELLO_BOARD_GET_ORGANIZATION_BY_ID_BOARD_BY_FIELD: "Action"
    TRELLO_BOARD_LIST_ACTIONS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_LIST_LABELS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_MARK_AS_VIEWED_BY_ID_BOARD: "Action"
    TRELLO_BOARD_REMOVE_MEMBER: "Action"
    TRELLO_BOARD_REMOVE_POWER_UP: "Action"
    TRELLO_BOARD_UPDATE_BY_ID: "Action"
    TRELLO_BOARD_UPDATE_CLOSED_BY_ID: "Action"
    TRELLO_BOARD_UPDATE_DESCRIPTION_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_LABEL_NAMES_BLUE_BY_ID: "Action"
    TRELLO_BOARD_UPDATE_LABEL_NAMES_GREEN_BY_ID_BOARD_PUT: "Action"
    TRELLO_BOARD_UPDATE_LABEL_NAMES_ORANGE_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_LABEL_NAMES_PURPLE_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_LABEL_NAMES_RED: "Action"
    TRELLO_BOARD_UPDATE_LABEL_NAMES_YELLOW_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_MEMBERSHIPS_BY_ID_BOARD_BY_ID_MEMBERSHIP: "Action"
    TRELLO_BOARD_UPDATE_MEMBERS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_MEMBERS_BY_ID_BOARD_BY_ID_MEMBER: "Action"
    TRELLO_BOARD_UPDATE_MY_PREF_S_EMAIL_LIST_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_MY_PREF_S_EMAIL_POSITION_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_MY_PREF_S_SHOW_LIST_GUIDE_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_MY_PREF_S_SHOW_SIDEBAR: "Action"
    TRELLO_BOARD_UPDATE_MY_PREF_S_SHOW_SIDEBAR_ACTIONS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_MY_PREF_S_SHOW_SIDEBAR_ACTIVITY_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_NAME_BY_ID: "Action"
    TRELLO_BOARD_UPDATE_ORGANIZATION_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_PREF_S_BACKGROUND_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_PREF_S_CALENDAR_FEED_ENABLED_BY_ID: "Action"
    TRELLO_BOARD_UPDATE_PREF_S_CARD_AGING_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_PREF_S_CARD_COVERS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_PREF_S_COMMENTS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_PREF_S_INVITATIONS_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_PREF_S_PERMISSION_LEVEL_BY_ID: "Action"
    TRELLO_BOARD_UPDATE_PREF_S_SELF_JOIN_BY_ID_BOARD: "Action"
    TRELLO_BOARD_UPDATE_PREF_S_SHOW_SIDEBAR_MEMBERS_BY_ID: "Action"
    TRELLO_BOARD_UPDATE_SUBSCRIBED_BY_ID: "Action"
    TRELLO_BOARD_UPDATE_VOTING_PREF_S_BY_ID: "Action"
    TRELLO_CARD_ADD_ACTIONS_COMMENTS_BY_ID_CARD: "Action"
    TRELLO_CARD_ADD_ATTACHMENTS_BY_ID_CARD: "Action"
    TRELLO_CARD_ADD_CHECKLISTS: "Action"
    TRELLO_CARD_ADD_CHECKLIST_CHECK_ITEM: "Action"
    TRELLO_CARD_ADD_LABELS: "Action"
    TRELLO_CARD_ADD_MEMBERS: "Action"
    TRELLO_CARD_ADD_MEMBERS_VOTED: "Action"
    TRELLO_CARD_ADD_STICKERS_BY_ID_CARD: "Action"
    TRELLO_CARD_AD_DID_LABELS_TO_CARD: "Action"
    TRELLO_CARD_CONVERT_CHECK_ITEM_TO_CARD: "Action"
    TRELLO_CARD_CREATE_AND_UPDATE: "Action"
    TRELLO_CARD_DELETE_ATTACHMENTS_BY_ID_CARD_BY_ID_ATTACHMENT: "Action"
    TRELLO_CARD_DELETE_CHECKLIST_BY_ID_CARD_BY_ID_CHECKLIST: "Action"
    TRELLO_CARD_GET_ATTACHMENTS_BY_IDS: "Action"
    TRELLO_CARD_GET_ATTACHMENTS_BY_ID_CARD: "Action"
    TRELLO_CARD_GET_BOARD_BY_ID: "Action"
    TRELLO_CARD_GET_BOARD_BY_ID_CARD_BY_FIELD: "Action"
    TRELLO_CARD_GET_BY_ID: "Action"
    TRELLO_CARD_GET_BY_ID_FIELD: "Action"
    TRELLO_CARD_GET_CARDS_LIST_BY_ID_CARD_BY_FIELD: "Action"
    TRELLO_CARD_GET_CHECKLISTS_BY_ID: "Action"
    TRELLO_CARD_GET_CHECK_ITEM_STATES_BY_ID: "Action"
    TRELLO_CARD_GET_LIST_BY_ID: "Action"
    TRELLO_CARD_GET_MEMBERS_VOTED_BY_ID_CARD: "Action"
    TRELLO_CARD_GET_STICKERS_BY_ID_CARD: "Action"
    TRELLO_CARD_GET_STICKER_BY_IDS: "Action"
    TRELLO_CARD_LIST_CARD_ACTIONS_BY_ID: "Action"
    TRELLO_CARD_LIST_MEMBERS_BY_ID_CARD: "Action"
    TRELLO_CARD_MARK_ASSOCIATED_NOTIFICATIONS_READ: "Action"
    TRELLO_CARD_REMOVE_ACTION_COMMENT_BY_ID_CARD_BY_ID_ACTION: "Action"
    TRELLO_CARD_REMOVE_BY_ID_CARD: "Action"
    TRELLO_CARD_REMOVE_CHECKLIST_CHECK_ITEM: "Action"
    TRELLO_CARD_REMOVE_LABELS_BY_ID_CARD_BY_COLOR: "Action"
    TRELLO_CARD_REMOVE_LABEL_BY_ID_CARD_BY_ID_LABEL: "Action"
    TRELLO_CARD_REMOVE_MEMBERS_VOTED_BY_ID_CARD_BY_ID_MEMBER: "Action"
    TRELLO_CARD_REMOVE_MEMBER_BY_ID_MEMBER: "Action"
    TRELLO_CARD_REMOVE_STICKER_BY_IDS: "Action"
    TRELLO_CARD_UPDATE_ACTION_COMMENT_BY_ID_CARD_BY_ID_ACTION: "Action"
    TRELLO_CARD_UPDATE_ATTACHMENT_COVER_BY_ID_CARD: "Action"
    TRELLO_CARD_UPDATE_BOARD_BY_ID_CARD: "Action"
    TRELLO_CARD_UPDATE_BY_ID_CARD: "Action"
    TRELLO_CARD_UPDATE_CHECKLIST_CHECK_ITEM: "Action"
    TRELLO_CARD_UPDATE_CHECKLIST_CHECK_ITEM_NAME_BY_ID: "Action"
    TRELLO_CARD_UPDATE_CHECKLIST_CHECK_ITEM_STATE: "Action"
    TRELLO_CARD_UPDATE_CHECK_I_TEMPOS_BY_ID: "Action"
    TRELLO_CARD_UPDATE_CLOSED_BY_ID: "Action"
    TRELLO_CARD_UPDATE_DESCRIPTION_BY_ID_CARD: "Action"
    TRELLO_CARD_UPDATE_DUE_BY_ID: "Action"
    TRELLO_CARD_UPDATE_ID_LIST_BY_ID_CARD: "Action"
    TRELLO_CARD_UPDATE_ID_MEMBERS: "Action"
    TRELLO_CARD_UPDATE_LABELS: "Action"
    TRELLO_CARD_UPDATE_NAME_BY_ID: "Action"
    TRELLO_CARD_UPDATE_POS_BY_ID_CARD: "Action"
    TRELLO_CARD_UPDATE_STICKERS_BY_ID_CARD_BY_ID_STICKER: "Action"
    TRELLO_CARD_UPDATE_SUBSCRIBED_BY_ID_CARD: "Action"
    TRELLO_CHECKLIST_ADD_CHECK_ITEMS_BY_ID_CHECKLIST: "Action"
    TRELLO_CHECKLIST_CREATE: "Action"
    TRELLO_CHECKLIST_GET_BOARD_BY_ID_CHECKLIST: "Action"
    TRELLO_CHECKLIST_GET_BOARD_BY_ID_CHECKLIST_BY_FIELD: "Action"
    TRELLO_CHECKLIST_GET_BY_ID: "Action"
    TRELLO_CHECKLIST_GET_BY_ID_FIELD: "Action"
    TRELLO_CHECKLIST_GET_CARDS_BY_FILTER: "Action"
    TRELLO_CHECKLIST_GET_CHECK_ITEMS_BY_ID_CHECKLIST: "Action"
    TRELLO_CHECKLIST_GET_CHECK_ITEMS_BY_ID_CHECKLIST_BY_ID_CHECK_ITEM: "Action"
    TRELLO_CHECKLIST_LIST_CARDS_BY_ID_CHECKLIST: "Action"
    TRELLO_CHECKLIST_REMOVE_BY_ID: "Action"
    TRELLO_CHECKLIST_REMOVE_BY_ID_CHECK_ITEM: "Action"
    TRELLO_CHECKLIST_UPDATE_BY_ID_CHECKLIST: "Action"
    TRELLO_CHECKLIST_UPDATE_ID_CARD_BY_ID_CHECKLIST: "Action"
    TRELLO_CHECKLIST_UPDATE_NAME_BY_ID_CHECKLIST: "Action"
    TRELLO_CHECKLIST_UPDATE_POS_BY_ID_CHECKLIST: "Action"
    TRELLO_LABEL_CREATE_LABELS: "Action"
    TRELLO_LABEL_GET_BOARD_BY_ID_LABEL: "Action"
    TRELLO_LABEL_GET_BOARD_BY_ID_LABEL_BY_FIELD: "Action"
    TRELLO_LABEL_GET_BY_ID_LABEL: "Action"
    TRELLO_LABEL_REMOVE_BY_ID_LABEL: "Action"
    TRELLO_LABEL_UPDATE_BY_ID_LABEL: "Action"
    TRELLO_LABEL_UPDATE_COLOR_BY_ID_LABEL: "Action"
    TRELLO_LABEL_UPDATE_NAME_BY_ID_LABEL: "Action"
    TRELLO_LIST_ARCHIVE_ALL_CARDS_BY_ID_LIST: "Action"
    TRELLO_LIST_CREATE_CARDS_BY_ID_LIST: "Action"
    TRELLO_LIST_CREATE_LIST: "Action"
    TRELLO_LIST_GET_ACTIONS_BY_ID_LIST: "Action"
    TRELLO_LIST_GET_BOARD_BY_ID_LIST_BY_FIELD: "Action"
    TRELLO_LIST_GET_BY_ID_LIST: "Action"
    TRELLO_LIST_GET_BY_ID_LIST_BY_FIELD: "Action"
    TRELLO_LIST_GET_CARDS_BY_FILTER: "Action"
    TRELLO_LIST_GET_CARDS_BY_ID_LIST: "Action"
    TRELLO_LIST_ID_BOARD_GET: "Action"
    TRELLO_LIST_MOVE_ALL_CARDS_BY_ID_LIST: "Action"
    TRELLO_LIST_UPDATE_BY_ID_LIST: "Action"
    TRELLO_LIST_UPDATE_CLOSED_BY_ID_LIST: "Action"
    TRELLO_LIST_UPDATE_ID_BOARD_BY_ID_LIST: "Action"
    TRELLO_LIST_UPDATE_NAME_BY_ID_LIST: "Action"
    TRELLO_LIST_UPDATE_POS_BY_ID_LIST: "Action"
    TRELLO_LIST_UPDATE_SUBSCRIBED_BY_ID_LIST: "Action"
    TRELLO_MEMBER_ADD_BOARD_BACKGROUNDS: "Action"
    TRELLO_MEMBER_ADD_BOARD_STARS_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_ADD_CUSTOM_BOARD_BACKGROUNDS: "Action"
    TRELLO_MEMBER_ADD_CUSTOM_E_MOJI_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_ADD_CUSTOM_STICKERS: "Action"
    TRELLO_MEMBER_ADD_ONE_TIME_MESSAGES_DISMISSED_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_CREATE_SAVED_SEARCH: "Action"
    TRELLO_MEMBER_DELETE_BOARD_BACKGROUND: "Action"
    TRELLO_MEMBER_GET_BOARDS: "Action"
    TRELLO_MEMBER_GET_BOARDS_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_GET_BOARDS_INVITED_BY_ID_MEMBER_BY_FIELD: "Action"
    TRELLO_MEMBER_GET_BOARD_BACKGROUNDS_BY_ID: "Action"
    TRELLO_MEMBER_GET_BOARD_BACKGROUND_BY_IDS: "Action"
    TRELLO_MEMBER_GET_BOARD_STARS_BY_ID: "Action"
    TRELLO_MEMBER_GET_BOARD_STAR_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_GET_BY_FIELD: "Action"
    TRELLO_MEMBER_GET_BY_ID: "Action"
    TRELLO_MEMBER_GET_CARDS_BY_FILTER: "Action"
    TRELLO_MEMBER_GET_CARDS_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_GET_CUSTOM_BOARD_BACKGROUNDS_BY_ID: "Action"
    TRELLO_MEMBER_GET_CUSTOM_BOARD_BACKGROUND_BY_IDS: "Action"
    TRELLO_MEMBER_GET_CUSTOM_E_MOJI_BY_IDS: "Action"
    TRELLO_MEMBER_GET_CUSTOM_STICKERS_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_GET_CUSTOM_STICKER_BY_IDS: "Action"
    TRELLO_MEMBER_GET_DELTAS_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_GET_INVITED_BOARDS: "Action"
    TRELLO_MEMBER_GET_NOTIFICATIONS_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_GET_NOTIFICATIONS_BY_ID_MEMBER_BY_FILTER: "Action"
    TRELLO_MEMBER_GET_ORGANIZATIONS: "Action"
    TRELLO_MEMBER_GET_SAVED_SEARCHES_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_GET_SAVED_SEARCH_BY_IDS: "Action"
    TRELLO_MEMBER_GET_TOKENS_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_LIST_ACTIONS_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_LIST_CUSTOM_E_MOJI_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_LIST_INVITED_ORGANIZATIONS_BY_ID: "Action"
    TRELLO_MEMBER_LIST_ORGANIZATIONS_BY_ID: "Action"
    TRELLO_MEMBER_LIST_ORGANIZATIONS_INVITED_BY_ID_MEMBER_BY_FIELD: "Action"
    TRELLO_MEMBER_REMOVE_BOARD_STAR_BY_ID_MEMBER_BY_ID_BOARD_STAR: "Action"
    TRELLO_MEMBER_REMOVE_CUSTOM_BOARD_BACKGROUND_BY_ID: "Action"
    TRELLO_MEMBER_REMOVE_CUSTOM_STICKER_BY_IDS: "Action"
    TRELLO_MEMBER_REMOVE_SAVED_SEARCH_BY_IDS: "Action"
    TRELLO_MEMBER_UPDATE_AVATAR_SOURCE: "Action"
    TRELLO_MEMBER_UPDATE_BIO_BY_ID: "Action"
    TRELLO_MEMBER_UPDATE_BOARD_BACKGROUNDS_BY_ID: "Action"
    TRELLO_MEMBER_UPDATE_BOARD_STAR: "Action"
    TRELLO_MEMBER_UPDATE_BOARD_STARS_ID_BOARD: "Action"
    TRELLO_MEMBER_UPDATE_BOARD_STAR_POS_BY_ID_MEMBER_BY_ID_BOARD_STAR: "Action"
    TRELLO_MEMBER_UPDATE_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_UPDATE_CUSTOM_BOARD_BACKGROUNDS: "Action"
    TRELLO_MEMBER_UPDATE_FULL_NAME: "Action"
    TRELLO_MEMBER_UPDATE_INITIALS_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_UPDATE_PREF_S_COLORBLIND_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_UPDATE_PREF_S_LOCALE_BY_ID_MEMBER: "Action"
    TRELLO_MEMBER_UPDATE_PREF_S_MINUTES_BETWEEN_SUMMARIES_BY_ID: "Action"
    TRELLO_MEMBER_UPDATE_SAVED_SEARCHES_BY_ID_MEMBER_BY_ID_SAVED_SEARCH: "Action"
    TRELLO_MEMBER_UPDATE_SAVED_SEARCHES_NAME_BY_ID_MEMBER_BY_ID_SAVED_SEARCH: "Action"
    TRELLO_MEMBER_UPDATE_SAVED_SEARCHES_POS_BY_ID_MEMBER_BY_ID_SAVED_SEARCH: "Action"
    TRELLO_MEMBER_UPDATE_SAVED_SEARCH_QUERY_BY_ID_MEMBER_BY_ID_SAVED_SEARCH: "Action"
    TRELLO_MEMBER_UPDATE_USERNAME_BY_ID: "Action"
    TRELLO_MEMBER_UPLOAD_AVATAR_BY_ID: "Action"
    TRELLO_NOTIFICATION_GET_BOARD_BY_FIELD: "Action"
    TRELLO_NOTIFICATION_GET_BOARD_BY_ID: "Action"
    TRELLO_NOTIFICATION_GET_BY_ID: "Action"
    TRELLO_NOTIFICATION_GET_BY_ID_FIELD: "Action"
    TRELLO_NOTIFICATION_GET_CARD_BY_ID: "Action"
    TRELLO_NOTIFICATION_GET_CARD_BY_ID_NOTIFICATION_BY_FIELD: "Action"
    TRELLO_NOTIFICATION_GET_DISPLAY_BY_ID_NOTIFICATION: "Action"
    TRELLO_NOTIFICATION_GET_ENTITIES_BY_ID_NOTIFICATION: "Action"
    TRELLO_NOTIFICATION_GET_LIST_BY_ID_NOTIFICATION: "Action"
    TRELLO_NOTIFICATION_GET_LIST_BY_ID_NOTIFICATION_BY_FIELD: "Action"
    TRELLO_NOTIFICATION_GET_MEMBER_BY_ID_FIELD: "Action"
    TRELLO_NOTIFICATION_GET_MEMBER_BY_ID_NOTIFICATION_BY_FIELD: "Action"
    TRELLO_NOTIFICATION_GET_MEMBER_BY_NOTIFICATION: "Action"
    TRELLO_NOTIFICATION_GET_MEMBER_CREATOR_BY_ID: "Action"
    TRELLO_NOTIFICATION_GET_ORGANIZATION_BY_ID_NOTIFICATION: "Action"
    TRELLO_NOTIFICATION_GET_ORGANIZATION_BY_ID_NOTIFICATION_BY_FIELD: "Action"
    TRELLO_NOTIFICATION_MARK_ALL_AS_READ: "Action"
    TRELLO_NOTIFICATION_UPDATE_BY_ID_NOTIFICATION: "Action"
    TRELLO_NOTIFICATION_UPDATE_UNREAD_BY_ID_NOTIFICATION: "Action"
    TRELLO_ORGANIZATION_CREATE: "Action"
    TRELLO_ORGANIZATION_DELETE_PREF_S_ASSOCIATED_DOMAIN_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_GET_ACTIONS_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_GET_BOARDS_BY_ID_ORG_BY_FILTER: "Action"
    TRELLO_ORGANIZATION_GET_BOARDS_BY_OR_GID: "Action"
    TRELLO_ORGANIZATION_GET_BY_ID_AND_FIELD: "Action"
    TRELLO_ORGANIZATION_GET_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_GET_MEMBERSHIPS_BY_ID_ORG_BY_ID_MEMBERSHIP: "Action"
    TRELLO_ORGANIZATION_GET_MEMBERS_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_GET_MEMBERS_INVITED_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_GET_MEMBERS_INVITED_BY_ID_ORG_BY_FIELD: "Action"
    TRELLO_ORGANIZATION_GET_ORGANIZATION_DELTAS: "Action"
    TRELLO_ORGANIZATION_LIST_MEMBERSHIPS_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_LIST_MEMBERS_BY_ID_ORG_BY_FILTER: "Action"
    TRELLO_ORGANIZATION_LIST_MEMBERS_CARDS_BY_ID_ORG_BY_ID_MEMBER: "Action"
    TRELLO_ORGANIZATION_REMOVE_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_REMOVE_INVITE_RESTRICT_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_REMOVE_LOGO_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_REMOVE_MEMBER_ALL: "Action"
    TRELLO_ORGANIZATION_REMOVE_MEMBER_BY_ID_ORG_BY_ID_MEMBER: "Action"
    TRELLO_ORGANIZATION_UPDATE_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_DESCRIPTION_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_DISPLAY_NAME_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_MEMBERSHIP_BY_ID_ORG_BY_ID_MEMBERSHIP: "Action"
    TRELLO_ORGANIZATION_UPDATE_MEMBERS_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_MEMBERS_BY_ID_ORG_BY_ID_MEMBER: "Action"
    TRELLO_ORGANIZATION_UPDATE_MEMBERS_DEACTIVATED_BY_ID_ORG_BY_ID_MEMBER: "Action"
    TRELLO_ORGANIZATION_UPDATE_NAME_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_PREF_S_ASSOCIATED_DOMAIN_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_PREF_S_BOARD_VISIBILITY_RESTRICT_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_PREF_S_BOARD_VISIBILITY_RESTRICT_PUBLIC_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_PREF_S_EXTERNAL_MEMBERS_DISABLED_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_PREF_S_GOOGLE_APPS_VERSION_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_PREF_S_ORG_INVITE_RESTRICT_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_PREF_S_PERMISSION_LEVEL_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_PREF_S_VISIBILITY_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPDATE_WEBSITE_BY_ID_ORG: "Action"
    TRELLO_ORGANIZATION_UPLOAD_LOGO_BY_ID_ORG: "Action"
    TRELLO_SEARCH_FIND_MEMBERS: "Action"
    TRELLO_SEARCH_GET_RESULTS: "Action"
    TRELLO_SESSION_CREATE_AND_UPDATE: "Action"
    TRELLO_SESSION_GET_SOCKET_SESSIONS: "Action"
    TRELLO_SESSION_UPDATE_STATUS_BY_ID_SESSION: "Action"
    TRELLO_SESSION_UPDATE_STATUS_BY_ID_SESSION_2: "Action"
    TRELLO_TOKEN_DELETE_BY_TOKEN: "Action"
    TRELLO_TOKEN_GET_BY_TOKEN: "Action"
    TRELLO_TOKEN_GET_BY_TOKEN_BY_FIELD: "Action"
    TRELLO_TOKEN_GET_MEMBER_BY_FIELD: "Action"
    TRELLO_TOKEN_GET_MEMBER_BY_TOKEN: "Action"
    TRELLO_TOKEN_GET_WEB_HOOKS: "Action"
    TRELLO_TOKEN_GET_WEB_HOOK_BY_ID: "Action"
    TRELLO_TOKEN_REGISTER_WEB_HOOK: "Action"
    TRELLO_TOKEN_REMOVE_BY_TOKEN_BY_ID_WEB_HOOK: "Action"
    TRELLO_TOKEN_UPDATE_WEB_HOOKS_BY_TOKEN: "Action"
    TRELLO_TYPE_GET_BY_ID: "Action"
    TRELLO_WEB_HOOK_GET_BY_ID: "Action"
    TRELLO_WEB_HOOK_GET_BY_ID_FIELD: "Action"
    TRELLO_WEB_HOOK_REMOVE_BY_ID: "Action"
    TRELLO_WEB_HOOK_UPDATE: "Action"
    TRELLO_WEB_HOOK_UPDATE_ACTIVE_BY_ID: "Action"
    TRELLO_WEB_HOOK_UPDATE_BY_ID_WEB_HOOK: "Action"
    TRELLO_WEB_HOOK_UPDATE_CALLBACK_URL_BY_ID: "Action"
    TRELLO_WEB_HOOK_UPDATE_DESCRIPTION_BY_ID_WEB_HOOK: "Action"
    TRELLO_WEB_HOOK_UPDATE_MODEL_BY_ID: "Action"
    TWILIO_SEND_SMS: "Action"
    TWILIO_SEND_WHATS_APP_MESSAGE: "Action"
    TYPEFORM_GET_ABOUT_ME: "Action"
    WEATHERMAP_WEATHER: "Action"
    WEBTOOL_SCRAPE_WEBSITE_CONTENT: "Action"
    WEBTOOL_SCRAPE_WEBSITE_ELEMENT: "Action"
    WHATSAPP_APPLICATION_GET_SETTINGS: "Action"
    WHATSAPP_APPLICATION_LIST_MEDIA_PROVIDERS: "Action"
    WHATSAPP_APPLICATION_REMOVE_PROVIDER: "Action"
    WHATSAPP_APPLICATION_RESET_SETTINGS: "Action"
    WHATSAPP_APPLICATION_SET_SHARDS: "Action"
    WHATSAPP_APPLICATION_UPDATE_SETTINGS: "Action"
    WHATSAPP_BACKUP_RESTORE_SETTINGS_POST: "Action"
    WHATSAPP_BACKUP_RESTORE_SETTINGS_POST_2: "Action"
    WHATSAPP_BUSINESS_PROFILE_GET: "Action"
    WHATSAPP_BUSINESS_PROFILE_UPDATE: "Action"
    WHATSAPP_CERTIFICATES_DELETE_WEB_HOOK_CA: "Action"
    WHATSAPP_CERTIFICATES_DOWNLOAD_CA_CERTIFICATE: "Action"
    WHATSAPP_CERTIFICATES_DOWNLOAD_WEB_HOOK_CA_CERTIFICATE: "Action"
    WHATSAPP_CERTIFICATES_UPLOAD_EXTERNAL_CERTIFICATE: "Action"
    WHATSAPP_CERTIFICATES_UPLOAD_WEB_HOOK_CA_CERTIFICATE: "Action"
    WHATSAPP_CONTACTS_CREATE_CONTACT: "Action"
    WHATSAPP_GROUPS_CREATE_GROUP: "Action"
    WHATSAPP_GROUPS_DELETE_GROUP_ICON: "Action"
    WHATSAPP_GROUPS_DELETE_INVITE: "Action"
    WHATSAPP_GROUPS_DEMOTE_ADMIN: "Action"
    WHATSAPP_GROUPS_GET_ALL: "Action"
    WHATSAPP_GROUPS_GET_ICON_BINARY: "Action"
    WHATSAPP_GROUPS_GET_INFO: "Action"
    WHATSAPP_GROUPS_GET_INVITE_DETAILS: "Action"
    WHATSAPP_GROUPS_LEAVE_GROUP: "Action"
    WHATSAPP_GROUPS_PROMOTE_TO_ADMIN: "Action"
    WHATSAPP_GROUPS_REMOVE_PARTICIPANT: "Action"
    WHATSAPP_GROUPS_SET_GROUP_ICON: "Action"
    WHATSAPP_GROUPS_UPDATE_INFO: "Action"
    WHATSAPP_HEALTH_CHECK_STATUS: "Action"
    WHATSAPP_HEALTH_GET_APP_STATS: "Action"
    WHATSAPP_HEALTH_GET_DB_STATS: "Action"
    WHATSAPP_HEALTH_GET_METRICS_DATA: "Action"
    WHATSAPP_HEALTH_GET_SUPPORT_INFO: "Action"
    WHATSAPP_MEDIA_DOWNLOAD: "Action"
    WHATSAPP_MEDIA_REMOVE_MEDIA: "Action"
    WHATSAPP_MEDIA_UPLOAD_MEDIA: "Action"
    WHATSAPP_MESSAGES_MARK_AS_READ: "Action"
    WHATSAPP_MESSAGES_SEND_MESSAGE: "Action"
    WHATSAPP_PROFILE_GET_ABOUT: "Action"
    WHATSAPP_PROFILE_GET_PHOTO: "Action"
    WHATSAPP_PROFILE_REMOVE_PHOTO: "Action"
    WHATSAPP_PROFILE_UPDATE_ABOUT: "Action"
    WHATSAPP_PROFILE_UPDATE_PHOTO: "Action"
    WHATSAPP_REGISTRATION_REQUEST_CODE: "Action"
    WHATSAPP_REGISTRATION_VERIFY_ACCOUNT: "Action"
    WHATSAPP_TWO_STEP_VERIFICATION_DISABLE: "Action"
    WHATSAPP_TWO_STEP_VERIFICATION_ENABLE_ACCOUNT: "Action"
    WHATSAPP_USERS_CREATE_USER: "Action"
    WHATSAPP_USERS_GET_BY_USERNAME: "Action"
    WHATSAPP_USERS_PERFORM_LOGIN: "Action"
    WHATSAPP_USERS_PERFORM_LOG_OUT: "Action"
    WHATSAPP_USERS_REMOVE_USER: "Action"
    WHATSAPP_USERS_UPDATE_USER: "Action"
    WORKABLE_GET_ACCOUNT_DEPARTMENT_ACTION: "Action"
    WORKABLE_GET_EXTERNAL_RECRUITER_LIST_ACTION: "Action"
    WORKABLE_GET_LEGAL_ENTITIES_ACTION: "Action"
    WORKABLE_GET_MEMBERS_LIST_ACTION: "Action"
    WORKABLE_GET_REQUIREMENT_PIPELINE_STAGE_ACTION: "Action"
    WORKABLE_GET_SPECIFIC_ACCOUNT_ACTION: "Action"
    WORKABLE_WORKABLE_ACCOUNT_ACCESS_ACTION: "Action"
    YOUSEARCH_SEARCH: "Action"
    YOUTUBE_LIST_CAPTION_TRACK: "Action"
    YOUTUBE_LIST_CHANNEL_VIDEOS: "Action"
    YOUTUBE_LIST_USER_PLAYLISTS: "Action"
    YOUTUBE_LIST_USER_SUBSCRIPTIONS: "Action"
    YOUTUBE_LOAD_CAPTIONS: "Action"
    YOUTUBE_SEARCH_YOU_TUBE: "Action"
    YOUTUBE_SUBSCRIBE_CHANNEL: "Action"
    YOUTUBE_UPDATE_THUMBNAIL: "Action"
    YOUTUBE_UPDATE_VIDEO: "Action"
    YOUTUBE_VIDEO_DETAILS: "Action"
    ZENDESK_COUNT_ZENDESK_ORGANIZATIONS: "Action"
    ZENDESK_CREATE_ZENDESK_ORGANIZATION: "Action"
    ZENDESK_CREATE_ZENDESK_TICKET: "Action"
    ZENDESK_DELETE_ZENDESK_ORGANIZATION: "Action"
    ZENDESK_DELETE_ZENDESK_TICKET: "Action"
    ZENDESK_GET_ABOUT_ME: "Action"
    ZENDESK_GET_ALL_ZENDESK_ORGANIZATIONS: "Action"
    ZENDESK_GET_ZENDESK_ORGANIZATION: "Action"
    ZENDESK_UPDATE_ZENDESK_ORGANIZATION: "Action"
    ZEPTOOL_ADD_MEMORY: "Action"
    ZEPTOOL_CREATE_SESSION: "Action"
    ZEPTOOL_GET_MEMORY: "Action"
    ZEPTOOL_SEARCH_MEMORY: "Action"
    ZOOM_ANALYTICS_DETAILS: "Action"
    ZOOM_ANALYTICS_SUMMARY: "Action"
    ZOOM_ARCHIVING_GET_STATISTICS: "Action"
    ZOOM_ARCHIVING_MEETING_FILES_DELETE: "Action"
    ZOOM_ARCHIVING_MEETING_FILES_LIST: "Action"
    ZOOM_ARCHIVING_MEETING_FILES_LIST_2: "Action"
    ZOOM_ARCHIVING_UPDATE_AUTO_DELETE_STATUS: "Action"
    ZOOM_CLOUD_RECORDING_CREATE_REGISTRANT: "Action"
    ZOOM_CLOUD_RECORDING_DELETE_MEETING_RECORDINGS: "Action"
    ZOOM_CLOUD_RECORDING_DELETE_RECORDING: "Action"
    ZOOM_CLOUD_RECORDING_GET_MEETING_RECORDINGS: "Action"
    ZOOM_CLOUD_RECORDING_GET_SETTINGS: "Action"
    ZOOM_CLOUD_RECORDING_LIST_RECORDINGS: "Action"
    ZOOM_CLOUD_RECORDING_LIST_REGISTRANTS: "Action"
    ZOOM_CLOUD_RECORDING_LIST_REGISTRATION_QUESTIONS: "Action"
    ZOOM_CLOUD_RECORDING_RECOVER_RECORDING_STATUS: "Action"
    ZOOM_CLOUD_RECORDING_RECOVER_STATUS: "Action"
    ZOOM_CLOUD_RECORDING_UPDATE_REGISTRANT_STATUS: "Action"
    ZOOM_CLOUD_RECORDING_UPDATE_REGISTRATION_QUESTIONS: "Action"
    ZOOM_CLOUD_RECORDING_UPDATE_SETTINGS: "Action"
    ZOOM_DEVICES_ASSIGN_DEVICE_ZP_A_ASSIGNMENT: "Action"
    ZOOM_DEVICES_CHANGE_DEVICE_ASSOCIATION: "Action"
    ZOOM_DEVICES_CREATE_NEW_DEVICE: "Action"
    ZOOM_DEVICES_GET_DETAIL: "Action"
    ZOOM_DEVICES_GET_ZP_A_VERSION_INFO: "Action"
    ZOOM_DEVICES_LIST: "Action"
    ZOOM_DEVICES_LIST_ZD_M_GROUP_INFO: "Action"
    ZOOM_DEVICES_REMOVE_DEVICE_ZM_D: "Action"
    ZOOM_DEVICES_REMOVE_ZP_A_DEVICE_BY_VENDOR_AND_MAC_ADDRESS: "Action"
    ZOOM_DEVICES_UPDATE_DEVICE_NAME: "Action"
    ZOOM_DEVICES_UPGRADE_ZP_A_OS_APP: "Action"
    ZOOM_H_323_DEVICES_CREATE_DEVICE: "Action"
    ZOOM_H_323_DEVICES_DELETE_DEVICE: "Action"
    ZOOM_H_323_DEVICES_LIST_DEVICES: "Action"
    ZOOM_H_323_DEVICES_UPDATE_DEVICE_INFO: "Action"
    ZOOM_MEETINGS_ADD_REGISTRANT: "Action"
    ZOOM_MEETINGS_BATCH_REGISTRANTS_CREATE: "Action"
    ZOOM_MEETINGS_CONTROL_IN_MEETING_FEATURES: "Action"
    ZOOM_MEETINGS_CREATE_BATCH_POLLS: "Action"
    ZOOM_MEETINGS_CREATE_INVITE_LINKS: "Action"
    ZOOM_MEETINGS_CREATE_MEETING: "Action"
    ZOOM_MEETINGS_CREATE_POLL: "Action"
    ZOOM_MEETINGS_CREATE_TEMPLATE_FROM_MEETING: "Action"
    ZOOM_MEETINGS_DELETE_MEETING_CHAT_MESSAGE: "Action"
    ZOOM_MEETINGS_DELETE_MEETING_SURVEY: "Action"
    ZOOM_MEETINGS_DELETE_REGISTRANT: "Action"
    ZOOM_MEETINGS_GETS_IP_URI_WITH_PASS_CODE: "Action"
    ZOOM_MEETINGS_GET_DETAILS: "Action"
    ZOOM_MEETINGS_GET_DETAILS_2: "Action"
    ZOOM_MEETINGS_GET_INVITATION_NOTE: "Action"
    ZOOM_MEETINGS_GET_JOIN_TOKEN: "Action"
    ZOOM_MEETINGS_GET_JOIN_TOKEN_LOCAL_RECORDING: "Action"
    ZOOM_MEETINGS_GET_LIVE_STREAM_DETAILS: "Action"
    ZOOM_MEETINGS_GET_MEETING_ARCHIVE_TOKEN_FOR_LOCAL_ARCHIVING: "Action"
    ZOOM_MEETINGS_GET_MEETING_SUMMARY: "Action"
    ZOOM_MEETINGS_GET_MEETING_SURVEY: "Action"
    ZOOM_MEETINGS_GET_MEETING_TOKEN: "Action"
    ZOOM_MEETINGS_GET_PAST_MEETING_PARTICIPANTS: "Action"
    ZOOM_MEETINGS_GET_POLL: "Action"
    ZOOM_MEETINGS_GET_REGISTRANT_DETAILS: "Action"
    ZOOM_MEETINGS_LIST_HOST_SCHEDULED: "Action"
    ZOOM_MEETINGS_LIST_MEETING_POLLS: "Action"
    ZOOM_MEETINGS_LIST_MEETING_SUMMARIES: "Action"
    ZOOM_MEETINGS_LIST_MEETING_TEMPLATES: "Action"
    ZOOM_MEETINGS_LIST_PAST_MEETING_INSTANCES: "Action"
    ZOOM_MEETINGS_LIST_PAST_MEETING_POLLS: "Action"
    ZOOM_MEETINGS_LIST_PAST_MEETING_QA: "Action"
    ZOOM_MEETINGS_LIST_REGISTRANTS: "Action"
    ZOOM_MEETINGS_LIST_REGISTRATION_QUESTIONS: "Action"
    ZOOM_MEETINGS_LIST_UPCOMING_MEETINGS: "Action"
    ZOOM_MEETINGS_LIVE_STREAM_STATUS_UPDATE: "Action"
    ZOOM_MEETINGS_POLL_DELETE: "Action"
    ZOOM_MEETINGS_REMOVE_MEETING: "Action"
    ZOOM_MEETINGS_UPDATE_DETAILS: "Action"
    ZOOM_MEETINGS_UPDATE_LIVE_STREAM: "Action"
    ZOOM_MEETINGS_UPDATE_MEETING_POLL: "Action"
    ZOOM_MEETINGS_UPDATE_MEETING_STATUS: "Action"
    ZOOM_MEETINGS_UPDATE_MESSAGE: "Action"
    ZOOM_MEETINGS_UPDATE_REGISTRANT_STATUS: "Action"
    ZOOM_MEETINGS_UPDATE_REGISTRATION_QUESTIONS: "Action"
    ZOOM_MEETINGS_UPDATE_SURVEY: "Action"
    ZOOM_PAC_LIST_ACCOUNTS: "Action"
    ZOOM_REPORTS_GET_ACTIVE_INACTIVE_HOST_REPORTS: "Action"
    ZOOM_REPORTS_GET_BILLING_DEPARTMENT_REPORTS: "Action"
    ZOOM_REPORTS_GET_BILLING_INVOICES: "Action"
    ZOOM_REPORTS_GET_CLOUD_RECORDING_USAGE_REPORT: "Action"
    ZOOM_REPORTS_GET_DAILY_USAGE_REPORT: "Action"
    ZOOM_REPORTS_GET_MEETING_DETAIL_REPORTS: "Action"
    ZOOM_REPORTS_GET_MEETING_PARTICIPANT_REPORTS: "Action"
    ZOOM_REPORTS_GET_MEETING_POLL_REPORTS: "Action"
    ZOOM_REPORTS_GET_MEETING_QA_REPORT: "Action"
    ZOOM_REPORTS_GET_MEETING_REPORTS: "Action"
    ZOOM_REPORTS_GET_MEETING_SURVEY_REPORT: "Action"
    ZOOM_REPORTS_GET_OPERATION_LOGS_REPORT: "Action"
    ZOOM_REPORTS_GET_TELEPHONE_REPORTS: "Action"
    ZOOM_REPORTS_GET_WEB_IN_AR_DETAILS_REPORT: "Action"
    ZOOM_REPORTS_GET_WEB_IN_AR_POLL_REPORTS: "Action"
    ZOOM_REPORTS_GET_WEB_IN_AR_QA_REPORT: "Action"
    ZOOM_REPORTS_GET_WEB_IN_AR_SURVEY_REPORT: "Action"
    ZOOM_REPORTS_LIST_SIGN_IN_SIGN_OUT_ACTIVITIES: "Action"
    ZOOM_REPORTS_LIST_UPCOMING_EVENTS_REPORT: "Action"
    ZOOM_REPORTS_WEB_IN_AR_PARTICIPANTS_LIST: "Action"
    ZOOM_SIP_PHONE_DELETE_PHONE: "Action"
    ZOOM_SIP_PHONE_ENABLE_USERS_IP_PHONE: "Action"
    ZOOM_SIP_PHONE_LIST: "Action"
    ZOOM_SIP_PHONE_UPDATE_SPECIFIC_PHONE: "Action"
    ZOOM_TRACKING_FIELD_CREATE_FIELD: "Action"
    ZOOM_TRACKING_FIELD_DELETE_FIELD: "Action"
    ZOOM_TRACKING_FIELD_GET: "Action"
    ZOOM_TRACKING_FIELD_LIST: "Action"
    ZOOM_TRACKING_FIELD_UPDATE: "Action"
    ZOOM_TSP_ADD_USE_RTSP_ACCOUNT: "Action"
    ZOOM_TSP_DELETE_USE_RTSP_ACCOUNT: "Action"
    ZOOM_TSP_GET_ACCOUNT_INFO: "Action"
    ZOOM_TSP_GET_USE_RTSP_ACCOUNT: "Action"
    ZOOM_TSP_LIST_USE_RTSP_ACCOUNTS: "Action"
    ZOOM_TSP_SET_GLOBAL_DIAL_IN_URL: "Action"
    ZOOM_TSP_UPDATE_ACCOUNT_TSP_INFORMATION: "Action"
    ZOOM_TSP_UPDATE_USE_RTSP_ACCOUNT: "Action"
    ZOOM_WEB_IN_ARS_ADD_PANELISTS: "Action"
    ZOOM_WEB_IN_ARS_ADD_REGISTRANT: "Action"
    ZOOM_WEB_IN_ARS_CREATE_BATCH_REGISTRANTS: "Action"
    ZOOM_WEB_IN_ARS_CREATE_BRANDING_NAME_TAG: "Action"
    ZOOM_WEB_IN_ARS_CREATE_INVITE_LINKS: "Action"
    ZOOM_WEB_IN_ARS_CREATE_POLL: "Action"
    ZOOM_WEB_IN_ARS_CREATE_WEB_IN_AR: "Action"
    ZOOM_WEB_IN_ARS_CREATE_WEB_IN_AR_TEMPLATE: "Action"
    ZOOM_WEB_IN_ARS_DELETE_BRANDING_NAME_TAG: "Action"
    ZOOM_WEB_IN_ARS_DELETE_BRANDING_VIRTUAL_BACKGROUND: "Action"
    ZOOM_WEB_IN_ARS_DELETE_BRANDING_WALLPAPER: "Action"
    ZOOM_WEB_IN_ARS_DELETE_MESSAGE_BY_ID: "Action"
    ZOOM_WEB_IN_ARS_DELETE_POLL: "Action"
    ZOOM_WEB_IN_ARS_DELETE_REGISTRANT: "Action"
    ZOOM_WEB_IN_ARS_DELETE_SURVEY: "Action"
    ZOOM_WEB_IN_ARS_GETS_IP_URI_WITH_PASS_CODE: "Action"
    ZOOM_WEB_IN_ARS_GET_DETAILS: "Action"
    ZOOM_WEB_IN_ARS_GET_JOIN_TOKEN_LOCAL_RECORDING: "Action"
    ZOOM_WEB_IN_ARS_GET_LIVE_STREAM_DETAILS: "Action"
    ZOOM_WEB_IN_ARS_GET_MEETING_ARCHIVE_TOKEN_FOR_LOCAL_ARCHIVING: "Action"
    ZOOM_WEB_IN_ARS_GET_POLL_DETAILS: "Action"
    ZOOM_WEB_IN_ARS_GET_SESSION_BRANDING: "Action"
    ZOOM_WEB_IN_ARS_GET_SURVEY: "Action"
    ZOOM_WEB_IN_ARS_GET_WEB_IN_ART_OKEN: "Action"
    ZOOM_WEB_IN_ARS_JOIN_TOKEN_LIVE_STREAMING: "Action"
    ZOOM_WEB_IN_ARS_LIST_ABSENTEES: "Action"
    ZOOM_WEB_IN_ARS_LIST_PANELISTS: "Action"
    ZOOM_WEB_IN_ARS_LIST_PARTICIPANTS: "Action"
    ZOOM_WEB_IN_ARS_LIST_PAST_INSTANCES: "Action"
    ZOOM_WEB_IN_ARS_LIST_PAST_WEB_IN_AR_QA: "Action"
    ZOOM_WEB_IN_ARS_LIST_POLLS: "Action"
    ZOOM_WEB_IN_ARS_LIST_POLL_RESULTS: "Action"
    ZOOM_WEB_IN_ARS_LIST_REGISTRANTS: "Action"
    ZOOM_WEB_IN_ARS_LIST_REGISTRATION_QUESTIONS: "Action"
    ZOOM_WEB_IN_ARS_LIST_TRACKING_SOURCES: "Action"
    ZOOM_WEB_IN_ARS_LIST_WEB_IN_ARS: "Action"
    ZOOM_WEB_IN_ARS_LIST_WEB_IN_AR_TEMPLATES: "Action"
    ZOOM_WEB_IN_ARS_REGISTRANT_DETAILS: "Action"
    ZOOM_WEB_IN_ARS_REMOVE_PANELIST: "Action"
    ZOOM_WEB_IN_ARS_REMOVE_PANELISTS: "Action"
    ZOOM_WEB_IN_ARS_REMOVE_WEB_IN_AR: "Action"
    ZOOM_WEB_IN_ARS_SET_DEFAULT_BRANDING_VIRTUAL_BACKGROUND: "Action"
    ZOOM_WEB_IN_ARS_UPDATE_BRANDING_NAME_TAG: "Action"
    ZOOM_WEB_IN_ARS_UPDATE_LIVE_STREAM: "Action"
    ZOOM_WEB_IN_ARS_UPDATE_LIVE_STREAM_STATUS: "Action"
    ZOOM_WEB_IN_ARS_UPDATE_POLL: "Action"
    ZOOM_WEB_IN_ARS_UPDATE_REGISTRANT_STATUS: "Action"
    ZOOM_WEB_IN_ARS_UPDATE_REGISTRATION_QUESTIONS: "Action"
    ZOOM_WEB_IN_ARS_UPDATE_SCHEDULED_WEB_IN_AR: "Action"
    ZOOM_WEB_IN_ARS_UPDATE_STATUS: "Action"
    ZOOM_WEB_IN_ARS_UPDATE_SURVEY: "Action"
    ZOOM_WEB_IN_ARS_UPLOAD_BRANDING_VIRTUAL_BACKGROUND: "Action"
    ZOOM_WEB_IN_ARS_UPLOAD_BRANDING_WALLPAPER: "Action"

    def __init__(self, value: t.Union[str, te.Self, t.Type["SentinalObject"]]) -> None:
        """Create an Enum"""
        if hasattr(value, "sentinal"):
            raise ValueError()
        super().__init__(value=value)  # type: ignore

    @property
    def name(self) -> str:
        """Action name."""
        return self.load().name

    @property
    def app(self) -> str:
        """App name where the actions belongs to."""
        return self.load().app

    @property
    def tags(self) -> t.List[str]:
        """List of tags for action."""
        return self.load().tags

    @property
    def no_auth(self) -> bool:
        """If set `True` the action does not require authentication."""
        return self.load().no_auth

    @property
    def is_local(self) -> bool:
        """If set `True` the `app` is a local app."""
        return self.load().is_local

    @property
    def is_runtime(self) -> bool:
        """If set `True` the `app` is a runtime app."""
        return self.load().is_runtime

    @property
    def shell(self) -> bool:
        """If set `True` the tool will be executed using a shell."""
        return self.load().shell

    @classmethod
    def add(cls, name: str, data: ActionData) -> None:
        """Add a new action."""
        add_runtime_action(name=name, data=data)
